@@ -4,9 +4,11 @@
 #  -----------------------------------------------------------------------------------------
 from __future__ import annotations
 import os
+import re
 from typing import TYPE_CHECKING, Callable, Any, cast, Literal, Generator
 from warnings import warn
 import types
+import inspect
 
 import ibm_watsonx_ai._wrappers.requests as requests
 from ibm_watsonx_ai.metanames import AIServiceMetaNames
@@ -47,7 +49,7 @@ class AIServices(WMLResource):
         :type ai_service: str | Callable
 
         :param meta_props: metadata for storing an AI service asset. To see available meta names
-            use ``client._ai_services.ConfigurationMetaNames.show()``
+            use ``client._ai_services.ConfigurationMetaNames.show()`` or direct to :class:`~metanames.AIServiceMetaNames` class.
         :type meta_props: dict
 
         :return: metadata of the stored AI service
@@ -93,8 +95,8 @@ class AIServices(WMLResource):
                 client._ai_services.ConfigurationMetaNames.NAME: "AI service example",
                 client._ai_services.ConfigurationMetaNames.DESCRIPTION: "This is AI service function",
                 client._ai_services.ConfigurationMetaNames.SOFTWARE_SPEC_ID: "53dc4cf1-252f-424b-b52d-5cdd9814987f",
-                client._ai_services.ConfigurationMetaNames.REQUEST_DOCUMENTATION: documentation_request,
-                client._ai_services.ConfigurationMetaNames.RESPONSE_DOCUMENTATION: documentation_response,
+                client._ai_services.ConfigurationMetaNames.DOCUMENTATION_REQUEST: documentation_request,
+                client._ai_services.ConfigurationMetaNames.DOCUMENTATION_RESPONSE: documentation_response,
                 }
 
             def deployable_ai_service(context, params={"k1":"v1"}, **kwargs):
@@ -159,7 +161,7 @@ class AIServices(WMLResource):
                 ai_service_metadata["project_id"] = self._client.default_project_id
 
             response_post = requests.post(
-                self._client.service_instance._href_definitions.get_ai_services_href(),
+                self._client._href_definitions.get_ai_services_href(),
                 json=ai_service_metadata,
                 params=self._client._params(skip_for_create=True),
                 headers=self._client._get_headers(),
@@ -173,7 +175,7 @@ class AIServices(WMLResource):
 
             # uploading code
             ai_service_content_url = (
-                self._client.service_instance._href_definitions.get_ai_service_href(
+                self._client._href_definitions.get_ai_service_href(
                     details["metadata"]["id"]
                 )
                 + "/code"
@@ -249,9 +251,7 @@ class AIServices(WMLResource):
             details, changes, with_validation=True
         )
 
-        url = self._client.service_instance._href_definitions.get_ai_service_href(
-            ai_service_id
-        )
+        url = self._client._href_definitions.get_ai_service_href(ai_service_id)
         response = requests.patch(
             url,
             json=patch_payload,
@@ -283,9 +283,7 @@ class AIServices(WMLResource):
         )
         try:
             function_content_url = (
-                self._client.service_instance._href_definitions.get_ai_service_href(
-                    ai_service_id
-                )
+                self._client._href_definitions.get_ai_service_href(ai_service_id)
                 + "/code"
             )
 
@@ -349,10 +347,7 @@ class AIServices(WMLResource):
         AIServices._validate_type(filename, "filename", str, True)
 
         artifact_content_url = (
-            self._client.service_instance._href_definitions.get_ai_service_href(
-                ai_service_id
-            )
-            + "/code"
+            self._client._href_definitions.get_ai_service_href(ai_service_id) + "/code"
         )
 
         try:
@@ -402,12 +397,15 @@ class AIServices(WMLResource):
                 e,
             )
 
-    def delete(self, ai_service_id: str) -> Literal["SUCCESS"]:
+    def delete(self, ai_service_id: str, force: bool = False) -> Literal["SUCCESS"]:
         """Delete a stored AI service asset.
 
         :param ai_service_id: stored AI service ID
         :type ai_service_id: str
 
+        :param force: if True, the delete operation will proceed even when the AI service deployment exists, defaults to False
+        :type force: bool, optional
+
         :return: status "SUCCESS" if deletion is successful
         :rtype: Literal["SUCCESS"]
 
@@ -417,30 +415,28 @@ class AIServices(WMLResource):
 
             client._ai_services.delete(ai_service_id)
         """
-
         self._client._check_if_either_is_set()
         AIServices._validate_type(ai_service_id, "ai_service_id", str, True)
 
-        if self._if_deployment_exist_for_asset(ai_service_id):
+        if not force and self._if_deployment_exist_for_asset(ai_service_id):
             raise WMLClientError(
                 "Cannot delete AI service that has existing deployments. Please delete all associated deployments and try again"
             )
 
-        ai_service_endpoint = (
-            self._client.service_instance._href_definitions.get_ai_service_href(
-                ai_service_id
-            )
+        ai_service_endpoint = self._client._href_definitions.get_ai_service_href(
+            ai_service_id
         )
+
         self._logger.debug(
-            "Deletion artifact AI service endpoint: {}".format(ai_service_endpoint)
+            "Deletion artifact AI service endpoint: %s" % ai_service_endpoint
         )
-        response_delete = requests.delete(
+        response = requests.delete(
             ai_service_endpoint,
             params=self._client._params(),
             headers=self._client._get_headers(),
         )
 
-        return self._handle_response(204, "AI service deletion", response_delete, False)
+        return self._handle_response(204, "AI service deletion", response, False)
 
     def get_details(
         self,
@@ -508,7 +504,7 @@ class AIServices(WMLResource):
             warn(spec_state_setting_warning)
 
         self._client._check_if_either_is_set()
-        url = self._client.service_instance._href_definitions.get_ai_services_href()
+        url = self._client._href_definitions.get_ai_services_href()
 
         if ai_service_id is None:
             if spec_state:
@@ -617,7 +613,7 @@ class AIServices(WMLResource):
 
         AIServices._validate_type(ai_service_id, "ai_service_id", str, True)
 
-        url = self._client.service_instance._href_definitions.get_ai_services_href()
+        url = self._client._href_definitions.get_ai_services_href()
         return self._create_revision_artifact(url, ai_service_id, "ai_service_id")
 
     def get_revision_details(
@@ -648,9 +644,7 @@ class AIServices(WMLResource):
         AIServices._validate_type(ai_service_id, "ai_service_id", str, True)
         AIServices._validate_type(rev_id, "rev_id", str, True)
 
-        url = self._client.service_instance._href_definitions.get_ai_service_href(
-            ai_service_id
-        )
+        url = self._client._href_definitions.get_ai_service_href(ai_service_id)
         return self._get_with_or_without_limit(
             url,
             limit=None,
@@ -686,9 +680,7 @@ class AIServices(WMLResource):
 
         AIServices._validate_type(ai_service_id, "ai_service_id", str, True)
 
-        url = self._client.service_instance._href_definitions.get_ai_service_href(
-            ai_service_id
-        )
+        url = self._client._href_definitions.get_ai_service_href(ai_service_id)
 
         ai_services_resources = self._get_artifact_details(
             url + "/revisions",
@@ -785,16 +777,14 @@ class AIServices(WMLResource):
     def _populate_default_params(
         ai_service_function: Callable, _validate_values: bool = True
     ) -> str:
-        import inspect
-        import re
 
         # remove indention
         code_lines = inspect.getsource(ai_service_function).split("\n")
-        r = re.compile(r"^ *")
-        m = r.search(code_lines[0])
-        intend = m.group(0)  # type: ignore[union-attr]
-
-        code = "\n".join([line.replace(intend, "", 1) for line in code_lines])
+        indent = re.match(r"^ *", code_lines[0]).group(0)
+        code = "\n".join(
+            line[len(indent) :] if line.startswith(indent) else line
+            for line in code_lines
+        )
 
         signature = inspect.signature(ai_service_function)
 
@@ -810,35 +800,44 @@ class AIServices(WMLResource):
                     )
 
         args_list = list(signature.parameters.keys())
+        patterns = []
+        type_hint_pattern = r"(?:[:]\s*[\S\[\]\s]+\s*?)?"
+        # all but last -> expect a trailing comma in the signature
+        for arg in args_list[:-1]:
+            p = signature.parameters[arg]
+            if p.default is inspect.Parameter.empty:
+                patterns.append(rf"\s*{arg}{type_hint_pattern}\s*")
+            else:
+                patterns.append(rf"\s*{arg}\s*{type_hint_pattern}\s*=\s*([\s\S]+?)\s*")
 
-        patterns_list = [
-            (
-                rf"\s*{arg}\s*=\s*(.+)\s*"
-                if signature.parameters[arg].default is not inspect.Parameter.empty
-                else rf"\s*{arg}\s*"
-            )
-            for arg in args_list[:-1]
-        ]
-
-        # to match `**kwargs`
-        if signature.parameters[args_list[-1]].default is inspect.Parameter.empty:
-            patterns_list.append(rf"\s*(?:\*\*)?\s*{args_list[-1]}\s*")
+        # last arg -> allow optional comma before the ')'
+        last = args_list[-1]
+        p_last = signature.parameters[last]
+        if p_last.default is inspect.Parameter.empty:
+            patterns.append(rf"\s*(?:\*\*)?{last}\s*")
         else:
-            patterns_list.append(rf"\s*{args_list[-1]}\s*=\s*(.+)\s*")
+            patterns.append(rf"\s*{last}\s*{type_hint_pattern}\s*=\s*([\s\S]+?)\s*")
 
-        args_pattern = ",".join(patterns_list)
+        args_pattern = ",".join(patterns)
+        regex = re.compile(
+            rf"^def {ai_service_function.__name__}\s*\(\s*{args_pattern}\s*\)\s*:",
+            re.DOTALL | re.MULTILINE,
+        )
+        m = regex.search(code)
 
-        pattern = rf"^def {ai_service_function.__name__}\s*\({args_pattern}\)\s*:"
-        res = re.match(pattern, code)  # type: ignore[call-overload]
+        if m is None:
+            raise ValueError(
+                "Unable to process AI service function content. "
+                "Please make sure the function object has simply Python signature."
+            )
 
         defaults = [
-            v
-            for value in signature.parameters.values()
-            if (v := value.default) is not inspect.Parameter.empty
+            param.default
+            for param in signature.parameters.values()
+            if param.default is not inspect.Parameter.empty
         ]
         for i in range(len(defaults) - 1, -1, -1):
-            default = defaults[i]
-            code = (
-                code[: res.start(i + 1)] + default.__repr__() + code[res.end(i + 1) :]
-            )
+            start, end = m.start(i + 1), m.end(i + 1)
+            code = code[:start] + defaults[i].__repr__() + code[end:]
+
         return code
