@@ -9,6 +9,7 @@ import json
 from typing import TYPE_CHECKING
 from urllib.parse import unquote, quote
 from warnings import warn
+from cachetools import cached, TTLCache
 
 import ibm_watsonx_ai._wrappers.requests as requests
 from ibm_watsonx_ai.messages.messages import Messages
@@ -113,7 +114,7 @@ class Connections(WMLResource):
         if connection_id:
             with self.requests_retry_session() as sess:
                 response = sess.get(
-                    self._client.service_instance._href_definitions.get_connection_by_id_href(
+                    self._client._href_definitions.get_connection_by_id_href(
                         connection_id
                     ),
                     params=self._client._params(),
@@ -131,9 +132,7 @@ class Connections(WMLResource):
         else:
             with self.requests_retry_session() as sess:
                 response = sess.post(
-                    self._client.service_instance._href_definitions.get_asset_search_href(
-                        "connection"
-                    ),
+                    self._client._href_definitions.get_asset_search_href("connection"),
                     json={"query": "*:*", "include": "entity"},
                     params=self._client._params(),
                     headers=header_param,
@@ -271,7 +270,7 @@ class Connections(WMLResource):
         print(Messages.get_message(message_id="creating_connections"))
 
         creation_response = requests.post(
-            self._client.service_instance._href_definitions.get_connections_href(),
+            self._client._href_definitions.get_connections_href(),
             headers=self._client._get_headers(),
             json=connection_meta,
             params=self._client._params(),
@@ -332,10 +331,8 @@ class Connections(WMLResource):
 
         Connections._validate_type(connection_id, "connection_id", str, True)
 
-        connection_endpoint = (
-            self._client.service_instance._href_definitions.get_connection_by_id_href(
-                connection_id
-            )
+        connection_endpoint = self._client._href_definitions.get_connection_by_id_href(
+            connection_id
         )
         response_delete = requests.delete(
             connection_endpoint,
@@ -407,10 +404,11 @@ class Connections(WMLResource):
         datasource_details = []
 
         def get_res(url):
-            response = requests.get(
-                url,
-                headers=self._client._get_headers(),
-            )
+            with self.requests_retry_session() as sess:
+                response = sess.get(
+                    url,
+                    headers=self._client._get_headers(),
+                )
 
             res = self._handle_response(
                 200, "list datasource types", response, _silent_response_logging=True
@@ -419,7 +417,7 @@ class Connections(WMLResource):
             return res, response.json().get("next", {}).get("href")
 
         res, url = get_res(
-            self._client.service_instance._href_definitions.get_connection_data_types_href()
+            self._client._href_definitions.get_connection_data_types_href()
         )
         datasource_details.extend(res)
 
@@ -447,10 +445,10 @@ class Connections(WMLResource):
 
         space_values = [
             (
-                m["entity"]["name"],
-                m["metadata"]["asset_id"],
-                m["entity"]["type"],
-                m["entity"]["status"],
+                m["entity"].get("name"),
+                m["metadata"].get("asset_id"),
+                m["entity"].get("type"),
+                m["entity"].get("status"),
             )
             for m in datasource_details
         ]
@@ -513,7 +511,7 @@ class Connections(WMLResource):
             table = self._list_uploaded_db_drivers_new_api()
         except:
             response = requests.get(
-                self._client.service_instance._href_definitions.get_wsd_model_attachment_href()
+                self._client._href_definitions.get_wsd_model_attachment_href()
                 + "dbdrivers",
                 headers=self._client._get_headers(no_content_type=True),
                 params=self._client._params(),
@@ -536,7 +534,9 @@ class Connections(WMLResource):
 
         **Example:**
 
-         >>> result = client.connections.get_uploaded_db_drivers()
+        .. code-block:: python
+
+            result = client.connections.get_uploaded_db_drivers()
 
         """
         if not self._client.ICP_PLATFORM_SPACES or self._client.CPD_version < 4.6:
@@ -545,7 +545,7 @@ class Connections(WMLResource):
             )
 
         response = requests.get(
-            self._client.service_instance._href_definitions.get_connections_files_href(),
+            self._client._href_definitions.get_connections_files_href(),
             headers=self._client._get_headers(no_content_type=True),
         )
         result = self._handle_response(
@@ -572,6 +572,38 @@ class Connections(WMLResource):
         jars = [[name] for name in self.get_uploaded_db_drivers()]
         return self._list(jars, ["NAME"], None)
 
+    @cached(cache=TTLCache(maxsize=32, ttl=5 * 60))
+    def _get_datasource_type_details(
+        self, datasource_type: str, connection_properties: bool = False
+    ) -> dict:
+        """Get datasource type details for the given datasource type ID or NAME.
+
+        :param datasource_type: ID or NAME of the datasource type
+        :type datasource_type: str
+
+        :param connection_properties: if True, the connection properties are included in the returned details. defaults to False
+        :type connection_properties: bool
+
+        :return: Datasource type details
+        :rtype: dict
+        """
+        Connections._validate_type(datasource_type, "datasource_type", str, False)
+        header_param = self._client._get_headers()
+        params = {"connection_properties": connection_properties}
+
+        with self.requests_retry_session() as sess:
+            response = sess.get(
+                self._client._href_definitions.get_connection_data_type_href(
+                    datasource_type
+                ),
+                params=params,
+                headers=header_param,
+            )
+
+        return self._handle_response(
+            200, "get datasource details", response, _silent_response_logging=True
+        )
+
     def get_datasource_type_details_by_id(
         self, datasource_type_id: str, connection_properties: bool = False
     ) -> dict:
@@ -579,6 +611,7 @@ class Connections(WMLResource):
 
         :param datasource_type_id: ID of the datasource type
         :type datasource_type_id: str
+
         :param connection_properties: if True, the connection properties are included in the returned details. defaults to False
         :type connection_properties: bool
 
@@ -592,21 +625,35 @@ class Connections(WMLResource):
             client.connections.get_datasource_type_details_by_id(datasource_type_id)
 
         """
-
-        Connections._validate_type(datasource_type_id, "datasource_type", str, False)
-        header_param = self._client._get_headers()
-        params = {"connection_properties": connection_properties}
-
-        response = requests.get(
-            self._client.service_instance._href_definitions.get_connection_data_type_by_id_href(
-                datasource_type_id
-            ),
-            params=params,
-            headers=header_param,
+        return self._get_datasource_type_details(
+            datasource_type=datasource_type_id,
+            connection_properties=connection_properties,
         )
 
-        return self._handle_response(
-            200, "get datasource details", response, _silent_response_logging=True
+    def get_datasource_type_details_by_name(
+        self, datasource_type_name: str, connection_properties: bool = False
+    ) -> dict:
+        """Get datasource type details for the given datasource type name.
+
+        :param datasource_type_name: NAME of the datasource type
+        :type datasource_type_name: str
+
+        :param connection_properties: if True, the connection properties are included in the returned details. defaults to False
+        :type connection_properties: bool
+
+        :return: Datasource type details
+        :rtype: dict
+
+        **Example:**
+
+        .. code-block:: python
+
+            client.connections.get_datasource_type_details_by_name(datasource_type_name)
+
+        """
+        return self._get_datasource_type_details(
+            datasource_type=datasource_type_name,
+            connection_properties=connection_properties,
         )
 
     def get_datasource_type_uid_by_name(self, name: str) -> str:
@@ -653,12 +700,11 @@ class Connections(WMLResource):
         """
         Connections._validate_type(name, "name", str, True)
 
-        datasource_details = self._get_datasource_details()
+        datasource_details = self.get_datasource_type_details_by_name(
+            datasource_type_name=name
+        )
+        datasource_id = datasource_details["metadata"]["asset_id"]
 
-        datasource_id = None
-        for i, ds_resource in enumerate(datasource_details):
-            if ds_resource["entity"]["name"] == name:
-                datasource_id = ds_resource["metadata"]["asset_id"]
         return datasource_id
 
     def get_write_mode_by_datasource_type(self, datasource_type: str) -> str:
@@ -673,7 +719,7 @@ class Connections(WMLResource):
 
         with self.requests_retry_session() as sess:
             response = sess.get(
-                self._client.service_instance._href_definitions.get_connection_data_type_by_id_href(
+                self._client._href_definitions.get_connection_data_type_href(
                     datasource_type
                 ),
                 params=params,
@@ -715,7 +761,7 @@ class Connections(WMLResource):
 
             with open(path, "rb") as fdata:
                 content_upload_url = (
-                    self._client.service_instance._href_definitions.get_wsd_model_attachment_href()
+                    self._client._href_definitions.get_wsd_model_attachment_href()
                     + "dbdrivers/"
                     + quote(driver_file_name, safe="")
                 )
@@ -761,8 +807,10 @@ class Connections(WMLResource):
         driver_file_name = path.split("/")[-1]
 
         with open(path, "rb") as fdata:
-            content_upload_url = self._client.service_instance._href_definitions.get_connections_file_href(
-                quote(driver_file_name, safe="")
+            content_upload_url = (
+                self._client._href_definitions.get_connections_file_href(
+                    quote(driver_file_name, safe="")
+                )
             )
             response = requests.post(
                 content_upload_url,
@@ -840,7 +888,7 @@ class Connections(WMLResource):
                 )
 
             signed_url = (
-                self._client.service_instance._href_definitions.get_wsd_model_attachment_href()
+                self._client._href_definitions.get_wsd_model_attachment_href()
                 + quote("dbdrivers/" + jar_name, safe="")
                 + "/signed"
             )
@@ -868,7 +916,7 @@ class Connections(WMLResource):
     def requests_retry_session(
         retries: int = 3,
         backoff_factor: float = 0.3,
-        status_forcelist: Iterable[int] = (500, 502, 503, 504, 521, 524),
+        status_forcelist: Iterable[int] = (500, 502, 503, 504, 520, 521, 524),
         session: requests.Session | None = None,
     ) -> requests.Session:
         from requests.adapters import HTTPAdapter
