@@ -10,7 +10,7 @@ import io
 import json
 import logging
 
-from typing import TYPE_CHECKING, Any
+from typing import TYPE_CHECKING, Any, Iterator
 from queue import Empty
 
 from ibm_watsonx_ai.utils import DisableWarningsLogger
@@ -27,23 +27,43 @@ if TYPE_CHECKING:
 logger = logging.getLogger(__name__)
 
 
-def _asynch_download(args):
+def _prepare_iterator(el: Any) -> Iterator[Any]:
+    from langchain_core.documents import Document
+
+    if hasattr(el, "__iter__") and not isinstance(el, Document):  # is iterable
+        yield from el
+    else:
+        yield el
+
+
+def _asynch_download(load_doc):
     """Helper function for parallel downloading documents (full asynchronous version)."""
-    (q_input, qs_output) = args
 
-    while True:
-        try:
-            i, doc = q_input.get(block=False)
+    def asynch_download(args):
+
+        (q_input, qs_output) = args
+
+        while True:
             try:
-                doc.download()
-                qs_output[i].put(TextLoader(doc).load())
-            except Exception as e:
-                if "cryptography>=3.1 is required for AES algorithm" in str(e):
-                    e = "Encrypted files are not supported. Please decrypt your file and try again."
+                i, doc = q_input.get(block=False)
+                try:
+                    loaded_doc = load_doc(doc)
 
-                qs_output[i].put(LoadingDocumentError(doc.document_id, e))
-        except Empty:
-            return
+                    for el in _prepare_iterator(loaded_doc):
+                        qs_output[i].put(el)
+
+                    qs_output[i].put(
+                        "End"
+                    )  # send signal that no more data will be sent via this queue
+                except Exception as e:
+                    if "cryptography>=3.1 is required for AES algorithm" in str(e):
+                        e = "Encrypted files are not supported. Please decrypt your file and try again."
+
+                    qs_output[i].put(LoadingDocumentError(doc.document_id, e))
+            except Empty:
+                return
+
+    return asynch_download
 
 
 class TextLoader:
@@ -80,6 +100,10 @@ class TextLoader:
             "application/vnd.ms-powerpoint": self._pptx_to_string,
             "application/json": self._json_to_string,
             "application/x-yaml": self._yaml_to_string,
+            "application/xml": self._xml_to_string,
+            "text/csv": self._csv_to_string,
+            "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet": self._xlsx_to_string,
+            "application/vnd.ms-excel": self._xlsx_to_string,
         }
 
         try:
@@ -119,6 +143,14 @@ class TextLoader:
             return "application/json"
         elif filename.endswith(".yaml"):
             return "application/x-yaml"
+        elif filename.endswith(".xml"):
+            return "application/xml"
+        elif filename.endswith(".csv"):
+            return "text/csv"
+        elif filename.endswith(".xlsx"):
+            return "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet"
+        elif filename.endswith(".xls"):
+            return "application/vnd.ms-excel"
         else:
             raise WMLClientError(f"Cannot identify file type.")
 
@@ -226,3 +258,64 @@ class TextLoader:
 
         loaded_yaml = yaml.load(binary_data, Loader=yaml.Loader)
         return "\n\n".join(cls._extract_content_from_py_structure(loaded_yaml))
+
+    @classmethod
+    def _xml_to_string(cls, binary_data: bytes) -> str:
+        from xml.etree import ElementTree
+
+        def xml_element_parser(elem: ElementTree.Element) -> list[dict | str]:
+            result = [dict(elem.attrib), {}]
+
+            children = list(elem)
+            if children:
+                for child in children:
+                    child_tag = child.tag
+                    child_data = xml_element_parser(child)
+
+                    if child_tag in result[1]:
+                        if isinstance(result[1][child_tag], list):
+                            result[1][child_tag].append(child_data)
+                        else:
+                            result[1][child_tag] = [result[1][child_tag], child_data]
+                    else:
+                        result[1][child_tag] = child_data
+
+            # Add element text to the result list if exists, at 2nd place after attribute
+            if text := (elem.text or "").strip():
+                result.insert(1, text)
+
+            return result
+
+        root = ElementTree.fromstring(binary_data)
+        root_result = []
+
+        if root.attrib:
+            root_result.append(root.attrib)
+        if root.text is not None and (text := root.text.strip()):
+            root_result.append(text)
+
+        for child in root:
+            parsed_child = xml_element_parser(child)
+            root_result.append({child.tag: parsed_child})
+
+        result = {root.tag: root_result}
+
+        return "\n\n".join(cls._extract_content_from_py_structure(result))
+
+    @classmethod
+    def _df_to_key_value_str(cls, df: "DataFrame") -> str:
+        return "\n".join([json.dumps(row)[1:-1] for row in df.to_dict("records")])
+
+    @classmethod
+    def _csv_to_string(cls, binary_data: bytes) -> str:
+        import pandas as pd
+
+        return cls._df_to_key_value_str(
+            pd.read_csv(io.BytesIO(binary_data), index_col=[0])
+        )
+
+    @classmethod
+    def _xlsx_to_string(cls, binary_data: bytes) -> str:
+        import pandas as pd
+
+        return cls._df_to_key_value_str(pd.read_excel(binary_data))
