@@ -45,9 +45,12 @@ from ibm_watsonx_ai.metanames import (
 from ibm_watsonx_ai.libs.repo.util.library_imports import LibraryChecker
 from ibm_watsonx_ai.utils.autoai.utils import all_logging_disabled
 
+from urllib.parse import urlparse, parse_qs
+
 if TYPE_CHECKING:
     from ibm_watsonx_ai import APIClient
     from ibm_watsonx_ai.lifecycle import SpecStates
+    from ibm_watsonx_ai.foundation_models.inference import ModelInference
     import pandas
 
 lib_checker = LibraryChecker()
@@ -255,7 +258,7 @@ class Deployments(WMLResource):
                         metaProps.update({DeploymentMetaNames.BASE_MODEL_ID: model_id})
         # --- end note
 
-        url = self._client.service_instance._href_definitions.get_deployments_href()
+        url = self._client._href_definitions.get_deployments_href()
 
         response = self._client.httpx_client.post(
             url,
@@ -463,7 +466,7 @@ class Deployments(WMLResource):
             "version": self._client.version_param,
         }
 
-        url = self._client.service_instance._href_definitions.get_deployments_href()
+        url = self._client._href_definitions.get_deployments_href()
         res = self._client.httpx_client.get(
             url, headers=self._client._get_headers(), params=params
         )
@@ -560,7 +563,7 @@ class Deployments(WMLResource):
                 "'deployment_id' is not an id: '{}'".format(deployment_id)
             )
 
-        url = self._client.service_instance._href_definitions.get_deployments_href()
+        url = self._client._href_definitions.get_deployments_href()
 
         query_params = self._client._params()
 
@@ -718,10 +721,8 @@ class Deployments(WMLResource):
                 "'deployment_id' is not an id: '{}'".format(deployment_id)
             )
 
-        deployment_url = (
-            self._client.service_instance._href_definitions.get_deployment_href(
-                deployment_id
-            )
+        deployment_url = self._client._href_definitions.get_deployment_href(
+            deployment_id
         )
 
         response_delete = self._client.httpx_client.delete(
@@ -975,7 +976,7 @@ class Deployments(WMLResource):
                             if "guid" in m["metadata"]
                             else m["metadata"]["id"]
                         ),
-                        m["entity"]["name"],
+                        m["metadata"].get("name", ""),
                         m["entity"]["status"]["state"],
                         m["metadata"]["created_at"],
                         m["entity"].get("deployed_asset_type", "unknown"),
@@ -1171,9 +1172,7 @@ class Deployments(WMLResource):
                 }
             )
 
-        url = self._client.service_instance._href_definitions.get_deployment_href(
-            deployment_id
-        )
+        url = self._client._href_definitions.get_deployment_href(deployment_id)
 
         response = self._client.httpx_client.patch(
             url,
@@ -1271,9 +1270,7 @@ class Deployments(WMLResource):
         if transaction_id is not None:
             headers.update({"x-global-transaction-id": transaction_id})
         # making change - connection keep alive
-        scoring_url = (
-            self._client.service_instance._href_definitions.get_async_deployment_job_href()
-        )
+        scoring_url = self._client._href_definitions.get_async_deployment_job_href()
         params = self._client._params()
 
         if not self._client.ICP_PLATFORM_SPACES and retention is not None:
@@ -1531,9 +1528,7 @@ class Deployments(WMLResource):
         )
         if job_id is not None:
             Deployments._validate_type(job_id, "job_id", str, True)
-        url = (
-            self._client.service_instance._href_definitions.get_async_deployment_job_href()
-        )
+        url = self._client._href_definitions.get_async_deployment_job_href()
 
         params = self._client._params()
         if include:
@@ -1680,10 +1675,8 @@ class Deployments(WMLResource):
                 job_details = self.get_job_details(job_id=job_id)
                 run_id = job_details["entity"]["platform_job"]["run_id"]
 
-                jobs_runs_url = (
-                    self._client.service_instance._href_definitions.get_jobs_runs_href(
-                        job_id=job_id, run_id=run_id
-                    )
+                jobs_runs_url = self._client._href_definitions.get_jobs_runs_href(
+                    job_id=job_id, run_id=run_id
                 )
 
                 response_delete = self._client.httpx_client.delete(
@@ -1696,9 +1689,7 @@ class Deployments(WMLResource):
             except:
                 pass
 
-        url = self._client.service_instance._href_definitions.get_async_deployment_jobs_href(
-            job_id
-        )
+        url = self._client._href_definitions.get_async_deployment_jobs_href(job_id)
 
         if hard_delete is True:
             params.update({"hard_delete": "true"})
@@ -1739,7 +1730,7 @@ class Deployments(WMLResource):
         deployment_id: str,
         inference_type: Literal["text", "text_stream", "chat", "chat_stream"],
         params: dict | None = None,
-    ) -> Any:
+    ) -> "ModelInference":
         """Based on provided deployment_id and params get ModelInference object.
         Verify that the deployment with the given deployment_id has generating methods.
         """
@@ -1748,25 +1739,31 @@ class Deployments(WMLResource):
 
         match inference_type:
             case "text":
-                generated_url = self._client.service_instance._href_definitions.get_fm_deployment_generation_href(
-                    deployment_id=deployment_id, item="text"
+                generated_url = (
+                    self._client._href_definitions.get_fm_deployment_generation_href(
+                        deployment_id=deployment_id, item="text"
+                    )
                 )
             case "text_stream":
                 if self._client._use_fm_ga_api:
-                    generated_url = self._client.service_instance._href_definitions.get_fm_deployment_generation_stream_href(
+                    generated_url = self._client._href_definitions.get_fm_deployment_generation_stream_href(
                         deployment_id=deployment_id
                     )
                 else:  # Remove on CPD 5.0 release
-                    generated_url = self._client.service_instance._href_definitions.get_fm_deployment_generation_href(
+                    generated_url = self._client._href_definitions.get_fm_deployment_generation_href(
                         deployment_id=deployment_id, item="text_stream"
                     )
             case "chat":
-                generated_url = self._client.service_instance._href_definitions.get_fm_deployment_chat_href(
-                    deployment_id=deployment_id
+                generated_url = (
+                    self._client._href_definitions.get_fm_deployment_chat_href(
+                        deployment_id=deployment_id
+                    )
                 )
             case "chat_stream":
-                generated_url = self._client.service_instance._href_definitions.get_fm_deployment_chat_stream_href(
-                    deployment_id=deployment_id
+                generated_url = (
+                    self._client._href_definitions.get_fm_deployment_chat_stream_href(
+                        deployment_id=deployment_id
+                    )
                 )
             case _:
                 raise InvalidValue(
@@ -1786,19 +1783,21 @@ class Deployments(WMLResource):
                 .get("status", {})
                 .get("serving_urls", [])
             )
-        if inference_type in ["text", "text_stream"]:
-            if generated_url not in inference_url_list:
 
-                if all(
-                    "/text/generation" not in inference_url
-                    for inference_url in inference_url_list
-                ):
-                    raise WMLClientError(
-                        Messages.get_message(
-                            deployment_id,
-                            message_id="fm_deployment_has_not_inference_for_generation",
-                        )
-                    )
+        if (
+            inference_type in ["text", "text_stream"]
+            and generated_url not in inference_url_list
+            and all(
+                "/text/generation" not in inference_url
+                for inference_url in inference_url_list
+            )
+        ):
+            raise WMLClientError(
+                Messages.get_message(
+                    deployment_id,
+                    message_id="fm_deployment_has_not_inference_for_generation",
+                )
+            )
 
         return ModelInference(
             deployment_id=deployment_id, params=params, api_client=self._client
@@ -1810,11 +1809,12 @@ class Deployments(WMLResource):
         prompt: str | None = None,
         params: dict | None = None,
         guardrails: bool = False,
-        guardrails_hap_params: bool | None = None,
-        guardrails_pii_params: bool | None = None,
+        guardrails_hap_params: dict | None = None,
+        guardrails_pii_params: dict | None = None,
         concurrency_limit: int = DEFAULT_CONCURRENCY_LIMIT,
         async_mode: bool = False,
         validate_prompt_variables: bool = True,
+        guardrails_granite_guardian_params: dict | None = None,
     ) -> dict:
         """Generate a raw response with `prompt` for given `deployment_id`.
 
@@ -1847,6 +1847,9 @@ class Deployments(WMLResource):
                                           This parameter is only applicable in a Prompt Template Asset deployment scenario and should not be changed for different cases, defaults to True
         :type validate_prompt_variables: bool
 
+        :param guardrails_granite_guardian_params: parameters for Granite Guardian moderations
+        :type guardrails_granite_guardian_params: dict, optional
+
         :return: scoring result containing generated content
         :rtype: dict
         """
@@ -1860,6 +1863,7 @@ class Deployments(WMLResource):
             params=params,
             async_mode=async_mode,
             validate_prompt_variables=validate_prompt_variables,
+            guardrails_granite_guardian_params=guardrails_granite_guardian_params,
         )
 
     def generate_text(
@@ -1869,10 +1873,11 @@ class Deployments(WMLResource):
         params: dict | None = None,
         raw_response: bool = False,
         guardrails: bool = False,
-        guardrails_hap_params: bool | None = None,
-        guardrails_pii_params: bool | None = None,
+        guardrails_hap_params: dict | None = None,
+        guardrails_pii_params: dict | None = None,
         concurrency_limit: int = DEFAULT_CONCURRENCY_LIMIT,
         validate_prompt_variables: bool = True,
+        guardrails_granite_guardian_params: dict | None = None,
     ) -> str:
         """Given the selected deployment (deployment_id), a text prompt as input, and the parameters and concurrency_limit,
         the selected inference will generate a completion text as generated_text response.
@@ -1904,6 +1909,9 @@ class Deployments(WMLResource):
                                           This parameter is only applicable in a Prompt Template Asset deployment scenario and should not be changed for different cases, defaults to True
         :type validate_prompt_variables: bool
 
+        :param guardrails_granite_guardian_params: parameters for Granite Guardian moderations
+        :type guardrails_granite_guardian_params: dict, optional
+
         :return: generated content
         :rtype: str
 
@@ -1928,6 +1936,7 @@ class Deployments(WMLResource):
             concurrency_limit=concurrency_limit,
             params=params,
             validate_prompt_variables=validate_prompt_variables,
+            guardrails_granite_guardian_params=guardrails_granite_guardian_params,
         )
 
     def generate_text_stream(
@@ -1937,9 +1946,10 @@ class Deployments(WMLResource):
         params: dict | None = None,
         raw_response: bool = False,
         guardrails: bool = False,
-        guardrails_hap_params: bool | None = None,
-        guardrails_pii_params: bool | None = None,
+        guardrails_hap_params: dict | None = None,
+        guardrails_pii_params: dict | None = None,
         validate_prompt_variables: bool = True,
+        guardrails_granite_guardian_params: dict | None = None,
     ) -> str:
         """Given the selected deployment (deployment_id), a text prompt as input and parameters,
         the selected inference will generate a streamed text as generate_text_stream.
@@ -1968,6 +1978,9 @@ class Deployments(WMLResource):
                                           This parameter is only applicable in a Prompt Template Asset deployment scenario and should not be changed for different cases, defaults to True
         :type validate_prompt_variables: bool
 
+        :param guardrails_granite_guardian_params: parameters for Granite Guardian moderations
+        :type guardrails_granite_guardian_params: dict, optional
+
         :return: generated content
         :rtype: str
 
@@ -1991,37 +2004,87 @@ class Deployments(WMLResource):
             guardrails_hap_params=guardrails_hap_params,
             guardrails_pii_params=guardrails_pii_params,
             validate_prompt_variables=validate_prompt_variables,
+            guardrails_granite_guardian_params=guardrails_granite_guardian_params,
         )
 
     def chat(
-        self, deployment_id: str, messages: ListType[dict], context: str | None = None
+        self,
+        deployment_id: str,
+        messages: ListType[dict],
+        context: str | None = None,
+        tools: list | None = None,
+        tool_choice: dict | None = None,
+        tool_choice_option: Literal["none", "auto"] | None = None,
     ) -> dict:
         d_inference = self._get_model_inference(deployment_id, "chat")
-
-        return d_inference.chat(messages=messages, context=context)
+        return d_inference.chat(
+            messages=messages,
+            context=context,
+            tools=tools,
+            tool_choice=tool_choice,
+            tool_choice_option=tool_choice_option,
+        )
 
     def chat_stream(
-        self, deployment_id: str, messages: ListType[dict], context: str | None = None
+        self,
+        deployment_id: str,
+        messages: ListType[dict],
+        context: str | None = None,
+        tools: list | None = None,
+        tool_choice: dict | None = None,
+        tool_choice_option: Literal["none", "auto"] | None = None,
     ) -> Generator:
         d_inference = self._get_model_inference(deployment_id, "chat_stream")
-
-        return d_inference.chat_stream(messages=messages, context=context)
+        return d_inference.chat_stream(
+            messages=messages,
+            context=context,
+            tools=tools,
+            tool_choice=tool_choice,
+            tool_choice_option=tool_choice_option,
+        )
 
     async def achat(
-        self, deployment_id: str, messages: ListType[dict], context: str | None = None
+        self,
+        deployment_id: str,
+        messages: ListType[dict],
+        context: str | None = None,
+        tools: list | None = None,
+        tool_choice: dict | None = None,
+        tool_choice_option: Literal["none", "auto"] | None = None,
     ) -> dict:
         d_inference = self._get_model_inference(deployment_id, "chat")
-
-        return await d_inference.achat(messages=messages, context=context)
+        return await d_inference.achat(
+            messages=messages,
+            context=context,
+            tools=tools,
+            tool_choice=tool_choice,
+            tool_choice_option=tool_choice_option,
+        )
 
     async def achat_stream(
-        self, deployment_id: str, messages: ListType[dict], context: str | None = None
+        self,
+        deployment_id: str,
+        messages: ListType[dict],
+        context: str | None = None,
+        tools: list | None = None,
+        tool_choice: dict | None = None,
+        tool_choice_option: Literal["none", "auto"] | None = None,
     ) -> AsyncGenerator:
         d_inference = self._get_model_inference(deployment_id, "chat_stream")
+        return await d_inference.achat_stream(
+            messages=messages,
+            context=context,
+            tools=tools,
+            tool_choice=tool_choice,
+            tool_choice_option=tool_choice_option,
+        )
 
-        return await d_inference.achat_stream(messages=messages, context=context)
-
-    def run_ai_service(self, deployment_id: str, ai_service_payload: dict) -> Any:
+    def run_ai_service(
+        self,
+        deployment_id: str,
+        ai_service_payload: dict,
+        path_suffix: str | None = None,
+    ) -> Any:
         """Execute an AI service by providing a scoring payload.
 
         :param deployment_id: unique ID of the deployment
@@ -2030,6 +2093,9 @@ class Deployments(WMLResource):
         :param ai_service_payload: AI service payload to be passed to generate the method
         :type ai_service_payload: dict
 
+        :param path_suffix: path suffix to be appended to the scoring url, defaults to None
+        :type path_suffix: str, optional
+
         :return: response of the AI service
         :rtype: Any
 
@@ -2041,11 +2107,13 @@ class Deployments(WMLResource):
         Deployments._validate_type(ai_service_payload, "ai_service_payload", dict, True)
 
         scoring_url = (
-            self._client.service_instance._href_definitions.get_deployment_href(
-                deployment_id
-            )
+            self._client._href_definitions.get_deployment_href(deployment_id)
             + "/ai_service"
         )
+
+        if path_suffix is not None:
+            scoring_url += "/" + path_suffix
+
         response_scoring = self._client.httpx_client.post(
             url=scoring_url,
             json=ai_service_payload,
@@ -2070,7 +2138,9 @@ class Deployments(WMLResource):
         return self._handle_response(200, "AI Service run", response_scoring)
 
     def run_ai_service_stream(
-        self, deployment_id: str, ai_service_payload: dict
+        self,
+        deployment_id: str,
+        ai_service_payload: dict,
     ) -> Generator:
         """Execute an AI service by providing a scoring payload.
 
@@ -2087,9 +2157,7 @@ class Deployments(WMLResource):
         Deployments._validate_type(ai_service_payload, "ai_service_payload", dict, True)
 
         scoring_url = (
-            self._client.service_instance._href_definitions.get_deployment_href(
-                deployment_id
-            )
+            self._client._href_definitions.get_deployment_href(deployment_id)
             + "/ai_service_stream"
         )
 
@@ -2124,6 +2192,12 @@ class RuntimeContext:
     :param request_payload_json: Request payload for testing of generate/ generate_stream call of AI Service.
     :type request_payload_json: dict, optional
 
+    :param method: HTTP request method for testing of generate/ generate_stream call of AI Service.
+    :type method: str, optional
+
+    :param path: Request endpoint path for testing of generate/ generate_stream call of AI Service.
+    :type path: str, optional
+
     ``
     RuntimeContext`` initialized for testing purposes before deployment:
 
@@ -2162,9 +2236,17 @@ class RuntimeContext:
         generate_output = generate(context)  # returns {"body": {"field2": "value2"}}
     """
 
-    def __init__(self, api_client: APIClient, request_payload_json: dict | None = None):
+    def __init__(
+        self,
+        api_client: APIClient,
+        request_payload_json: dict | None = None,
+        method: str | None = None,
+        path: str | None = None,
+    ):
         self._api_client = api_client
         self.request_payload_json = request_payload_json
+        self.method = method
+        self.path = path
 
     @property
     def request_payload_json(self) -> dict | None:
@@ -2201,3 +2283,37 @@ class RuntimeContext:
     def get_space_id(self) -> str:
         """Return default space id."""
         return self._api_client.default_space_id
+
+    def get_method(self) -> str:
+        """Return the HTTP request method: 'GET', 'POST', etc."""
+        return self.method or ""
+
+    def get_path_suffix(self) -> str:
+        """Return the suffix of ai_service endpoint including the query parameters."""
+        try:
+            suffix = self.path.split("ai_service", 1)[1]
+        except IndexError as e:
+            raise ValueError(
+                "Couldn't find the path suffix since endpoint URL is incorrect."
+            ) from e
+        if suffix:
+            suffix = suffix.removeprefix("/")
+        return suffix
+
+    def get_query_parameters(self) -> dict:
+        """Return the query parameters from the ai_service endpoint as a dict."""
+        parsed_url = urlparse(self.path)
+        query = parsed_url.query
+        params = parse_qs(query)
+        if params:
+            flat_params = {k: v[0] for k, v in params.items()}
+            return flat_params
+        else:
+            return {}
+
+    def get_bytes(self) -> bytes:
+        """Return the request data as bytes."""
+        payload_json = self.get_json()
+        payload_str = json.dumps(payload_json)
+        bytes_data = payload_str.encode("utf-8")
+        return bytes_data
