@@ -65,6 +65,11 @@ from ..base_experiment.base_experiment import BaseExperiment
 __all__ = ["AutoAI"]
 
 from ...credentials import Credentials
+from ...foundation_models.schema import (
+    AutoAIRAGModelConfig,
+    AutoAIRAGCustomModelConfig,
+    AutoAIRAGRetrievalConfig,
+)
 
 if TYPE_CHECKING:
     from .optimizers import RAGOptimizer
@@ -718,16 +723,18 @@ class AutoAI(BaseExperiment):
             else None
         )
         include_batched_ensemble_estimators = (
-            translate_str_include_batched_ensemble_estimators_param(
-                include_batched_ensemble_estimators
+            include_batched_ensemble_estimators
+            if include_batched_ensemble_estimators is None
+            else (
+                translate_str_include_batched_ensemble_estimators_param(
+                    include_batched_ensemble_estimators
+                )
             )
-            if include_batched_ensemble_estimators
-            else None
         )
         pipeline_types = (
-            translate_str_pipeline_types_param(pipeline_types)
-            if pipeline_types != None
-            else None
+            pipeline_types
+            if pipeline_types is None
+            else (translate_str_pipeline_types_param(pipeline_types))
         )
 
         if include_batched_ensemble_estimators:
@@ -935,9 +942,13 @@ class AutoAI(BaseExperiment):
         chunking: list[dict] | None = None,
         embedding_models: list[str] | None = None,
         retrieval_methods: list[str] | None = None,
-        foundation_models: list[str] | None = None,
+        foundation_models: (
+            list[str | dict | AutoAIRAGModelConfig | AutoAIRAGCustomModelConfig] | None
+        ) = None,
         max_number_of_rag_patterns: int | None = None,
         optimization_metrics: list[str] | None = None,
+        generation: dict[str, Any] | None = None,
+        retrieval: list[dict[str, Any] | AutoAIRAGRetrievalConfig] | None = None,
         **kwargs: Any,
     ) -> "RAGOptimizer":
         """Initialize an AutoAi RAG optimizer.
@@ -957,8 +968,8 @@ class AutoAI(BaseExperiment):
         :param retrieval_methods: Retrieval methods to be used.
         :type retrieval_methods: list[str], optional
 
-        :param foundation_models: The foundation models to try.
-        :type foundation_models: list[str], optional
+        :param foundation_models: List of foundation models to try. Custom foundation models and model config are also supported for Cloud and CPD >= 5.2.
+        :type foundation_models: list[str | dict | AutoAIRAGModelConfig | AutoAIRAGCustomModelConfig], optional
 
         :param max_number_of_rag_patterns: The maximum number of RAG patterns to create.
         :type max_number_of_rag_patterns: int, optional
@@ -966,11 +977,19 @@ class AutoAI(BaseExperiment):
         :param optimization_metrics: The metric name(s) to be used for optimization.
         :type optimization_metrics: list[str], optional
 
+        :param generation: Properties describing the generation step.
+        :type generation: dict[str, Any], optional
+
+        :param retrieval: Retrieval settings to be used.
+        :type retrieval: list[dict[str, Any] | AutoAIRAGRetrievalConfig], optional
+
         :return: AutoAI RAG optimizer
         :rtype: RAGOptimizer
 
         **Examples**
 
+        - RAG Optimizer configuration with defaults
+
         .. code-block:: python
 
             from ibm_watsonx_ai.experiment import AutoAI
@@ -978,11 +997,35 @@ class AutoAI(BaseExperiment):
 
             optimizer = experiment.rag_optimizer(
                 name="RAG - AutoAI",
-                description="Sample description",
-                max_number_of_rag_patterns=5,
-                optimization_metrics=["answer_correctness"]
+                description="Sample description"
             )
 
+
+        - RAG Optimizer configuration with example values of parameters
+
+        .. code-block:: python
+
+            from ibm_watsonx_ai.experiment import AutoAI
+
+            experiment = AutoAI(...)
+
+            optimizer = experiment.rag_optimizer(
+                name="RAG - AutoAI",
+                description="Sample description",
+                max_number_of_rag_patterns=5,
+                chunking=[
+                    {"method": "recursive", "chunk_size": 1024, "chunk_overlap": 64},
+                    {"method": "semantic", "chunk_size": 1024},
+                ],
+                embedding_models=["ibm/slate-125m-english-rtrvr-v2"],
+                foundation_models=["meta-llama/llama-3-3-70b-instruct"],
+                retrieval=[
+                    {"method": "window", "number_of_chunks": 3, "window_size": 2},
+                    {"method": "simple", "number_of_chunks": 5},
+                ],
+                generation={"language": {"auto-detect": True}},
+                optimization_metrics=["answer_correctness"],
+            )
         """
         from .engines import RAGEngine
 
@@ -1000,6 +1043,8 @@ class AutoAI(BaseExperiment):
             max_number_of_rag_patterns=max_number_of_rag_patterns,
             optimization_metrics=optimization_metrics,
             engine=engine,
+            generation=generation,
+            retrieval=retrieval,
             **kwargs,
         )
         optimizer._workspace = self._workspace
