@@ -2,8 +2,12 @@
 #  (C) Copyright IBM Corp. 2024-2025.
 #  https://opensource.org/licenses/BSD-3-Clause
 #  -----------------------------------------------------------------------------------------
-
+import gzip
+import io
+import json
 import time
+from copy import deepcopy
+
 from pandas import DataFrame
 from typing import TYPE_CHECKING, Any, Literal
 
@@ -61,8 +65,8 @@ class RAGEngine(WMLResource):
         self,
         *,
         input_data_references: list[DataConnection],
-        test_data_references: list[DataConnection],
         results_reference: DataConnection,
+        test_data_references: list[DataConnection] | None = None,
         vector_store_references: list[DataConnection] | None = None,
         background_mode: bool = True,
     ) -> dict:
@@ -71,12 +75,12 @@ class RAGEngine(WMLResource):
         :param input_data_references: Data storage connection details to inform where training data is stored
         :type input_data_references: list[DataConnection]
 
-        :param test_data_references: A set of test data references
-        :type test_data_references: list[DataConnection]
-
         :param results_reference: The training results
         :type results_reference: DataConnection
 
+        :param test_data_references: A set of test data references
+        :type test_data_references: list[DataConnection], optional
+
         :param vector_store_references: A set of vector store references
         :type vector_store_references: list[DataConnection], optional
 
@@ -95,19 +99,20 @@ class RAGEngine(WMLResource):
                 input_conn.location.userfs = "true"  # type: ignore[union-attr]
             input_conn.set_client(self.workspace.api_client)
 
-        for test_conn in test_data_references:
-            if self.workspace.api_client.project_type == "local_git_storage":
-                input_conn.location.userfs = "true"  # type: ignore[union-attr]
-            test_conn.set_client(self.workspace.api_client)
+        if test_data_references is not None:
+            for test_conn in test_data_references:
+                if self.workspace.api_client.project_type == "local_git_storage":
+                    input_conn.location.userfs = "true"  # type: ignore[union-attr]
+                test_conn.set_client(self.workspace.api_client)
 
         self._initialize_training_metadata(
             input_data_references=input_data_references,
-            test_data_references=test_data_references,
             results_reference=results_reference,
+            test_data_references=test_data_references,
             vector_store_references=vector_store_references,
         )
 
-        url = self._client.service_instance._href_definitions.get_autoai_rag_href()
+        url = self._client._href_definitions.get_autoai_rag_href()
 
         response_train_post = requests.post(
             url=url,
@@ -151,11 +156,25 @@ class RAGEngine(WMLResource):
                 )
             else:
                 error_msg = (
-                    details.get("entity", {}).get("message", {}).get("text", None)
+                    details.get("entity", {})
+                    .get("status", {})
+                    .get("message", {})
+                    .get("text")
                 )
+
+                if error_msg is not None:
+                    prepared_error_message = "Error message: '{}'".format(error_msg)
+                else:
+                    prepared_error_message = (
+                        "No error message was returned from the service."
+                    )
+
+                    if status == "canceled":
+                        prepared_error_message += " Please verify if your instance has enough resources to run AutoAI."
+
                 print(
-                    "\nTraining of '{}' failed with status: '{}'. Error message: '{}'".format(
-                        self._current_run_id, str(status), error_msg
+                    "\nTraining of '{}' failed with status: '{}'. {}".format(
+                        self._current_run_id, str(status), prepared_error_message
                     )
                 )
 
@@ -182,7 +201,7 @@ class RAGEngine(WMLResource):
 
         self._client._check_if_either_is_set()
 
-        url = self._client.service_instance._href_definitions.get_autoai_rag_id_href(
+        url = self._client._href_definitions.get_autoai_rag_id_href(
             self._current_run_id
         )
         params = self._client._params()
@@ -224,7 +243,7 @@ class RAGEngine(WMLResource):
 
         self._client._check_if_either_is_set()
 
-        url = self._client.service_instance._href_definitions.get_autoai_rag_href()
+        url = self._client._href_definitions.get_autoai_rag_href()
 
         return self._get_artifact_details(
             base_url=url,
@@ -373,7 +392,11 @@ class RAGEngine(WMLResource):
             "chunking": ["method", "chunk_size", "chunk_overlap"],
             "embeddings": ["model_id"],
             "vector_store": ["distance_metric"],
-            "retrieval": ["method", "number_of_chunks"],
+            "retrieval": [
+                "method",
+                "number_of_chunks",
+                "hybrid_ranker",
+            ],
             "generation": ["model_id"],
         }
         data_sorted_by_value.update(
@@ -396,17 +419,28 @@ class RAGEngine(WMLResource):
                         data_sorted_by_value, f"mean_{metric_name}", metric_item["mean"]
                     )
 
-            settings = result["context"]["rag_pattern"]["settings"]
+            settings = deepcopy(result["context"]["rag_pattern"]["settings"])
+
+            if "deployment_id" in settings.get("generation", {}):
+                settings["generation"]["model_id"] = settings["generation"].pop(
+                    "deployment_id"
+                )
+
+            def get_nested_value(data_dict: dict, dict_key: str) -> dict | str:
+                keys = dict_key.split(".")
+                for key in keys:
+                    if isinstance(data_dict, dict) and key in data_dict:
+                        data_dict = data_dict[key]
+                    else:
+                        return ""
+                return data_dict
 
             for key, sub_keys in setting_keys.items():
                 if key in settings:
                     for sub_key in sub_keys:
-                        if sub_key in settings[key]:
-                            add_to_data(
-                                data_sorted_by_value,
-                                f"{key}.{sub_key}",
-                                settings[key][sub_key],
-                            )
+                        full_key = f"{key}.{sub_key}"
+                        value = get_nested_value(settings[key], sub_key)
+                        add_to_data(data_sorted_by_value, full_key, value)
 
         data_sorted_by_optimization_metrics: dict = {"Pattern_Name": rag_pattern_names}
 
@@ -435,6 +469,7 @@ class RAGEngine(WMLResource):
         :rtype: RAGPattern
 
         """
+        from ibm_watsonx_ai.foundation_models.extensions.rag.pattern import RAGPattern
 
         details = self.get_run_details()
 
@@ -442,6 +477,19 @@ class RAGEngine(WMLResource):
             details=details, pattern_name=pattern_name
         )
 
+        service_code = self._get_inference_service(details, pattern_name)
+        service_metadata = (
+            self._get_service_metadata(details, pattern_name) if service_code else None
+        )
+
+        if service_metadata and service_code:
+            return RAGPattern(
+                space_id=details["metadata"].get("space_id"),
+                project_id=details["metadata"].get("project_id"),
+                api_client=self._client,
+                store_params=service_metadata,
+                _service_code=service_code,  # if empty, fallback to legacy params for backward compatibility
+            )
         embeddings_settings = pattern_details["context"]["rag_pattern"]["settings"][
             "embeddings"
         ]
@@ -458,29 +506,191 @@ class RAGEngine(WMLResource):
         vector_store_settings = pattern_details["context"]["rag_pattern"]["settings"][
             "vector_store"
         ]
-        datasource_type = None
         connection_id = None
 
         if vector_store_references := details["entity"].get("vector_store_references"):
             connection_id = vector_store_references[0]["connection"]["id"]
-        else:
-            datasource_type = vector_store_settings["datasource_type"]
-
-        from ibm_watsonx_ai.foundation_models.extensions.rag import VectorStore
-
-        vector_store = VectorStore(
-            connection_id=connection_id,
-            datasource_type=datasource_type,
-            index_name=vector_store_settings.get("index_name"),
-            distance_metric=vector_store_settings.get("distance_metric"),
-            embeddings=embeddings,
-            api_client=self._client,
-        )
+        datasource_type = vector_store_settings["datasource_type"]
 
         retrieval_settings = pattern_details["context"]["rag_pattern"]["settings"][
             "retrieval"
         ]
 
+        if hybrid_ranker := retrieval_settings.get("hybrid_ranker"):
+            strategy: str = hybrid_ranker["strategy"]
+            model_id: str | None = hybrid_ranker.get("sparse_vectors", {}).get(
+                "model_id"
+            )
+            alpha: float | None = hybrid_ranker.get("alpha")
+            k: int | None = hybrid_ranker.get("k")
+
+        ranker_config: dict[str, Any] | None = None
+
+        schema_fields: list[dict[str, str]] = deepcopy(
+            vector_store_settings["schema"]["fields"]
+        )
+
+        if datasource_type == "elasticsearch":
+            from ibm_watsonx_ai.foundation_models.extensions.rag.vector_stores import (
+                ElasticsearchVectorStore,
+                VectorStore,
+            )
+
+            distance_metric = vector_store_settings.get("distance_metric")
+            if distance_metric == "euclidean":
+                distance_metric = "EUCLIDEAN_DISTANCE"
+            else:
+                distance_metric = "COSINE"
+
+            if hybrid_ranker:
+                from ibm_watsonx_ai.foundation_models.extensions.rag.vector_stores import (
+                    HybridStrategyElasticsearch,
+                )
+
+                sparse_vector_embeddings = None
+                vector_embeddings = None
+
+                for field in schema_fields:
+                    if field.get("role") == "sparse_vector_embeddings":
+                        sparse_vector_embeddings = field.get("name")
+                    elif field.get("role") == "vector_embeddings":
+                        vector_embeddings = field.get("name")
+
+                retrieval_strategies: dict[str, Any] = {
+                    "dense": {"vector_field": vector_embeddings},
+                }
+
+                if strategy == "weighted":
+                    if not isinstance(alpha, (float, int)):
+                        raise WMLClientError(
+                            "Parameter `alpha` was not provided as a proper value and 'weighted' retrieval strategy couldn't be define."
+                        )
+                    retrieval_strategies["dense"]["boost"] = alpha
+
+                    if model_id == "bm25":
+                        retrieval_strategies["bm25"] = {
+                            "boost": (1 - alpha),
+                        }
+
+                    else:
+                        retrieval_strategies["sparse"] = {
+                            "model_id": model_id,
+                            "boost": (1 - alpha),
+                            "vector_field": sparse_vector_embeddings,
+                        }
+                    use_rrf = False
+                    rrf_params = None
+                else:
+
+                    if model_id == "bm25":
+                        retrieval_strategies["bm25"] = {}
+
+                    else:
+                        retrieval_strategies["sparse"] = {
+                            "model_id": model_id,
+                            "vector_field": sparse_vector_embeddings,
+                        }
+                    use_rrf = True
+                    rrf_params = {"k": k}
+
+                hybrid_strategy = HybridStrategyElasticsearch(
+                    retrieval_strategies=retrieval_strategies,
+                    use_rrf=use_rrf,
+                    rrf_params=rrf_params,
+                )
+                vector_store = ElasticsearchVectorStore(
+                    connection_id=connection_id,
+                    index_name=vector_store_settings.get("index_name"),
+                    strategy=hybrid_strategy,
+                    distance_strategy=distance_metric,
+                    embedding=embeddings,
+                    api_client=self._client,
+                )
+            else:
+                vector_store = VectorStore(
+                    connection_id=connection_id,
+                    datasource_type=datasource_type,
+                    index_name=vector_store_settings.get("index_name"),
+                    distance_metric=vector_store_settings.get("distance_metric"),
+                    embeddings=embeddings,
+                    api_client=self._client,
+                )
+        elif datasource_type in ("milvus", "milvuswxd"):
+            from ibm_watsonx_ai.foundation_models.extensions.rag.vector_stores import (
+                MilvusVectorStore,
+                VectorStore,
+            )
+
+            bm25_builtin_func = None
+            if hybrid_ranker:
+
+                if model_id == "bm25":
+                    from ibm_watsonx_ai.foundation_models.extensions.rag.vector_stores import (
+                        MilvusBM25BuiltinFunction,
+                    )
+
+                    sparse_vector_field = next(
+                        filter(
+                            lambda field: field.get("role")
+                            == "sparse_vector_embeddings",
+                            schema_fields,
+                        ),
+                        None,
+                    )
+
+                    bm25_builtin_func = MilvusBM25BuiltinFunction(
+                        output_field_names=(
+                            sparse_vector_field or {"name": "sparse"}
+                        ).get("name")
+                    )
+
+                if strategy == "weighted":
+                    if not isinstance(alpha, (float, int)):
+                        raise WMLClientError(
+                            "Parameter `alpha` was not provided as a proper value and 'weighted' retrieval strategy couldn't be define."
+                        )
+                    ranker_config = {"weighted": {"weights": [alpha, (1 - alpha)]}}
+                else:
+                    ranker_config = {"rrf": {"k": k}}
+
+                vector_field = []
+                for field in schema_fields:
+                    if field["role"] in (
+                        "vector_embeddings",
+                        "sparse_vector_embeddings",
+                    ):
+                        vector_field.append(field["name"])
+
+                vector_store = MilvusVectorStore(
+                    api_client=self._client,
+                    connection_id=connection_id,
+                    collection_name=vector_store_settings.get("index_name"),
+                    embedding_function=embeddings,
+                    vector_field=vector_field,
+                    builtin_function=bm25_builtin_func,
+                )
+            else:
+                vector_store = VectorStore(
+                    connection_id=connection_id,
+                    datasource_type=datasource_type,
+                    index_name=vector_store_settings.get("index_name"),
+                    distance_metric=vector_store_settings.get("distance_metric"),
+                    embeddings=embeddings,
+                    api_client=self._client,
+                )
+
+        else:
+            from ibm_watsonx_ai.foundation_models.extensions.rag import VectorStore
+
+            vector_store = VectorStore(
+                connection_id=connection_id,
+                datasource_type=datasource_type,
+                index_name=vector_store_settings.get("index_name"),
+                distance_metric=vector_store_settings.get("distance_metric"),
+                embeddings=embeddings,
+                api_client=self._client,
+            )
+
         from ibm_watsonx_ai.foundation_models.extensions.rag import Retriever
 
         retriever = Retriever(
@@ -493,13 +703,26 @@ class RAGEngine(WMLResource):
         generation_settings = pattern_details["context"]["rag_pattern"]["settings"][
             "generation"
         ]
-        model = ModelInference(
-            model_id=generation_settings["model_id"],
-            params=generation_settings["parameters"],
-            project_id=details["metadata"].get("project_id"),
-            space_id=details["metadata"].get("space_id"),
-            api_client=self._client,
+        default_max_sequence_length = generation_settings["parameters"].pop(
+            "max_sequence_length", None
         )
+        if model_id := generation_settings.get("model_id"):
+            model = ModelInference(
+                model_id=model_id,
+                params=generation_settings["parameters"],
+                project_id=details["metadata"].get("project_id"),
+                space_id=details["metadata"].get("space_id"),
+                api_client=self._client,
+            )
+        else:
+            model = ModelInference(
+                deployment_id=generation_settings["deployment_id"],
+                params=generation_settings["parameters"],
+                project_id=details["metadata"].get("project_id"),
+                space_id=details["metadata"].get("space_id"),
+                api_client=self._client,
+                validate=False,
+            )
 
         from ibm_watsonx_ai.foundation_models.extensions.rag.chunker.langchain_chunker import (
             LangChainChunker,
@@ -510,17 +733,27 @@ class RAGEngine(WMLResource):
         ]
         chunker = LangChainChunker(**chunking_settings)
 
-        from ibm_watsonx_ai.foundation_models.extensions.rag.pattern import RAGPattern
+        data_connections = None
+        if vector_store._datasource_type == "chroma" and (
+            input_data_references := details["entity"]["input_data_references"]
+        ):
+            data_connections = [
+                DataConnection.from_dict(data_reference)
+                for data_reference in input_data_references
+            ]
 
         pattern = RAGPattern(
             retriever=retriever,
             model=model,
-            prompt_template_text=generation_settings["prompt_template_text"],
-            context_template_text=generation_settings["context_template_text"],
+            prompt_template_text=generation_settings.get("prompt_template_text"),
+            context_template_text=generation_settings.get("context_template_text"),
             space_id=details["metadata"].get("space_id"),
             project_id=details["metadata"].get("project_id"),
             api_client=self._client,
             chunker=chunker,
+            input_data_references=data_connections,
+            ranker_config=ranker_config,
+            default_max_sequence_length=default_max_sequence_length,
         )
 
         return pattern
@@ -724,7 +957,7 @@ class RAGEngine(WMLResource):
                 ]
 
             pattern_details = None
-            metrics_best_score = 0
+            metrics_best_score = -1  # to avoid skipping `0.0` metric score
             for pattern in results:
                 for test_data in pattern["metrics"]["test_data"]:
                     if (
@@ -835,7 +1068,9 @@ class RAGEngine(WMLResource):
             "embedding_models": params.get("embedding_models"),
             "retrieval_methods": params.get("retrieval_methods"),
             "foundation_models": params.get("foundation_models"),
+            "generation": params.get("generation"),
             "max_number_of_rag_patterns": params.get("max_number_of_rag_patterns"),
+            "retrieval": params.get("retrieval"),
         }
 
         # note: For CPD 5.1, only chunking_methods = ["recursive"] (default) was supported.
@@ -883,8 +1118,8 @@ class RAGEngine(WMLResource):
     def _initialize_training_metadata(
         self,
         input_data_references: list[DataConnection],
-        test_data_references: list[DataConnection],
         results_reference: DataConnection,
+        test_data_references: list[DataConnection] | None = None,
         vector_store_references: list[DataConnection] | None = None,
     ) -> None:
         """Initialization of training metadata.
@@ -892,12 +1127,12 @@ class RAGEngine(WMLResource):
         :param input_data_references: Data storage connection details to inform where training data is stored
         :type input_data_references: list[DataConnection]
 
-        :param test_data_references: A set of test data references
-        :type test_data_references: list[DataConnection]
-
         :param results_reference: The training results
         :type results_reference: DataConnection
 
+        :param test_data_references: A set of test data references
+        :type test_data_references: list[DataConnection], optional
+
         :param vector_store_references: A set of vector store references
         :type vector_store_references: list[DataConnection], optional
 
@@ -907,14 +1142,15 @@ class RAGEngine(WMLResource):
             connection._to_dict() for connection in input_data_references
         ]
 
-        self._training_metadata[self.ConfigurationMetaNames.TEST_DATA_REFERENCES] = [
-            connection._to_dict() for connection in test_data_references
-        ]
-
         self._training_metadata[self.ConfigurationMetaNames.RESULTS_REFERENCE] = (
             results_reference._to_dict()
         )
 
+        if test_data_references is not None:
+            self._training_metadata[
+                self.ConfigurationMetaNames.TEST_DATA_REFERENCES
+            ] = [connection._to_dict() for connection in test_data_references]
+
         hardware_specifications_name = "L"  # Added as a default
         hardware_specifications_id = (
             self._client.hardware_specifications.get_id_by_name(
@@ -927,7 +1163,7 @@ class RAGEngine(WMLResource):
             "name": hardware_specifications_name,
         }
 
-        if vector_store_references:
+        if vector_store_references is not None:
             self._training_metadata[
                 self.ConfigurationMetaNames.VECTOR_STORE_REFERENCES
             ] = [connection._to_dict() for connection in vector_store_references]
@@ -947,7 +1183,7 @@ class RAGEngine(WMLResource):
 
     def _is_autoai_rag_endpoint_available(self) -> bool:
         try:
-            url = self._client.service_instance._href_definitions.get_autoai_rag_href()
+            url = self._client._href_definitions.get_autoai_rag_href()
 
             response_autoai_rag_api = self._client._session.get(
                 url=f"{url}?limit=1",
@@ -957,3 +1193,84 @@ class RAGEngine(WMLResource):
             return response_autoai_rag_api.status_code != 404
         except:
             return False
+
+    def _get_inference_service(
+        self, details: dict, pattern_name: str | None = None
+    ) -> str | None:
+
+        code_connection = self._determine_connection(
+            details=details,
+            location_key="inference_service_code",
+            pattern_name=pattern_name,
+        )
+
+        if not code_connection:
+            return None
+
+        # return to fallback mode for older CPD versions
+        if self._client.CPD_version < 5.3:
+            return None
+
+        code_compressed = bytes(code_connection.read(binary=True))
+
+        with gzip.GzipFile(fileobj=io.BytesIO(code_compressed)) as gz_file:
+            source_code = gz_file.read()
+
+        service_code = source_code.decode()
+        return service_code
+
+    def _get_service_metadata(
+        self, details: dict, pattern_name: str | None = None
+    ) -> dict | None:
+
+        connection = self._determine_connection(
+            details=details,
+            location_key="inference_service_metadata",
+            pattern_name=pattern_name,
+        )
+
+        if not connection:
+            return None
+
+        metadata_binary = bytes(connection.read(binary=True))
+
+        return json.loads(metadata_binary.decode("utf-8"))
+
+    def _determine_connection(
+        self,
+        details: dict,
+        location_key: str,
+        pattern_name: str | None = None,
+    ) -> DataConnection | None:
+
+        pattern_details = self.get_best_pattern_details(
+            details=details, pattern_name=pattern_name
+        )
+
+        file_location = (
+            pattern_details.get("context", {})
+            .get("rag_pattern", {})
+            .get("location", {})
+            .get(location_key)
+        )
+
+        if not file_location:  # Older CPD versions may not contain `location_key`
+            self._logger.debug(
+                f"Could not find specific location for `{location_key}`."
+            )
+            return None
+
+        results_ref = details["entity"]["results_reference"]
+
+        data_connection = DataConnection.from_dict(results_ref)
+
+        if data_connection.location is not None:
+            if hasattr(data_connection.location, "file_name"):
+                setattr(data_connection.location, "file_name", file_location)
+
+            if hasattr(data_connection.location, "path"):
+                setattr(data_connection.location, "path", file_location)
+
+        data_connection.set_client(self._client)
+
+        return data_connection
