@@ -75,6 +75,7 @@ class ServiceEngine(BaseEngine):
         self._stored_pipeline_details = None
         self._current_run_id = None
         self._20_class_limit_removal_test = False
+        self._onnx_model = None
 
     def _get_node_id(self):
         result = "automl"
@@ -1057,9 +1058,7 @@ class ServiceEngine(BaseEngine):
             pipeline_details = self.get_params()
             number_of_estimators = pipeline_details.get("max_num_daub_ensembles", 2)
 
-            base_url = (
-                self._api_client.service_instance._href_definitions.get_trainings_href()
-            )
+            base_url = self._api_client._href_definitions.get_trainings_href()
             url = f"{base_url.replace('https', 'wss')}/{self._current_run_id}"
 
             params = self._api_client._params()
@@ -1680,15 +1679,6 @@ class ServiceEngine(BaseEngine):
             training_id=self._current_run_id, _internal=True
         )
 
-        # note: recreation of s3 creds from connection asset
-        results_reference = DataConnection._from_dict(
-            run_params["entity"]["results_reference"]
-        )
-        results_reference._api_client = self._api_client
-        results_reference._check_if_connection_asset_is_s3()
-        run_params["entity"]["results_reference"] = results_reference._to_dict()
-        # --- end note
-
         if "timeseries" in run_params["entity"]["status"]["metrics"][0]["context"]:
             try_import_autoai_ts_libs()
         else:
@@ -1703,6 +1693,7 @@ class ServiceEngine(BaseEngine):
                 store=persist,
                 api_client=self._api_client,
                 auto_pipelines_parameters=self._auto_pipelines_parameters,
+                _onnx_model=self._onnx_model,
             )
 
         except LibraryNotCompatible as e:
