@@ -84,6 +84,7 @@ class WMLEngine(BaseEngine):
         self._wml_stored_pipeline_details = None
         self._current_run_id = None
         self._20_class_limit_removal_test = False
+        self._onnx_model = None
 
     def _get_node_id(self):
         result = "automl"
@@ -1061,9 +1062,7 @@ class WMLEngine(BaseEngine):
             wml_pipeline_details = self.get_params()
             number_of_estimators = wml_pipeline_details.get("max_num_daub_ensembles", 2)
 
-            base_url = (
-                self._wml_client.service_instance._href_definitions.get_trainings_href()
-            )
+            base_url = self._wml_client._href_definitions.get_trainings_href()
             url = f"{base_url.replace('https', 'wss')}/{self._current_run_id}"
 
             params = self._wml_client._params()
@@ -1716,6 +1715,7 @@ class WMLEngine(BaseEngine):
                 store=persist,
                 api_client=self._wml_client,
                 auto_pipelines_parameters=self._auto_pipelines_parameters,
+                _onnx_model=self._onnx_model,
             )
 
         except LibraryNotCompatible as e:
