@@ -15,10 +15,18 @@ from ibm_watsonx_ai.helpers.connections import (
     ContainerLocation,
     DataConnection,
 )
+from ibm_watsonx_ai.wml_client_error import WMLClientError
+from ibm_watsonx_ai.wml_resource import WMLResource
+from ibm_watsonx_ai.foundation_models.schema import BaseSchema
 
 if TYPE_CHECKING:
     from ibm_watsonx_ai.experiment.autoai.engines import RAGEngine
     from ibm_watsonx_ai.foundation_models.extensions.rag.pattern import RAGPattern
+    from ibm_watsonx_ai.foundation_models.schema import (
+        AutoAIRAGModelConfig,
+        AutoAIRAGCustomModelConfig,
+        AutoAIRAGRetrievalConfig,
+    )
 
 __all__ = ["RAGOptimizer"]
 
@@ -41,8 +49,8 @@ class RAGOptimizer:
     :param retrieval_methods: Retrieval methods to be used.
     :type retrieval_methods: list[str], optional
 
-    :param foundation_models: The foundation models to try.
-    :type foundation_models: list[str], optional
+    :param foundation_models: List of foundation models to try. Custom foundation models and model config are also supported for Cloud and CPD >= 5.2.
+    :type foundation_models: list[str | dict | AutoAIRAGModelConfig | AutoAIRAGCustomModelConfig], optional
 
     :param max_number_of_rag_patterns: The maximum number of RAG patterns to create.
     :type max_number_of_rag_patterns: int, optional
@@ -50,6 +58,12 @@ class RAGOptimizer:
     :param optimization_metrics: The metric name(s) to be used for optimization.
     :type optimization_metrics: list[str], optional
 
+    :param generation: Properties describing the generation step.
+    :type generation: dict[str, Any], optional
+
+    :param retrieval: Retrieval settings to be used.
+    :type retrieval: list[dict[str, Any] | AutoAIRAGRetrievalConfig], optional
+
     """
 
     def __init__(
@@ -60,10 +74,14 @@ class RAGOptimizer:
         chunking_methods: list[str] | None = None,
         embedding_models: list[str] | None = None,
         retrieval_methods: list[str] | None = None,
-        foundation_models: list[str] | None = None,
+        foundation_models: (
+            list[str | dict | AutoAIRAGModelConfig | AutoAIRAGCustomModelConfig] | None
+        ) = None,
         max_number_of_rag_patterns: int | None = None,
         optimization_metrics: list[str] | None = None,
         chunking: list[dict] | None = None,
+        generation: dict[str, Any] | None = None,
+        retrieval: list[dict[str, Any] | AutoAIRAGRetrievalConfig] | None = None,
         **kwargs: dict[str, Any],
     ):
         self._engine = engine
@@ -72,16 +90,70 @@ class RAGOptimizer:
             chunking_methods_deprecated_warning = "The parameter chunking_methods is deprecated, please use `chunking` instead"
             warn(chunking_methods_deprecated_warning, category=DeprecationWarning)
 
-        self._params = {
-            "name": name,
-            "description": description,
-            "chunking": chunking,
-            "embedding_models": embedding_models,
-            "retrieval_methods": retrieval_methods,
-            "foundation_models": foundation_models,
-            "max_number_of_rag_patterns": max_number_of_rag_patterns,
-            "optimization_metrics": optimization_metrics,
-        }
+        WMLResource._validate_type(
+            foundation_models, "foundation_models", list, mandatory=False
+        )
+        WMLResource._validate_type(retrieval, "retrieval", list, mandatory=False)
+
+        self._params: dict[str, Any] = {}
+
+        if foundation_models is not None:
+            if self._engine._client.CPD_version <= 5.1:
+                if any(not isinstance(fm, str) for fm in foundation_models):
+                    raise WMLClientError(
+                        "Parameter `foundation_models` must be a list of string for CPD 5.1 or below."
+                    )
+                self._params["foundation_models"] = foundation_models
+            else:
+                foundation_models_conv = []
+                for fm in foundation_models:
+                    if isinstance(fm, BaseSchema):
+                        foundation_models_conv.append(fm.to_dict())
+                    elif isinstance(fm, dict):
+                        foundation_models_conv.append(fm)
+                    elif isinstance(fm, str):
+                        foundation_models_conv.append({"model_id": fm})
+                    else:
+                        raise WMLClientError(
+                            f"Invalid item type '{type(fm)}' provided in `foundation_models` list."
+                        )
+
+                if isinstance(generation, dict):
+                    generation["foundation_models"] = foundation_models_conv
+                else:
+                    generation = {"foundation_models": foundation_models_conv}
+
+        if retrieval is not None:
+            if self._engine._client.CPD_version <= 5.1:
+                if any(not isinstance(r, dict) for r in retrieval):
+                    raise WMLClientError(
+                        "Parameter `retrieval` must be a list of 'dict' for CPD 5.1 or below."
+                    )
+            else:
+                for i in range(len(retrieval)):
+                    if isinstance(retrieval[i], BaseSchema):
+                        retrieval[i] = retrieval[i].to_dict()  # type: ignore[union-attr]
+                    elif isinstance(retrieval[i], dict):
+                        pass
+                    else:
+                        raise WMLClientError(
+                            f"Invalid item type '{type(retrieval[i])}' provided in `retrieval` list."
+                        )
+
+        self._params.update(
+            {
+                "name": name,
+                "description": description,
+                "chunking": chunking,
+                "embedding_models": embedding_models,
+                "retrieval_methods": retrieval_methods,
+                "retrieval": retrieval,
+                "generation": generation,
+                "max_number_of_rag_patterns": max_number_of_rag_patterns,
+                "optimization_metrics": optimization_metrics,
+            }
+        )
+
         self._engine.initiate_optimizer_metadata(self._params, **kwargs)
         self._engine._params = self._params
 
@@ -123,7 +195,7 @@ class RAGOptimizer:
     def run(
         self,
         input_data_references: list[DataConnection],
-        test_data_references: list[DataConnection],
+        test_data_references: list[DataConnection] | None = None,
         results_reference: DataConnection | None = None,
         vector_store_references: list[DataConnection] | None = None,
         background_mode: bool = True,
@@ -134,7 +206,7 @@ class RAGOptimizer:
         :type input_data_references: list[DataConnection]
 
         :param test_data_references: A set of test data references
-        :type test_data_references: list[DataConnection]
+        :type test_data_references: list[DataConnection], optional
 
         :param results_reference: The training results
         :type results_reference: DataConnection, optional
@@ -186,8 +258,8 @@ class RAGOptimizer:
 
         return self._engine.run(
             input_data_references=input_data_references,
-            test_data_references=test_data_references,
             results_reference=results_reference,
+            test_data_references=test_data_references,
             vector_store_references=vector_store_references,
             background_mode=background_mode,
         )
