@@ -5,8 +5,7 @@
 from __future__ import annotations
 from enum import Enum, EnumMeta
 from typing import TYPE_CHECKING, TypeAlias
-from warnings import catch_warnings, simplefilter
-
+from warnings import catch_warnings, simplefilter, warn
 
 from ibm_watsonx_ai.foundation_models.prompt_tuner import PromptTuner
 from ibm_watsonx_ai.foundation_models.fine_tuner import FineTuner
@@ -130,6 +129,9 @@ class TuneExperiment(BaseExperiment):
     ) -> PromptTuner:
         """Initialize a PromptTuner module.
 
+        .. note::
+            Prompt Tuning is deprecated for IBM Cloud Pak® for Data since 5.2 version and will be removed in a future release.
+
         :param name: name for the PromptTuner
         :type name: str
 
@@ -225,6 +227,11 @@ class TuneExperiment(BaseExperiment):
                 verbalizer="Extract the satisfaction from the comment. Return simple '1' for satisfied customer or '0' for unsatisfied. Input: {{input}} Output: ",
                 auto_update_model=True)
         """
+        if not self.client.CLOUD_PLATFORM_SPACES and self.client.CPD_version >= 5.2:
+            with catch_warnings():
+                simplefilter("default", category=DeprecationWarning)
+                prompt_tuning_warn = "Prompt Tuning is deprecated and will be removed in a future release."
+                warn(prompt_tuning_warn, category=DeprecationWarning)
 
         if isinstance(task_id, Enum):
             task_id = task_id.value
@@ -295,6 +302,7 @@ class TuneExperiment(BaseExperiment):
         auto_update_model: bool = True,
         group_by_name: bool = False,
         peft_parameters: dict | PeftParameters | None = None,
+        gradient_checkpointing: bool | None = None,
     ) -> FineTuner:
         """Initialize a FineTuner module.
 
@@ -351,6 +359,9 @@ class TuneExperiment(BaseExperiment):
         :param peft_parameters: parameters to be set when running a Fine Tuning job with LoRA/QLoRA
         :type peft_parameters: dict, PeftParameters, optional
 
+        :param gradient_checkpointing: reduces GPU memory required at the cost of slowing training by approx 20%
+        :type gradient_checkpointing: bool, optional
+
         **Examples**
 
         Example of init FineTuner
@@ -372,7 +383,8 @@ class TuneExperiment(BaseExperiment):
                 accumulate_steps=5,
                 verbalizer="### Input: {{input}} \\n\\n### Response: {{output}}",
                 response_template="\\n### Response:\\n",
-                auto_update_model=False)
+                auto_update_model=False,
+                gradient_checkpointing=True)
 
         Example of init FineTuner with LoRA/QLoRA
 
@@ -401,7 +413,8 @@ class TuneExperiment(BaseExperiment):
                     lora_dropout=0.05,
                     target_modules=['all-linear'],
                 ),
-                auto_update_model=False)
+                auto_update_model=False,
+                gradient_checkpointing=False)
         """
 
         if isinstance(task_id, Enum):
@@ -427,6 +440,7 @@ class TuneExperiment(BaseExperiment):
             group_by_name=group_by_name,
             api_client=self.client,
             peft_parameters=peft_parameters,
+            gradient_checkpointing=gradient_checkpointing,
         )
 
         return fine_tuner
