@@ -6,6 +6,8 @@
 import inspect
 import re
 import ast
+import uuid
+import gzip
 from enum import Enum
 from functools import cache
 from typing import Any, Callable, cast
@@ -267,6 +269,8 @@ class RAGPatternService(BaseRAGPatternService):
         _allow_store: bool = True,
     ) -> None:
 
+        self._code: str | None = None
+
         super().__init__(
             api_client=api_client,
             function=ai_service,
@@ -280,8 +284,6 @@ class RAGPatternService(BaseRAGPatternService):
             self.cached_function = cache(self.ai_service)
             self.cached_function(RuntimeContext(api_client=api_client))
 
-        self._source_file: tempfile._TemporaryFileWrapper | None
-
     @property
     def ai_service(self) -> Callable:
         return self._function
@@ -307,6 +309,37 @@ class RAGPatternService(BaseRAGPatternService):
         :rtype: Callable
         """
         if default_params:
+            args_spec = inspect.getfullargspec(function)
+            defaults: tuple | list = args_spec.defaults or []
+            args = args_spec.args or []
+
+            if args and args[-1] == "vector_store_settings":
+                vectorstore_params = default_params.get("vector_store") or {}
+                vector_store_settings = {
+                    "connection_id": vectorstore_params.get("connection_id"),
+                }
+                vs_datasource_type = vectorstore_params.get("datasource_type")
+                if isinstance(vs_datasource_type, str):
+                    if vs_datasource_type.startswith("milvus"):
+                        vector_store_settings["collection_name"] = (
+                            vectorstore_params.get("collection_name")
+                        )
+                    elif "elasticsearch" in vs_datasource_type:
+                        vector_store_settings["index_name"] = vectorstore_params.get(
+                            "index_name"
+                        )
+
+                if space_id := default_params.get("space_id"):
+                    vector_store_settings["space_id"] = space_id
+                else:
+                    vector_store_settings["project_id"] = default_params.get(
+                        "project_id"
+                    )
+                function.__defaults__ = (
+                    *defaults[:-1],
+                    vector_store_settings,
+                )
+
             source = AIServices._populate_default_params(function)
 
             tree = ast.parse(source)
@@ -322,7 +355,7 @@ class RAGPatternService(BaseRAGPatternService):
                 suffix="credentials",
             )
 
-            # APIClient params
+            # APIClient params, for Chroma scenario
             api_client_params = _get_components_replace_data(
                 {
                     "space_id": default_params.get("space_id"),
@@ -367,6 +400,50 @@ class RAGPatternService(BaseRAGPatternService):
                 embeddings_init_params.pop("space_id", None)
                 embeddings_init_params.pop("verify", None)
 
+            # Remove creds from wx embeddings in Elasticsearch
+            if (
+                vector_store_init_params is not None
+                and "ibm_watsonx_ai.foundation_models.embeddings.embeddings"
+                in (
+                    embeddings_init_params := (
+                        vector_store_init_params.get("embedding") or {}
+                    )
+                ).get("__module__", "")
+            ):
+                embeddings_init_params.pop("credentials", None)
+                embeddings_init_params.pop("project_id", None)
+                embeddings_init_params.pop("space_id", None)
+                embeddings_init_params.pop("verify", None)
+
+            # Remove creds from wx embeddings in Milvus
+            if (
+                vector_store_init_params is not None
+                and "embedding_function" in vector_store_init_params
+            ):
+                if (
+                    embedding_function := vector_store_init_params["embedding_function"]
+                ) is not None:
+                    if isinstance(embedding_function, list):
+                        for embeddings_init_params in embedding_function:
+                            if (
+                                "ibm_watsonx_ai.foundation_models.embeddings.embeddings"
+                                in (embeddings_init_params.get("__module__", ""))
+                            ):
+                                embeddings_init_params.pop("credentials", None)
+                                embeddings_init_params.pop("project_id", None)
+                                embeddings_init_params.pop("space_id", None)
+                                embeddings_init_params.pop("verify", None)
+                    else:
+                        embeddings_init_params = embedding_function
+                        if (
+                            "ibm_watsonx_ai.foundation_models.embeddings.embeddings"
+                            in (embeddings_init_params.get("__module__", ""))
+                        ):
+                            embeddings_init_params.pop("credentials", None)
+                            embeddings_init_params.pop("project_id", None)
+                            embeddings_init_params.pop("space_id", None)
+                            embeddings_init_params.pop("verify", None)
+
             replace_data = dict(
                 # For AST Call node
                 **credentials_params,
@@ -390,10 +467,6 @@ class RAGPatternService(BaseRAGPatternService):
                     "replace": True,
                 },
                 **new_chunker_init_params,
-                REPLACE_THIS_CODE_WITH_VECTOR_STORE={
-                    "value": vector_store_init_params,
-                    "replace": True,
-                },
                 # For AST Assign Node
                 REPLACE_THIS_CODE_WITH_WORD_TO_TOKEN_RATIO=default_params.get(
                     "word_to_token_ratio"
@@ -401,31 +474,75 @@ class RAGPatternService(BaseRAGPatternService):
                 REPLACE_THIS_CODE_WITH_INPUT_DATA_REFERENCES=default_params.get(
                     "input_data_references"
                 ),
+                REPLACE_THIS_CODE_WITH_VECTOR_STORE_ASSIGN=vector_store_init_params,
+                REPLACE_THIS_CODE_WITH_VECTOR_STORE_CALL={
+                    "value": vector_store_init_params,
+                    "replace": True,
+                },
             )
 
+            if ranker_config := default_params.get("ranker_config"):
+                ranker_type = next(iter(ranker_config.keys()))
+                ranker_params = next(iter(ranker_config.values()))
+                replace_data[
+                    "REPLACE_THIS_CODE_WITH_VECTOR_STORE_MILVUS_RANKER_TYPE"
+                ] = {
+                    "value": ranker_type,
+                    "replace": True,
+                }
+                replace_data[
+                    "REPLACE_THIS_CODE_WITH_VECTOR_STORE_MILVUS_RANKER_PARAMS"
+                ] = {
+                    "value": ranker_params,
+                    "replace": True,
+                }
+            else:
+                replace_data[
+                    "REPLACE_THIS_CODE_WITH_VECTOR_STORE_MILVUS_RANKER_TYPE"
+                ] = {
+                    "value": None,
+                    "replace": False,
+                }
+                replace_data[
+                    "REPLACE_THIS_CODE_WITH_VECTOR_STORE_MILVUS_RANKER_PARAMS"
+                ] = {
+                    "value": None,
+                    "replace": False,
+                }
+
             replacer = FunctionTransformer(
                 cast(ast.FunctionDef, func_def), **replace_data
             )
             new_tree = replacer.visit(tree)
             ast.fix_missing_locations(new_tree)
 
-            code = ast.unparse(new_tree)
+            self._code = ast.unparse(new_tree)
 
             if is_lib_installed("black"):
                 import black
 
-                code = black.format_str(code, mode=black.FileMode()).rstrip()
+                # If default values in inference service make the line too long
+                # use shorter signature
+                code_lines = self._code.split("\n")
+                tmp_signature = "def default_service(context):"
+                formatted_code = black.format_str(
+                    tmp_signature + "\n" + "\n".join(code_lines[1:]),
+                    mode=black.FileMode(),
+                ).rstrip()
+
+                self._code = (
+                    code_lines[0] + "\n" + "\n".join(formatted_code.split("\n")[1:])
+                )
 
             with tempfile.NamedTemporaryFile(
-                suffix="inference_service.py", delete=False
+                suffix="inference_service.py", delete=True
             ) as tmp_file:
-                tmp_file.write(code.encode())
+                tmp_file.write(self._code.encode())
 
-            self._source_file = tmp_file
-            compiled_code = compile(new_tree, filename=tmp_file.name, mode="exec")
-            namespace: dict = {}
-            exec(compiled_code, namespace)
-            function = namespace[function.__name__]
+                compiled_code = compile(new_tree, filename=tmp_file.name, mode="exec")
+                namespace: dict = {}
+                exec(compiled_code, namespace)
+                function = namespace[function.__name__]
         return function
 
     def _store(self, meta_props: dict) -> dict:
@@ -437,9 +554,25 @@ class RAGPatternService(BaseRAGPatternService):
         :return: details of the stored AI service
         :rtype: dict
         """
-        function_details = self.api_client.repository.store_ai_service(
-            ai_service=self.ai_service, meta_props=meta_props
-        )
+        if self._code is not None:
+
+            tmp_uid = "tmp_ai_service_python_function_code_{}.py.gz".format(
+                str(uuid.uuid4()).replace("-", "_")
+            )
+            with tempfile.NamedTemporaryFile(
+                suffix=tmp_uid, delete=True
+            ) as archive_name:
+                with gzip.GzipFile(mode="wb", fileobj=archive_name) as gzip_file:
+                    gzip_file.write(self._code.encode())
+                archive_name.seek(0)
+                function_details = self.api_client.repository.store_ai_service(
+                    ai_service=archive_name.name, meta_props=meta_props
+                )
+
+        else:
+            function_details = self.api_client.repository.store_ai_service(
+                ai_service=self.ai_service, meta_props=meta_props
+            )
 
         self.ai_service_id = self.api_client.repository.get_ai_service_id(
             function_details
@@ -454,7 +587,10 @@ class RAGPatternService(BaseRAGPatternService):
         :type insert_to_cell: bool, optional
         """
 
-        code = AIServices._populate_default_params(self.ai_service)
+        if self._code is not None:
+            code = self._code
+        else:
+            code = AIServices._populate_default_params(self.ai_service)
 
         if insert_to_cell:
             from IPython.core.getipython import get_ipython
@@ -516,6 +652,25 @@ class RAGPatternService(BaseRAGPatternService):
                 deploy_params={"description": "Optional deployed AI service description"}
             )
 
+        To override vector store `connection_id`, `index_name` or set specific scope id (project_id/space_id) use following:
+        .. code-block:: python
+
+            deployment_details = pattern.inference_service.deploy(
+                name="Example deployment name",
+                store_params={"software_spec_id": "<ID of the custom sw spec>"},
+                deploy_params={
+                                "online": {
+                                    "parameters": {
+                                        "vector_store_settings": {
+                                            "connection_id": "<connection_to_vector_store>",
+                                            "index_name": "<index_name>",
+                                            "project_id": "<project_id>",
+                                        }
+                                    }
+                                }
+                            }
+            )
+
         """
 
         if not space_id and not self.api_client.default_space_id:
@@ -533,7 +688,7 @@ class RAGPatternService(BaseRAGPatternService):
 
 
 class RAGPatternFunction(BaseRAGPatternService):
-    """Initialize ``RAGPatternSFunction`` object.
+    """Initialize ``RAGPatternFunction`` object.
 
     :param api_client: initialized APIClient object
     :type api_client: APIClient
