@@ -3,6 +3,10 @@
 #  https://opensource.org/licenses/BSD-3-Clause
 #  -----------------------------------------------------------------------------------------
 from collections import defaultdict
+import hashlib
+from typing import Any, cast, Generic, TypeVar
+import logging
+
 from ibm_watsonx_ai.wml_client_error import MissingExtension, MissingMetadata
 from ibm_watsonx_ai.wml_resource import WMLResource
 
@@ -15,10 +19,6 @@ try:
 except ImportError:
     raise MissingExtension("langchain")
 
-import hashlib
-from typing import Any, cast
-import logging
-
 from ibm_watsonx_ai.foundation_models.embeddings import BaseEmbeddings
 from ibm_watsonx_ai.foundation_models.extensions.rag.utils.utils import verbose_search
 from ibm_watsonx_ai.foundation_models.extensions.rag.vector_stores.base_vector_store import (
@@ -27,6 +27,8 @@ from ibm_watsonx_ai.foundation_models.extensions.rag.vector_stores.base_vector_s
 
 logger = logging.getLogger(__name__)
 
+T = TypeVar("T", bound=LangChainVectorStore)
+
 
 def merge_window_into_a_document(window: list[Document]) -> Document:
     """
@@ -122,7 +124,7 @@ def merge_window_into_a_document(window: list[Document]) -> Document:
     return Document(page_content=merged_text, metadata=merged_metadata)
 
 
-class LangChainVectorStoreAdapter(BaseVectorStore):
+class LangChainVectorStoreAdapter(Generic[T], BaseVectorStore):
     """
     Adapter for LangChain ``VectorStore`` base class.
 
@@ -130,14 +132,14 @@ class LangChainVectorStoreAdapter(BaseVectorStore):
     :type vector_store: langchain_core.vectorstore.VectorStore
     """
 
-    def __init__(self, vector_store: LangChainVectorStore) -> None:
+    def __init__(self, vector_store: T) -> None:
         super().__init__()
-        self._langchain_vector_store: LangChainVectorStore = vector_store
+        self._langchain_vector_store: T = vector_store
 
-    def get_client(self) -> Any:
+    def get_client(self) -> T:
         return self._langchain_vector_store
 
-    def set_embeddings(self, embedding_fn: BaseEmbeddings) -> None:
+    def _set_embeddings(self, embedding_fn: BaseEmbeddings) -> None:
         if hasattr(self._langchain_vector_store, "embedding"):
             self._langchain_vector_store.embedding = embedding_fn
         elif hasattr(self._langchain_vector_store, "_embedding"):
@@ -152,14 +154,32 @@ class LangChainVectorStoreAdapter(BaseVectorStore):
             )
 
     def add_documents(
-        self, content: list[str] | list[dict] | list, **kwargs: Any
+        self, content: list[str] | list[dict] | list[Document], **kwargs: Any
     ) -> list[str]:
+        """
+        Embed documents and add to the vectorstore.
+
+        :param content: Documents to add to the vectorstore.
+        :type content: list[str] | list[dict] | list[langchain_core.documents.Document]
+
+        :return: List of IDs of the added texts.
+        :rtype: list[str]
+        """
         ids, docs = self._process_documents(content)
         return self._langchain_vector_store.add_documents(docs, ids=ids, **kwargs)
 
     async def add_documents_async(
         self, content: list[str] | list[dict] | list, **kwargs: Any
     ) -> list[str]:
+        """
+        Embed documents and add to the vectorstore in asynchronous manner.
+
+        :param content: Documents to add to the vectorstore.
+        :type content: list[str] | list[dict] | list[langchain_core.documents.Document]
+
+        :return: List of IDs of the added texts.
+        :rtype: list[str]
+        """
         ids, docs = self._process_documents(content)
         return await self._langchain_vector_store.aadd_documents(
             docs, ids=ids, **kwargs
@@ -173,6 +193,27 @@ class LangChainVectorStoreAdapter(BaseVectorStore):
         verbose: bool = False,
         **kwargs: Any,
     ) -> list[Document] | list[tuple[Document, float]]:
+        """Searches for documents most similar to the query.
+
+        The method is designed as a wrapper for respective LangChain VectorStores' similarity search methods.
+        Therefore, additional search parameters passed in ``kwargs`` should be consistent with those methods,
+        and can be found in the LangChain documentation.
+
+        :param query: text query
+        :type query: str
+
+        :param k: number of documents to retrieve
+        :type k: int
+
+        :param include_scores: whether similarity scores of found documents should be returned, defaults to False
+        :type include_scores: bool
+
+        :param verbose: whether to display a table with the found documents, defaults to False
+        :type verbose: bool
+
+        :return: list of found documents
+        :rtype: list
+        """
         result: list[Document] | list[tuple[Document, float]]
         if include_scores:
             result = self._langchain_vector_store.similarity_search_with_score(
@@ -196,7 +237,30 @@ class LangChainVectorStoreAdapter(BaseVectorStore):
         window_size: int = 2,
         **kwargs: Any,
     ) -> list:
+        """Searches for documents most similar to the query and extend a document (a chunk) to its adjacent chunks (if they exist) from the same origin document.
 
+        The method is designed as a wrapper for respective LangChain VectorStores' similarity search methods.
+        Therefore, additional search parameters passed in ``kwargs`` should be consistent with those methods,
+        and can be found in the LangChain documentation.
+
+        :param query: text query
+        :type query: str
+
+        :param k: number of documents to retrieve
+        :type k: int
+
+        :param include_scores: whether similarity scores of found documents should be returned, defaults to False
+        :type include_scores: bool
+
+        :param verbose: whether to display a table with the found documents, defaults to False
+        :type verbose: bool
+
+        :param window_size: number of chunks
+        :type window_size: int, optional
+
+        :return: list of found documents
+        :rtype: list
+        """
         documents = self.search(query, k, include_scores, verbose, **kwargs)
         if window_size <= 0:
             return documents
@@ -338,7 +402,66 @@ class LangChainVectorStoreAdapter(BaseVectorStore):
 
         return window_documents
 
+    def _get_window_documents_db2(
+        self, doc_id: str, seq_nums_window: list[int]
+    ) -> list[Document]:
+        """
+        Receives a document ID and a list of chunks' sequence_numbers,
+        and searches the vector store according to the metadata.
+
+        :param doc_id: ID of document
+        :type doc_id: str
+
+        :param seq_nums_window: list of sequence numbers
+        :type seq_nums_window: list[int]
+
+        :return: list of documents from that document with these sequence_numbers
+        :rtype: list[Document]
+        """
+        table_name = self._langchain_vector_store.table_name  # type: ignore[attr-defined]
+
+        placeholders = ",".join("?" for _ in seq_nums_window)
+
+        sql = f"""
+            WITH extracted AS (
+              SELECT
+                JSON_VALUE(metadata, '$.sequence_number' RETURNING INTEGER) AS seq_num,
+                JSON_VALUE(metadata, '$.document_id') AS doc_id,
+                text AS page_content
+              FROM {table_name}
+            )
+            SELECT
+              seq_num,
+              doc_id,
+              page_content
+            FROM extracted
+            WHERE doc_id = ?
+              AND seq_num IN ({placeholders})
+            ORDER BY seq_num;
+        """
+
+        params = [doc_id] + seq_nums_window
+
+        cursor = self._langchain_vector_store.client.cursor()  # type: ignore[attr-defined]
+        cursor.execute(sql, tuple(params))
+        rows = cursor.fetchall()
+
+        window_documents = [
+            Document(
+                page_content=row[2],
+                metadata={
+                    "sequence_number": row[0],
+                    "document_id": row[1],
+                },
+            )
+            for row in rows
+        ]
+        return window_documents
+
     def delete(self, ids: list[str], **kwargs: Any) -> None:
+        """Delete by vector ID or other criteria. Sor more details see LangChain documentation
+        https://python.langchain.com/api_reference/core/vectorstores/langchain_core.vectorstores.base.VectorStore.html#langchain_core.vectorstores.base.VectorStore
+        """
         self._langchain_vector_store.delete(ids, **kwargs)
 
     def clear(self) -> None:
@@ -352,6 +475,7 @@ class LangChainVectorStoreAdapter(BaseVectorStore):
         )
 
     def as_langchain_retriever(self, **kwargs: Any) -> Any:
+        """Return Langchain VectorStoreRetriever initialized from this VectorStore."""
         return LangChainVectorStoreRetriever(
             vectorstore=self._langchain_vector_store, **kwargs
         )
@@ -373,12 +497,16 @@ class LangChainVectorStoreAdapter(BaseVectorStore):
         docs = self._as_langchain_documents(content)
         if docs:
             # Take only unique ID document. Get two lists, one with ids, one with documents
+            # For some documents, not all chars can be encoded properly.
+            # In such cases, replace invalid chars by question marks, i.e. setting errors="replace"
             return tuple(
                 map(  # type: ignore[return-value]
                     list,
                     zip(
                         *{
-                            hashlib.sha256(str(doc).encode()).hexdigest(): doc
+                            hashlib.sha256(
+                                str(doc).encode(errors="replace")
+                            ).hexdigest(): doc
                             for doc in docs
                         }.items()
                     ),
@@ -491,9 +619,17 @@ class LangChainVectorStoreAdapter(BaseVectorStore):
                 window_documents = self._get_window_documents_elasticsearch(
                     doc_id, seq_nums_window
                 )
+            case "DB2VS":
+                try:
+                    from langchain_db2 import DB2VS
+                except ImportError:
+                    raise MissingExtension("langchain_db2")
+                window_documents = self._get_window_documents_db2(
+                    doc_id, seq_nums_window
+                )
             case _:
                 raise TypeError(
-                    f"Currently we only support Milvus, Chroma and Elasticsearch. "
+                    f"Currently we only support Milvus, Chroma, Elasticsearch and DB2VS. "
                     f"Received {type(self._langchain_vector_store)}."
                 )
 
