@@ -182,7 +182,7 @@ class TextExtractions(WMLResource):
             )
 
         response = self._client.httpx_client.post(
-            url=self._client.service_instance._href_definitions.get_text_extractions_href(),
+            url=self._client._href_definitions.get_text_extractions_href(),
             json=payload,
             params=self._client._params(skip_for_create=True, skip_userfs=True),
             headers=self._client._get_headers(),
@@ -251,7 +251,7 @@ class TextExtractions(WMLResource):
         TextExtractions._validate_type(extraction_id, "extraction_id", str, False)
         if extraction_id is not None:
             response = self._client.httpx_client.get(
-                url=self._client.service_instance._href_definitions.get_text_extraction_href(
+                url=self._client._href_definitions.get_text_extraction_href(
                     extraction_id
                 ),
                 params=self._client._params(skip_userfs=True),
@@ -270,7 +270,7 @@ class TextExtractions(WMLResource):
 
             # TODO: pagination is not yet implemented
             response = self._client.httpx_client.get(
-                url=self._client.service_instance._href_definitions.get_text_extractions_href(),
+                url=self._client._href_definitions.get_text_extractions_href(),
                 params=(self._client._params(skip_userfs=True) | (_params or {})),
                 headers=self._client._get_headers(),
             )
@@ -295,9 +295,7 @@ class TextExtractions(WMLResource):
         params.update({"hard_delete": True})
 
         response = self._client.httpx_client.delete(
-            url=self._client.service_instance._href_definitions.get_text_extraction_href(
-                extraction_id
-            ),
+            url=self._client._href_definitions.get_text_extraction_href(extraction_id),
             params=params,
             headers=self._client._get_headers(),
         )
@@ -319,9 +317,7 @@ class TextExtractions(WMLResource):
         TextExtractions._validate_type(extraction_id, "extraction_id", str, True)
 
         response = self._client.httpx_client.delete(
-            url=self._client.service_instance._href_definitions.get_text_extraction_href(
-                extraction_id
-            ),
+            url=self._client._href_definitions.get_text_extraction_href(extraction_id),
             params=self._client._params(skip_userfs=True),
             headers=self._client._get_headers(),
         )
