@@ -54,6 +54,7 @@ class FineTuner(BaseTuner):
         peft_parameters: dict | PeftParameters | None = None,
         auto_update_model: bool = True,
         group_by_name: bool = False,
+        gradient_checkpointing: bool | None = None,
     ):
         BaseTuner.__init__(self, "fine")
         self._client = api_client
@@ -85,6 +86,7 @@ class FineTuner(BaseTuner):
             response_template=response_template,
             gpu=gpu,
             peft_parameters=peft_parameters,
+            gradient_checkpointing=gradient_checkpointing,
         )
 
         if not isinstance(self.name, str):
@@ -327,7 +329,15 @@ class FineTuner(BaseTuner):
         return details
 
     @staticmethod
-    def _extract_metrics(tuning_details: dict) -> dict:
+    def _ensure_metrics_list(metrics_raw: list | dict) -> list:
+        if isinstance(metrics_raw, list):
+            return metrics_raw
+        elif isinstance(metrics_raw, dict):
+            return [metrics_raw]
+        else:
+            return []
+
+    def _extract_metrics(self, tuning_details: dict) -> dict:
         """
         Extracts loss scores, epochs, and timestamps from tuning details.
 
@@ -339,7 +349,8 @@ class FineTuner(BaseTuner):
         """
 
         constant_variable_to_add = 1
-        metrics = tuning_details["entity"]["status"]["metrics"]
+        metrics_raw = tuning_details["entity"]["status"].get("metrics", [])
+        metrics = self._ensure_metrics_list(metrics_raw)
 
         scores = []
         epochs = []
@@ -430,8 +441,10 @@ class FineTuner(BaseTuner):
 
         details = self.get_run_details(include_metrics=True)
 
-        metrics = details["entity"]["status"].get("metrics", [{}])[0]
-        is_ml_metrics = "data" in metrics or "ml_metrics" in metrics
+        metrics_raw = details["entity"]["status"].get("metrics", [])
+        metrics_list = self._ensure_metrics_list(metrics_raw)
+
+        is_ml_metrics = any("data" in m or "ml_metrics" in m for m in metrics_list)
 
         if not is_ml_metrics:
             raise WMLClientError(
