@@ -6,11 +6,19 @@ from __future__ import annotations
 
 import json
 import copy
+from contextlib import contextmanager, asynccontextmanager
+from functools import partial
 from warnings import warn
 
-from functools import partial
 from abc import ABC, abstractmethod
-from typing import TYPE_CHECKING, Generator, Any, Literal, AsyncGenerator
+from typing import (
+    TYPE_CHECKING,
+    Generator,
+    Any,
+    Literal,
+    AsyncGenerator,
+    Callable,
+)
 from concurrent.futures import ThreadPoolExecutor, as_completed
 from dataclasses import fields
 
@@ -20,10 +28,7 @@ import requests as _requests
 from ibm_watsonx_ai.foundation_models.utils.utils import (
     HAPDetectionWarning,
     PIIDetectionWarning,
-)
-from ibm_watsonx_ai._wrappers.requests import (
-    _get_httpx_client,
-    _get_async_client,
+    GraniteGuardianDetectionWarning,
 )
 from ibm_watsonx_ai.foundation_models.schema import (
     TextChatParameters,
@@ -40,6 +45,9 @@ if TYPE_CHECKING:
 
 __all__ = ["BaseModelInference"]
 
+_RETRY_STATUS_CODES = [429, 503, 504, 520]
+LIMIT_RATE_HEADER = "x-requests-limit-rate"
+
 
 class BaseModelInference(WMLResource, ABC):
     """Base interface class for the model interface."""
@@ -47,23 +55,40 @@ class BaseModelInference(WMLResource, ABC):
     DEFAULT_CONCURRENCY_LIMIT = 8
 
     def __init__(
-        self, name: str, client: APIClient, persistent_connection: bool = True
+        self,
+        name: str,
+        client: APIClient,
+        persistent_connection: bool = True,
+        max_retries: int | None = None,
+        delay_time: float | None = None,
+        retry_status_codes: list[int] | None = None,
+        validate: bool = True,
     ):
         self._persistent_connection = persistent_connection
 
+        # to use in get_identifying_params(
+        self._validate = validate
+
         self._transport_params = requests._httpx_transport_params(client)
 
         WMLResource.__init__(self, name, client)
         if self._persistent_connection:
-            self._http_client = _get_httpx_client(
-                transport_params=self._transport_params
-            )
+            self._http_client = client.httpx_client
         else:
-            self._http_client = requests
+            self._http_client = requests  # type: ignore[assignment]
+            persistent_connection_warn = (
+                "`persistent_connection` is deprecated and will be removed in future. "
+            )
+            warn(persistent_connection_warn, category=DeprecationWarning)
 
-        self._async_http_client = _get_async_client(
-            transport_params=self._transport_params
-        )
+        self._async_http_client = client.async_httpx_client
+        # Set initially 8 requests per second as it is default for prod instances
+        # if header "x-requests-limit-rate" is different capacity will be updated
+        self.rate_limiter = requests.TokenBucket(rate=8, capacity=8)
+
+        self.retry_status_codes = retry_status_codes
+        self.max_retries = max_retries
+        self.delay_time = delay_time
 
     @abstractmethod
     def get_details(self) -> dict:
@@ -145,6 +170,7 @@ class BaseModelInference(WMLResource, ABC):
         concurrency_limit: int = DEFAULT_CONCURRENCY_LIMIT,
         async_mode: bool = False,
         validate_prompt_variables: bool = True,
+        guardrails_granite_guardian_params: dict | None = None,
     ) -> dict | list[dict] | Generator:
         """
         Given a text prompt as input, and parameters the selected inference
@@ -160,6 +186,7 @@ class BaseModelInference(WMLResource, ABC):
         guardrails: bool = False,
         guardrails_hap_params: dict | None = None,
         guardrails_pii_params: dict | None = None,
+        guardrails_granite_guardian_params: dict | None = None,
         validate_prompt_variables: bool = True,
     ) -> dict:
         """
@@ -177,6 +204,7 @@ class BaseModelInference(WMLResource, ABC):
         guardrails_hap_params: dict | None = None,
         guardrails_pii_params: dict | None = None,
         validate_prompt_variables: bool = True,
+        guardrails_granite_guardian_params: dict | None = None,
     ) -> AsyncGenerator:
         """
         Given a text prompt as input, and parameters the selected inference
@@ -194,6 +222,7 @@ class BaseModelInference(WMLResource, ABC):
         guardrails_hap_params: dict | None = None,
         guardrails_pii_params: dict | None = None,
         validate_prompt_variables: bool = True,
+        guardrails_granite_guardian_params: dict | None = None,
     ) -> Generator:
         """
         Given a text prompt as input, and parameters the selected inference
@@ -227,6 +256,7 @@ class BaseModelInference(WMLResource, ABC):
         guardrails: bool = False,
         guardrails_hap_params: dict | None = None,
         guardrails_pii_params: dict | None = None,
+        guardrails_granite_guardian_params: dict | None = None,
     ) -> dict:
         raise NotImplementedError
 
@@ -237,10 +267,11 @@ class BaseModelInference(WMLResource, ABC):
         guardrails: bool = False,
         guardrails_hap_params: dict | None = None,
         guardrails_pii_params: dict | None = None,
+        guardrails_granite_guardian_params: dict | None = None,
     ) -> dict:
         raise NotImplementedError
 
-    def _send_inference_payload(
+    def _send_inference_payload_raw(
         self,
         prompt: str | None,
         params: dict | TextGenParameters | None,
@@ -248,23 +279,26 @@ class BaseModelInference(WMLResource, ABC):
         guardrails: bool = False,
         guardrails_hap_params: dict | None = None,
         guardrails_pii_params: dict | None = None,
-        _http_client: requests.HTTPXClient | None = None,
-    ) -> dict:
+        guardrails_granite_guardian_params: dict | None = None,
+        _http_client: httpx.Client | None = None,
+    ) -> httpx.Response | _requests.Response:
         if self._client._use_fm_ga_api:
             payload = self._prepare_inference_payload(
-                prompt,
+                prompt=prompt,
                 params=params,
                 guardrails=guardrails,
                 guardrails_hap_params=guardrails_hap_params,
                 guardrails_pii_params=guardrails_pii_params,
+                guardrails_granite_guardian_params=guardrails_granite_guardian_params,
             )
         else:  # Remove on CPD 5.0 release
             payload = self._prepare_beta_inference_payload(
-                prompt,
+                prompt=prompt,
                 params=params,
                 guardrails=guardrails,
                 guardrails_hap_params=guardrails_hap_params,
                 guardrails_pii_params=guardrails_pii_params,
+                guardrails_granite_guardian_params=guardrails_granite_guardian_params,
             )
         http_client = _http_client or self._http_client
 
@@ -274,10 +308,31 @@ class BaseModelInference(WMLResource, ABC):
             params=self._client._params(skip_for_create=True, skip_userfs=True),
             headers=self._client._get_headers(),
         )
-        if not isinstance(http_client, httpx.Client):
-            post_params["_retry_status_codes"] = requests.DEFAULT_RETRY_STATUS_CODES
 
-        response_scoring = http_client.post(**post_params)
+        return self._post(http_client=http_client, **post_params)
+
+    def _send_inference_payload(
+        self,
+        prompt: str | None,
+        params: dict | TextGenParameters | None,
+        generate_url: str,
+        guardrails: bool = False,
+        guardrails_hap_params: dict | None = None,
+        guardrails_pii_params: dict | None = None,
+        guardrails_granite_guardian_params: dict | None = None,
+        _http_client: requests.HTTPXClient | httpx.Client | None = None,
+    ) -> dict:
+
+        response_scoring = self._send_inference_payload_raw(
+            prompt=prompt,
+            params=params,
+            generate_url=generate_url,
+            guardrails=guardrails,
+            guardrails_hap_params=guardrails_hap_params,
+            guardrails_pii_params=guardrails_pii_params,
+            guardrails_granite_guardian_params=guardrails_granite_guardian_params,
+            _http_client=_http_client,
+        )
 
         return self._handle_response(
             200,
@@ -309,10 +364,8 @@ class BaseModelInference(WMLResource, ABC):
             params=self._client._params(skip_for_create=True, skip_userfs=True),
             headers=self._client._get_headers(),
         )
-        if not isinstance(self._http_client, httpx.Client):
-            post_params["_retry_status_codes"] = requests.DEFAULT_RETRY_STATUS_CODES
 
-        response_scoring = self._http_client.post(**post_params)
+        response_scoring = self._post(self._http_client, **post_params)
 
         return self._handle_response(
             200,
@@ -326,10 +379,20 @@ class BaseModelInference(WMLResource, ABC):
         deployment_chat_url: str,
         messages: list[dict],
         context: str | None = None,
+        tools: list | None = None,
+        tool_choice: dict | None = None,
+        tool_choice_option: Literal["none", "auto"] | None = None,
     ) -> dict:
         payload: dict = {"messages": messages}
+
         if context:
-            payload.update({"context": context})
+            payload["context"] = context
+        if tools:
+            payload["tools"] = tools
+        if tool_choice:
+            payload["tool_choice"] = tool_choice
+        if tool_choice_option:
+            payload["tool_choice_option"] = tool_choice_option
 
         post_params: dict[str, Any] = dict(
             url=deployment_chat_url,
@@ -337,10 +400,8 @@ class BaseModelInference(WMLResource, ABC):
             params=self._client._params(skip_for_create=True, skip_userfs=True),
             headers=self._client._get_headers(),
         )
-        if not isinstance(self._http_client, httpx.Client):
-            post_params["_retry_status_codes"] = requests.DEFAULT_RETRY_STATUS_CODES
 
-        response_scoring = self._http_client.post(**post_params)
+        response_scoring = self._post(self._http_client, **post_params)
 
         if response_scoring.status_code == 404:
             raise UnsupportedOperation(
@@ -359,10 +420,20 @@ class BaseModelInference(WMLResource, ABC):
         deployment_chat_url: str,
         messages: list[dict],
         context: str | None = None,
+        tools: list | None = None,
+        tool_choice: dict | None = None,
+        tool_choice_option: Literal["none", "auto"] | None = None,
     ) -> dict:
         payload: dict = {"messages": messages}
+
         if context:
-            payload.update({"context": context})
+            payload["context"] = context
+        if tools:
+            payload["tools"] = tools
+        if tool_choice:
+            payload["tool_choice"] = tool_choice
+        if tool_choice_option:
+            payload["tool_choice_option"] = tool_choice_option
 
         post_params: dict[str, Any] = dict(
             url=deployment_chat_url,
@@ -371,7 +442,7 @@ class BaseModelInference(WMLResource, ABC):
             headers=self._client._get_headers(),
         )
 
-        response = await self._async_http_client.post(**post_params)
+        response = await self._apost(self._async_http_client, **post_params)
 
         if response.status_code == 404:
             raise UnsupportedOperation(
@@ -393,27 +464,29 @@ class BaseModelInference(WMLResource, ABC):
         guardrails: bool = False,
         guardrails_hap_params: dict | None = None,
         guardrails_pii_params: dict | None = None,
-        async_client: requests.HTTPXAsyncClient | None = None,
+        guardrails_granite_guardian_params: dict | None = None,
     ) -> dict:
         if self._client._use_fm_ga_api:
             payload = self._prepare_inference_payload(
-                prompt,
+                prompt=prompt,
                 params=params,
                 guardrails=guardrails,
                 guardrails_hap_params=guardrails_hap_params,
                 guardrails_pii_params=guardrails_pii_params,
+                guardrails_granite_guardian_params=guardrails_granite_guardian_params,
             )
         else:  # Remove on CPD 5.0 release
             payload = self._prepare_beta_inference_payload(
-                prompt,
+                prompt=prompt,
                 params=params,
                 guardrails=guardrails,
                 guardrails_hap_params=guardrails_hap_params,
                 guardrails_pii_params=guardrails_pii_params,
+                guardrails_granite_guardian_params=guardrails_granite_guardian_params,
             )
 
-        _async_http_client = async_client or self._async_http_client
-        response = await _async_http_client.post(
+        response = await self._apost(
+            self._async_http_client,
             url=generate_url,
             json=payload,
             headers=self._client._get_headers(),
@@ -430,26 +503,32 @@ class BaseModelInference(WMLResource, ABC):
         guardrails: bool = False,
         guardrails_hap_params: dict | None = None,
         guardrails_pii_params: dict | None = None,
-        async_client: requests.HTTPXAsyncClient | None = None,
+        guardrails_granite_guardian_params: dict | None = None,
     ) -> AsyncGenerator:
         if self._client._use_fm_ga_api:
             payload = self._prepare_inference_payload(
-                prompt,
+                prompt=prompt,
                 params=params,
                 guardrails=guardrails,
                 guardrails_hap_params=guardrails_hap_params,
                 guardrails_pii_params=guardrails_pii_params,
+                guardrails_granite_guardian_params=guardrails_granite_guardian_params,
             )
         else:  # Remove on CPD 5.0 release
             payload = self._prepare_beta_inference_payload(
-                prompt,
+                prompt=prompt,
                 params=params,
                 guardrails=guardrails,
                 guardrails_hap_params=guardrails_hap_params,
                 guardrails_pii_params=guardrails_pii_params,
+                guardrails_granite_guardian_params=guardrails_granite_guardian_params,
             )
 
-        stream_function = self._async_http_client.post_stream
+        if isinstance(self._async_http_client, requests.HTTPXAsyncClient):
+            stream_function = self._async_http_client.post_stream
+        else:
+            stream_function = self._async_http_client.stream  # type: ignore[assignment]
+
         kw_args: dict = dict(
             method="POST",
             url=generate_url,
@@ -458,7 +537,7 @@ class BaseModelInference(WMLResource, ABC):
             params=self._client._params(skip_for_create=True, skip_userfs=True),
         )
 
-        async with stream_function(**kw_args) as resp:
+        async with self._astream(stream_function, **kw_args) as resp:
             if resp.status_code == 200:
                 resp_iter = resp.aiter_lines()
 
@@ -514,15 +593,15 @@ class BaseModelInference(WMLResource, ABC):
         if isinstance(self._http_client, requests.HTTPXClient):
             stream_function = self._http_client.post_stream
             kw_args |= {"method": "POST"}
-
+        elif isinstance(self._http_client, httpx.Client):
+            stream_function = self._http_client.stream  # type: ignore[assignment]
+            kw_args |= {"method": "POST"}
         else:
             stream_function = requests.Session().post  # type: ignore[assignment]
             kw_args |= {
                 "stream": True,
-                "_retry_status_codes": requests.DEFAULT_RETRY_STATUS_CODES,
             }
-
-        with stream_function(**kw_args) as resp:
+        with self._stream(stream_function, **kw_args) as resp:
             if resp.status_code == 200:
                 resp_iter = (
                     resp.iter_lines()
@@ -531,7 +610,7 @@ class BaseModelInference(WMLResource, ABC):
                 )
                 for chunk in resp_iter:
                     if isinstance(resp, _requests.Response):
-                        chunk = chunk.decode("utf-8")
+                        chunk = chunk.decode("utf-8")  # type: ignore[union-attr]
                     field_name, _, response = chunk.partition(":")
                     if field_name == "data" and response:
                         try:
@@ -547,16 +626,46 @@ class BaseModelInference(WMLResource, ABC):
                     f"Request failed with: {resp.text} ({resp.status_code})"
                 )
 
+    @requests._with_retry_stream()
+    @contextmanager
+    def _stream(
+        self,
+        stream_function: Callable,
+        **kw_args: Any,
+    ) -> Generator[httpx.Response, None, None]:
+        """Handles streaming with retry."""
+        with stream_function(**kw_args) as resp:
+            yield resp
+
+    @requests._with_async_retry_stream()
+    @asynccontextmanager
+    async def _astream(
+        self,
+        stream_function: Callable,
+        **kw_args: Any,
+    ) -> AsyncGenerator[httpx.Response, None]:
+        async with stream_function(**kw_args) as resp:
+            yield resp
+
     def _generate_deployment_chat_stream_with_url(
         self,
         deployment_chat_stream_url: str,
         messages: list[dict],
         context: str | None = None,
+        tools: list | None = None,
+        tool_choice: dict | None = None,
+        tool_choice_option: Literal["none", "auto"] | None = None,
     ) -> Generator:
-
         payload: dict = {"messages": messages}
+
         if context:
-            payload.update({"context": context})
+            payload["context"] = context
+        if tools:
+            payload["tools"] = tools
+        if tool_choice:
+            payload["tool_choice"] = tool_choice
+        if tool_choice_option:
+            payload["tool_choice_option"] = tool_choice_option
 
         kw_args: dict = dict(
             url=deployment_chat_stream_url,
@@ -568,15 +677,16 @@ class BaseModelInference(WMLResource, ABC):
         if isinstance(self._http_client, requests.HTTPXClient):
             stream_function = self._http_client.post_stream
             kw_args |= {"method": "POST"}
-
+        elif isinstance(self._http_client, httpx.Client):
+            stream_function = self._http_client.stream  # type: ignore[assignment]
+            kw_args |= {"method": "POST"}
         else:
             stream_function = requests.Session().post  # type: ignore[assignment]
             kw_args |= {
                 "stream": True,
-                "_retry_status_codes": requests.DEFAULT_RETRY_STATUS_CODES,
             }
 
-        with stream_function(**kw_args) as resp:
+        with self._stream(stream_function, **kw_args) as resp:
             if resp.status_code == 200:
                 resp_iter = (
                     resp.iter_lines()
@@ -585,7 +695,7 @@ class BaseModelInference(WMLResource, ABC):
                 )
                 for chunk in resp_iter:
                     if isinstance(resp, _requests.Response):
-                        chunk = chunk.decode("utf-8")
+                        chunk = chunk.decode("utf-8")  # type: ignore[union-attr]
                     field_name, _, response = chunk.partition(":")
                     if field_name == "data" and response:
                         try:
@@ -611,13 +721,26 @@ class BaseModelInference(WMLResource, ABC):
         deployment_chat_stream_url: str,
         messages: list[dict],
         context: str | None = None,
+        tools: list | None = None,
+        tool_choice: dict | None = None,
+        tool_choice_option: Literal["none", "auto"] | None = None,
     ) -> AsyncGenerator:
-
         payload: dict = {"messages": messages}
-        if context:
-            payload.update({"context": context})
 
-        stream_function = self._async_http_client.post_stream
+        if context:
+            payload["context"] = context
+        if tools:
+            payload["tools"] = tools
+        if tool_choice:
+            payload["tool_choice"] = tool_choice
+        if tool_choice_option:
+            payload["tool_choice_option"] = tool_choice_option
+
+        if isinstance(self._async_http_client, requests.HTTPXAsyncClient):
+            stream_function = self._async_http_client.post_stream
+        else:
+            stream_function = self._async_http_client.stream  # type: ignore[assignment]
+
         kw_args: dict = dict(
             method="POST",
             url=deployment_chat_stream_url,
@@ -625,8 +748,7 @@ class BaseModelInference(WMLResource, ABC):
             headers=self._client._get_headers(),
             params=self._client._params(skip_for_create=True, skip_userfs=True),
         )
-
-        async with stream_function(**kw_args) as resp:
+        async with self._astream(stream_function, **kw_args) as resp:
             if resp.status_code == 200:
                 resp_iter = resp.aiter_lines()
 
@@ -663,7 +785,11 @@ class BaseModelInference(WMLResource, ABC):
             tool_choice_option=tool_choice_option,
         )
 
-        stream_function = self._async_http_client.post_stream
+        if isinstance(self._async_http_client, requests.HTTPXAsyncClient):
+            stream_function = self._async_http_client.post_stream
+        else:
+            stream_function = self._async_http_client.stream  # type: ignore[assignment]
+
         kw_args: dict = dict(
             method="POST",
             url=chat_stream_url,
@@ -672,7 +798,7 @@ class BaseModelInference(WMLResource, ABC):
             params=self._client._params(skip_for_create=True, skip_userfs=True),
         )
 
-        async with stream_function(**kw_args) as resp:
+        async with self._astream(stream_function, **kw_args) as resp:
             if resp.status_code == 200:
                 resp_iter = resp.aiter_lines()
 
@@ -691,6 +817,48 @@ class BaseModelInference(WMLResource, ABC):
                     f"Request failed with: ({resp.text} {resp.status_code})"
                 )
 
+    def __make_request(
+        self,
+        prompt: str | None,
+        params: dict | TextGenParameters | None,
+        generate_url: str,
+        guardrails: bool = False,
+        guardrails_hap_params: dict | None = None,
+        guardrails_pii_params: dict | None = None,
+        guardrails_granite_guardian_params: dict | None = None,
+        _http_client: requests.HTTPXClient | httpx.Client | None = None,
+    ) -> dict:
+        """Rate-limited request with dynamic token adjustment and retry logic."""
+        self.rate_limiter.acquire()
+
+        inference_response = self._send_inference_payload_raw(
+            prompt=prompt,
+            params=params,
+            generate_url=generate_url,
+            guardrails=guardrails,
+            guardrails_hap_params=guardrails_hap_params,
+            guardrails_pii_params=guardrails_pii_params,
+            guardrails_granite_guardian_params=guardrails_granite_guardian_params,
+            _http_client=_http_client,
+        )
+        rate_limit = int(inference_response.headers.get(LIMIT_RATE_HEADER, 8))
+        if rate_limit and rate_limit != self.rate_limiter.capacity:
+            self.rate_limiter.capacity = rate_limit
+
+        rate_limit_remaining = int(
+            inference_response.headers.get(
+                requests.REMAINING_LIMIT_HEADER, self.rate_limiter.capacity
+            )
+        )
+        self.rate_limiter.adjust_tokens(rate_limit_remaining)
+
+        return self._handle_response(
+            200,
+            "generate",
+            inference_response,
+            _field_to_hide="generated_text",
+        )
+
     def _generate_with_url(
         self,
         prompt: list | str | None,
@@ -699,50 +867,45 @@ class BaseModelInference(WMLResource, ABC):
         guardrails: bool = False,
         guardrails_hap_params: dict | None = None,
         guardrails_pii_params: dict | None = None,
+        guardrails_granite_guardian_params: dict | None = None,
         concurrency_limit: int = DEFAULT_CONCURRENCY_LIMIT,
     ) -> list | dict:
         """
         Helper method which implements multi-threading for with passed generate_url.
         """
         if isinstance(prompt, list):
-            try:
-                # For batch of prompts use keep-alive connection even if persistent_connection=False
-                http_client: requests.HTTPXClient = (
-                    _get_httpx_client(transport_params=self._transport_params)
-                    if not self._persistent_connection
-                    else self._http_client
-                )
-                _send_inference_payload_partial = partial(
-                    self._send_inference_payload,
-                    params=params,
-                    generate_url=generate_url,
-                    guardrails=guardrails,
-                    guardrails_hap_params=guardrails_hap_params,
-                    guardrails_pii_params=guardrails_pii_params,
-                    _http_client=http_client,
-                )
-                if (prompt_length := len(prompt)) <= concurrency_limit:
-                    with ThreadPoolExecutor(max_workers=prompt_length) as executor:
-                        generated_responses = list(
-                            executor.map(_send_inference_payload_partial, prompt)
-                        )
-                else:
-                    with ThreadPoolExecutor(max_workers=concurrency_limit) as executor:
-                        generated_responses = list(
-                            executor.map(_send_inference_payload_partial, prompt)
-                        )
-                return generated_responses
-            except Exception:
-                http_client = None  # type: ignore[assignment]
-                raise
-            finally:
-                if not self._persistent_connection and isinstance(
-                    http_client, httpx.Client
-                ):
-                    http_client.close()
+            # For batch of prompts use keep-alive connection even if persistent_connection=False
+            http_client: requests.HTTPXClient | httpx.Client = (
+                self._client.httpx_client
+                if not self._persistent_connection
+                else self._http_client
+            )
+            # If CLOUD, use __make_request which uses token bucket for throttling
+            if self._client.CLOUD_PLATFORM_SPACES:
+                func = self.__make_request
+            else:  # If CDP, don't use Token Bucket
+                func = self._send_inference_payload
+
+            inference_fn = partial(
+                func,
+                params=params,
+                generate_url=generate_url,
+                guardrails=guardrails,
+                guardrails_hap_params=guardrails_hap_params,
+                guardrails_pii_params=guardrails_pii_params,
+                guardrails_granite_guardian_params=guardrails_granite_guardian_params,
+                _http_client=http_client,
+            )  # If CDP, don't use Token Bucket
+
+            if (prompt_length := len(prompt)) <= concurrency_limit:
+                with ThreadPoolExecutor(max_workers=prompt_length) as executor:
+                    generated_responses = list(executor.map(inference_fn, prompt))
+            else:
+                with ThreadPoolExecutor(max_workers=concurrency_limit) as executor:
+                    generated_responses = list(executor.map(inference_fn, prompt))
+            return generated_responses
 
         else:
-
             response = self._send_inference_payload(
                 prompt,
                 params,
@@ -750,6 +913,7 @@ class BaseModelInference(WMLResource, ABC):
                 guardrails,
                 guardrails_hap_params,
                 guardrails_pii_params,
+                guardrails_granite_guardian_params,
             )
         return response
 
@@ -761,6 +925,7 @@ class BaseModelInference(WMLResource, ABC):
         guardrails: bool = False,
         guardrails_hap_params: dict | None = None,
         guardrails_pii_params: dict | None = None,
+        guardrails_granite_guardian_params: dict | None = None,
         concurrency_limit: int = DEFAULT_CONCURRENCY_LIMIT,
     ) -> Generator:
         """
@@ -774,69 +939,63 @@ class BaseModelInference(WMLResource, ABC):
 
         async_params["return_options"] = {"input_text": True}
         if isinstance(prompt, list):
-            try:
-                # For batch of prompts use keep-alive connection even if persistent_connection=False
-                http_client: requests.HTTPXClient = (
-                    _get_httpx_client(transport_params=self._transport_params)
-                    if not self._persistent_connection
-                    else self._http_client
-                )
-                _send_inference_payload_partial = partial(
-                    self._send_inference_payload,
-                    params=async_params,
-                    generate_url=generate_url,
-                    guardrails=guardrails,
-                    guardrails_hap_params=guardrails_hap_params,
-                    guardrails_pii_params=guardrails_pii_params,
-                    _http_client=http_client,
-                )
-                if (prompt_length := len(prompt)) <= concurrency_limit:
-                    with ThreadPoolExecutor(max_workers=prompt_length) as executor:
-                        generate_futures = [
-                            executor.submit(
-                                _send_inference_payload_partial, single_prompt
-                            )
-                            for single_prompt in prompt
-                        ]
-                        try:
-                            for future in as_completed(generate_futures):
-                                yield future.result()
-                        except:
-                            executor.shutdown(wait=False, cancel_futures=True)
-                            raise
+            # For batch of prompts use keep-alive connection even if persistent_connection=False
+            http_client: requests.HTTPXClient | httpx.Client = (
+                self._client.httpx_client
+                if not self._persistent_connection
+                else self._http_client
+            )
+            # If CLOUD, use __make_request which uses token bucket for throttling
+            if self._client.CLOUD_PLATFORM_SPACES:
+                func = self.__make_request
+            else:  # If CDP, don't use Token Bucket
+                func = self._send_inference_payload
 
-                else:
-                    with ThreadPoolExecutor(max_workers=concurrency_limit) as executor:
-                        generate_futures = [
-                            executor.submit(
-                                _send_inference_payload_partial, single_prompt
-                            )
-                            for single_prompt in prompt
-                        ]
-                        try:
-                            for future in as_completed(generate_futures):
-                                yield future.result()
-                        except:
-                            executor.shutdown(wait=False, cancel_futures=True)
-                            raise
-            except Exception:
-                http_client = None  # type: ignore[assignment]
-                raise
-            finally:
-                if not self._persistent_connection and isinstance(
-                    http_client, httpx.Client
-                ):
-                    http_client.close()
+            inference_fn = partial(
+                func,
+                params=async_params,
+                generate_url=generate_url,
+                guardrails=guardrails,
+                guardrails_hap_params=guardrails_hap_params,
+                guardrails_pii_params=guardrails_pii_params,
+                guardrails_granite_guardian_params=guardrails_granite_guardian_params,
+                _http_client=http_client,
+            )
 
+            if (prompt_length := len(prompt)) <= concurrency_limit:
+                with ThreadPoolExecutor(max_workers=prompt_length) as executor:
+                    generate_futures = [
+                        executor.submit(inference_fn, single_prompt)
+                        for single_prompt in prompt
+                    ]
+                    try:
+                        for future in as_completed(generate_futures):
+                            yield future.result()
+                    except:
+                        executor.shutdown(wait=False, cancel_futures=True)
+                        raise
+
+            else:
+                with ThreadPoolExecutor(max_workers=concurrency_limit) as executor:
+                    generate_futures = [
+                        executor.submit(inference_fn, single_prompt)
+                        for single_prompt in prompt
+                    ]
+                    try:
+                        for future in as_completed(generate_futures):
+                            yield future.result()
+                    except:
+                        executor.shutdown(wait=False, cancel_futures=True)
+                        raise
         else:
-
             response = self._send_inference_payload(
-                prompt,
-                async_params,
-                generate_url,
-                guardrails,
-                guardrails_hap_params,
-                guardrails_pii_params,
+                prompt=prompt,
+                params=async_params,
+                generate_url=generate_url,
+                guardrails=guardrails,
+                guardrails_hap_params=guardrails_hap_params,
+                guardrails_pii_params=guardrails_pii_params,
+                guardrails_granite_guardian_params=guardrails_granite_guardian_params,
             )
             yield response
 
@@ -849,22 +1008,25 @@ class BaseModelInference(WMLResource, ABC):
         guardrails: bool = False,
         guardrails_hap_params: dict | None = None,
         guardrails_pii_params: dict | None = None,
+        guardrails_granite_guardian_params: dict | None = None,
     ) -> Generator:
         if self._client._use_fm_ga_api:
             payload = self._prepare_inference_payload(
-                prompt,
+                prompt=prompt,
                 params=params,
                 guardrails=guardrails,
                 guardrails_hap_params=guardrails_hap_params,
                 guardrails_pii_params=guardrails_pii_params,
+                guardrails_granite_guardian_params=guardrails_granite_guardian_params,
             )
         else:  # Remove on CPD 5.0 release
             payload = self._prepare_beta_inference_payload(
-                prompt,
+                prompt=prompt,
                 params=params,
                 guardrails=guardrails,
                 guardrails_hap_params=guardrails_hap_params,
                 guardrails_pii_params=guardrails_pii_params,
+                guardrails_granite_guardian_params=guardrails_granite_guardian_params,
             )
 
         kw_args: dict = dict(
@@ -877,15 +1039,15 @@ class BaseModelInference(WMLResource, ABC):
         if isinstance(self._http_client, requests.HTTPXClient):
             stream_function = self._http_client.post_stream
             kw_args |= {"method": "POST"}
-
+        elif isinstance(self._http_client, httpx.Client):
+            stream_function = self._http_client.stream  # type: ignore[assignment]
+            kw_args |= {"method": "POST"}
         else:
             stream_function = requests.Session().post  # type: ignore[assignment]
             kw_args |= {
                 "stream": True,
-                "_retry_status_codes": requests.DEFAULT_RETRY_STATUS_CODES,
             }
-
-        with stream_function(**kw_args) as resp:
+        with self._stream(stream_function, **kw_args) as resp:
             if resp.status_code == 200:
                 resp_iter = (
                     resp.iter_lines()
@@ -894,7 +1056,7 @@ class BaseModelInference(WMLResource, ABC):
                 )
                 for chunk in resp_iter:
                     if isinstance(resp, _requests.Response):
-                        chunk = chunk.decode("utf-8")
+                        chunk = chunk.decode("utf-8")  # type: ignore[union-attr]
                     if chunk.rstrip() == "event: error":
                         chunk = next(resp_iter)
                         field_name, _, response = chunk.partition(":")
@@ -942,9 +1104,9 @@ class BaseModelInference(WMLResource, ABC):
             headers=self._client._get_headers(),
         )
         if not isinstance(self._http_client, httpx.Client):
-            post_params["_retry_status_codes"] = requests.DEFAULT_RETRY_STATUS_CODES
+            post_params["_retry_status_codes"] = _RETRY_STATUS_CODES
 
-        response_scoring = self._http_client.post(**post_params)
+        response_scoring = self._post(self._http_client, **post_params)
 
         if response_scoring.status_code == 404:
             raise WMLClientError("Tokenize is not supported for this release")
@@ -988,6 +1150,22 @@ class BaseModelInference(WMLResource, ABC):
                     f"Personally identifiable information detected: {pii_details}"
                 )
                 warn(identifiable_information_warning, category=PIIDetectionWarning)
+        granite_guardian_details = results.get("moderations", {}).get(
+            "granite_guardian"
+        )
+        if granite_guardian_details:
+            if granite_guardian_details[0].get("input"):
+                unsuitable_input_warning = next(
+                    warning.get("message")
+                    for warning in single_response.get("system", {}).get("warnings")
+                    if warning.get("id") == "UNSUITABLE_INPUT"
+                )
+                warn(unsuitable_input_warning, category=GraniteGuardianDetectionWarning)
+            else:
+                granite_guardian_warning = (
+                    f"Potentially granite guardian detected: {granite_guardian_details}"
+                )
+                warn(granite_guardian_warning, category=GraniteGuardianDetectionWarning)
         return results
 
     @staticmethod
@@ -1032,3 +1210,15 @@ class BaseModelInference(WMLResource, ABC):
             warn(invalid_params_warning)
 
         return valid_params
+
+    @requests._with_retry()
+    def _post(
+        self, http_client: Any, *args: Any, **kwargs: Any
+    ) -> httpx.Response | _requests.Response:
+        return http_client.post(*args, **kwargs)
+
+    @requests._with_async_retry()
+    async def _apost(
+        self, async_http_client: Any, *args: Any, **kwargs: Any
+    ) -> httpx.Response:
+        return await async_http_client.post(*args, **kwargs)
