@@ -107,14 +107,17 @@ class DeploymentModelInference(BaseModelInference):
     ) -> dict:
         self._validate_type(messages, "messages", list, True)
 
-        infer_chat_url = (
-            self._client.service_instance._href_definitions.get_fm_deployment_chat_href(
-                deployment_id=self.deployment_id
-            )
+        infer_chat_url = self._client._href_definitions.get_fm_deployment_chat_href(
+            deployment_id=self.deployment_id
         )
 
         return self._send_deployment_chat_payload(
-            deployment_chat_url=infer_chat_url, messages=messages, context=context
+            deployment_chat_url=infer_chat_url,
+            messages=messages,
+            context=context,
+            tools=tools,
+            tool_choice=tool_choice,
+            tool_choice_option=tool_choice_option,
         )
 
     def chat_stream(
@@ -128,14 +131,19 @@ class DeploymentModelInference(BaseModelInference):
     ) -> Generator:
         self._validate_type(messages, "messages", list, True)
 
-        infer_chat_url = self._client.service_instance._href_definitions.get_fm_deployment_chat_stream_href(
-            deployment_id=self.deployment_id
+        infer_chat_url = (
+            self._client._href_definitions.get_fm_deployment_chat_stream_href(
+                deployment_id=self.deployment_id
+            )
         )
 
         return self._generate_deployment_chat_stream_with_url(
             deployment_chat_stream_url=infer_chat_url,
             messages=messages,
             context=context,
+            tools=tools,
+            tool_choice=tool_choice,
+            tool_choice_option=tool_choice_option,
         )
 
     async def achat(
@@ -149,14 +157,17 @@ class DeploymentModelInference(BaseModelInference):
     ) -> dict:
         self._validate_type(messages, "messages", list, True)
 
-        infer_chat_url = (
-            self._client.service_instance._href_definitions.get_fm_deployment_chat_href(
-                deployment_id=self.deployment_id
-            )
+        infer_chat_url = self._client._href_definitions.get_fm_deployment_chat_href(
+            deployment_id=self.deployment_id
         )
 
         return await self._asend_deployment_chat_payload(
-            deployment_chat_url=infer_chat_url, messages=messages, context=context
+            deployment_chat_url=infer_chat_url,
+            messages=messages,
+            context=context,
+            tools=tools,
+            tool_choice=tool_choice,
+            tool_choice_option=tool_choice_option,
         )
 
     async def achat_stream(
@@ -170,14 +181,19 @@ class DeploymentModelInference(BaseModelInference):
     ) -> AsyncGenerator:
         self._validate_type(messages, "messages", list, True)
 
-        infer_chat_url = self._client.service_instance._href_definitions.get_fm_deployment_chat_stream_href(
-            deployment_id=self.deployment_id
+        infer_chat_url = (
+            self._client._href_definitions.get_fm_deployment_chat_stream_href(
+                deployment_id=self.deployment_id
+            )
         )
 
         return self._agenerate_deployment_chat_stream_with_url(
             deployment_chat_stream_url=infer_chat_url,
             messages=messages,
             context=context,
+            tools=tools,
+            tool_choice=tool_choice,
+            tool_choice_option=tool_choice_option,
         )
 
     @overload
@@ -191,6 +207,7 @@ class DeploymentModelInference(BaseModelInference):
         concurrency_limit: int = ...,
         async_mode: Literal[False] = ...,
         validate_prompt_variables: bool = ...,
+        guardrails_granite_guardian_params: dict | None = ...,
     ) -> dict | list[dict]: ...
 
     @overload
@@ -204,6 +221,7 @@ class DeploymentModelInference(BaseModelInference):
         concurrency_limit: int,
         async_mode: Literal[True],
         validate_prompt_variables: bool,
+        guardrails_granite_guardian_params: dict | None,
     ) -> Generator: ...
 
     @overload
@@ -217,6 +235,7 @@ class DeploymentModelInference(BaseModelInference):
         concurrency_limit: int = ...,
         async_mode: bool = ...,
         validate_prompt_variables: bool = ...,
+        guardrails_granite_guardian_params: dict | None = ...,
     ) -> dict | list[dict] | Generator: ...
 
     def generate(
@@ -229,6 +248,7 @@ class DeploymentModelInference(BaseModelInference):
         concurrency_limit: int = BaseModelInference.DEFAULT_CONCURRENCY_LIMIT,
         async_mode: bool = False,
         validate_prompt_variables: bool = True,
+        guardrails_granite_guardian_params: dict | None = None,
     ) -> dict | list[dict] | Generator:
         """
         Given a text prompt as input, and parameters the selected inference
@@ -242,8 +262,16 @@ class DeploymentModelInference(BaseModelInference):
         )
         self._validate_type(guardrails_hap_params, "guardrails_hap_params", dict, False)
         self._validate_type(guardrails_pii_params, "guardrails_pii_params", dict, False)
-        generate_text_url = self._client.service_instance._href_definitions.get_fm_deployment_generation_href(
-            deployment_id=self.deployment_id, item="text"
+        self._validate_type(
+            guardrails_granite_guardian_params,
+            "guardrails_granite_guardian_params",
+            dict,
+            False,
+        )
+        generate_text_url = (
+            self._client._href_definitions.get_fm_deployment_generation_href(
+                deployment_id=self.deployment_id, item="text"
+            )
         )
 
         if async_mode:
@@ -256,6 +284,7 @@ class DeploymentModelInference(BaseModelInference):
                 guardrails=guardrails,
                 guardrails_hap_params=guardrails_hap_params,
                 guardrails_pii_params=guardrails_pii_params,
+                guardrails_granite_guardian_params=guardrails_granite_guardian_params,
                 concurrency_limit=concurrency_limit,
             )
         else:
@@ -266,6 +295,7 @@ class DeploymentModelInference(BaseModelInference):
                 guardrails=guardrails,
                 guardrails_hap_params=guardrails_hap_params,
                 guardrails_pii_params=guardrails_pii_params,
+                guardrails_granite_guardian_params=guardrails_granite_guardian_params,
                 concurrency_limit=concurrency_limit,
             )
 
@@ -276,6 +306,7 @@ class DeploymentModelInference(BaseModelInference):
         guardrails: bool = False,
         guardrails_hap_params: dict | None = None,
         guardrails_pii_params: dict | None = None,
+        guardrails_granite_guardian_params: dict | None = None,
         validate_prompt_variables: bool = True,
     ) -> dict:
 
@@ -289,8 +320,16 @@ class DeploymentModelInference(BaseModelInference):
         self._validate_type(
             guardrails_pii_params, "guardrails_pii_params", dict, mandatory=False
         )
-        generate_text_url = self._client.service_instance._href_definitions.get_fm_deployment_generation_href(
-            deployment_id=self.deployment_id, item="text"
+        self._validate_type(
+            guardrails_granite_guardian_params,
+            "guardrails_granite_guardian_params",
+            dict,
+            mandatory=False,
+        )
+        generate_text_url = (
+            self._client._href_definitions.get_fm_deployment_generation_href(
+                deployment_id=self.deployment_id, item="text"
+            )
         )
 
         async_params = params or self.params or {}
@@ -299,12 +338,13 @@ class DeploymentModelInference(BaseModelInference):
             async_params = async_params.to_dict()
 
         return await self._asend_inference_payload(
-            prompt,
-            async_params,
-            generate_text_url,
-            guardrails,
-            guardrails_hap_params,
-            guardrails_pii_params,
+            prompt=prompt,
+            params=async_params,
+            generate_url=generate_text_url,
+            guardrails=guardrails,
+            guardrails_hap_params=guardrails_hap_params,
+            guardrails_pii_params=guardrails_pii_params,
+            guardrails_granite_guardian_params=guardrails_granite_guardian_params,
         )
 
     async def agenerate_stream(
@@ -315,6 +355,7 @@ class DeploymentModelInference(BaseModelInference):
         guardrails_hap_params: dict | None = None,
         guardrails_pii_params: dict | None = None,
         validate_prompt_variables: bool = True,
+        guardrails_granite_guardian_params: dict | None = None,
     ) -> AsyncGenerator:
         """
         Given a text prompt as input, and parameters the selected inference
@@ -328,14 +369,24 @@ class DeploymentModelInference(BaseModelInference):
         self._validate_type(prompt, "prompt", str, prompt_required)
         self._validate_type(guardrails_hap_params, "guardrails_hap_params", dict, False)
         self._validate_type(guardrails_pii_params, "guardrails_pii_params", dict, False)
+        self._validate_type(
+            guardrails_granite_guardian_params,
+            "guardrails_granite_guardian_params",
+            dict,
+            False,
+        )
 
         if self._client._use_fm_ga_api:
-            generate_stream_url = self._client.service_instance._href_definitions.get_fm_deployment_generation_stream_href(
-                deployment_id=self.deployment_id
+            generate_stream_url = (
+                self._client._href_definitions.get_fm_deployment_generation_stream_href(
+                    deployment_id=self.deployment_id
+                )
             )
         else:  # Remove on CPD 5.0 release
-            generate_stream_url = self._client.service_instance._href_definitions.get_fm_deployment_generation_href(
-                deployment_id=self.deployment_id, item="text_stream"
+            generate_stream_url = (
+                self._client._href_definitions.get_fm_deployment_generation_href(
+                    deployment_id=self.deployment_id, item="text_stream"
+                )
             )
 
         async_params = params or self.params or {}
@@ -344,12 +395,13 @@ class DeploymentModelInference(BaseModelInference):
             async_params = async_params.to_dict()
 
         return self._agenerate_stream_with_url(
-            prompt,
-            async_params,
-            generate_stream_url,
-            guardrails,
-            guardrails_hap_params,
-            guardrails_pii_params,
+            prompt=prompt,
+            params=async_params,
+            generate_url=generate_stream_url,
+            guardrails=guardrails,
+            guardrails_hap_params=guardrails_hap_params,
+            guardrails_pii_params=guardrails_pii_params,
+            guardrails_granite_guardian_params=guardrails_granite_guardian_params,
         )
 
     def generate_text_stream(
@@ -361,6 +413,7 @@ class DeploymentModelInference(BaseModelInference):
         guardrails_hap_params: dict | None = None,
         guardrails_pii_params: dict | None = None,
         validate_prompt_variables: bool = True,
+        guardrails_granite_guardian_params: dict | None = None,
     ) -> Generator:
         """
         Given a text prompt as input, and parameters the selected inference
@@ -372,13 +425,23 @@ class DeploymentModelInference(BaseModelInference):
         self._validate_type(prompt, "prompt", str, prompt_required)
         self._validate_type(guardrails_hap_params, "guardrails_hap_params", dict, False)
         self._validate_type(guardrails_pii_params, "guardrails_pii_params", dict, False)
+        self._validate_type(
+            guardrails_granite_guardian_params,
+            "guardrails_granite_guardian_params",
+            dict,
+            False,
+        )
         if self._client._use_fm_ga_api:
-            generate_text_stream_url = self._client.service_instance._href_definitions.get_fm_deployment_generation_stream_href(
-                deployment_id=self.deployment_id
+            generate_text_stream_url = (
+                self._client._href_definitions.get_fm_deployment_generation_stream_href(
+                    deployment_id=self.deployment_id
+                )
             )
         else:  # Remove on CPD 5.0 release
-            generate_text_stream_url = self._client.service_instance._href_definitions.get_fm_deployment_generation_href(
-                deployment_id=self.deployment_id, item="text_stream"
+            generate_text_stream_url = (
+                self._client._href_definitions.get_fm_deployment_generation_href(
+                    deployment_id=self.deployment_id, item="text_stream"
+                )
             )
 
         return self._generate_stream_with_url(
@@ -389,6 +452,7 @@ class DeploymentModelInference(BaseModelInference):
             guardrails=guardrails,
             guardrails_hap_params=guardrails_hap_params,
             guardrails_pii_params=guardrails_pii_params,
+            guardrails_granite_guardian_params=guardrails_granite_guardian_params,
         )
 
     def tokenize(self, prompt: str, return_tokens: bool = False) -> dict:
@@ -420,19 +484,28 @@ class DeploymentModelInference(BaseModelInference):
         guardrails: bool = False,
         guardrails_hap_params: dict | None = None,
         guardrails_pii_params: dict | None = None,
+        guardrails_granite_guardian_params: dict | None = None,
     ) -> dict:
         payload: dict = {
             "input": prompt,
         }
 
         if guardrails:
-            if guardrails_hap_params is None:
+            if (
+                guardrails_hap_params is None
+                and guardrails_granite_guardian_params is None
+            ):
                 guardrails_hap_params = dict(
                     input=True, output=True
                 )  # HAP enabled if guardrails = True
 
             for guardrail_type, guardrails_params in zip(
-                ("hap", "pii"), (guardrails_hap_params, guardrails_pii_params)
+                ("hap", "pii", "granite_guardian"),
+                (
+                    guardrails_hap_params,
+                    guardrails_pii_params,
+                    guardrails_granite_guardian_params,
+                ),
             ):
                 if guardrails_params is not None:
                     if "moderations" not in payload:
@@ -502,6 +575,7 @@ class DeploymentModelInference(BaseModelInference):
         guardrails: bool = False,
         guardrails_hap_params: dict | None = None,
         guardrails_pii_params: dict | None = None,
+        guardrails_granite_guardian_params: dict | None = None,
     ) -> dict:
         payload: dict = {
             "input": prompt,
@@ -521,6 +595,10 @@ class DeploymentModelInference(BaseModelInference):
 
             if guardrails_pii_params is not None:
                 payload["moderations"].update({"pii": guardrails_pii_params})
+            if guardrails_granite_guardian_params is not None:
+                payload["moderations"].update(
+                    {"granite_guardian": guardrails_granite_guardian_params}
+                )
 
         if params is not None:
             parameters = params
