@@ -4,7 +4,14 @@
 #  -----------------------------------------------------------------------------------------
 from __future__ import annotations
 
-from typing import TYPE_CHECKING, Generator, cast, overload, Literal, AsyncGenerator
+from typing import (
+    TYPE_CHECKING,
+    Generator,
+    cast,
+    overload,
+    Literal,
+    AsyncGenerator,
+)
 from enum import Enum
 from warnings import warn
 
@@ -20,7 +27,6 @@ from ibm_watsonx_ai._wrappers.requests import (
     _get_httpx_client,
     _get_async_client,
 )
-from ibm_watsonx_ai.messages.messages import Messages
 from ibm_watsonx_ai.foundation_models.schema import (
     TextChatParameters,
     TextGenParameters,
@@ -287,22 +293,48 @@ class ModelInference(BaseModelInference):
         :return: scoring result containing generated chat content.
         :rtype: dict
 
-        **Example:**
+        **Examples:**
 
-        .. code-block:: python
+        .. tab-set::
 
-            messages = [
-                {"role": "system", "content": "You are a helpful assistant."},
-                {"role": "user", "content": "Who won the world series in 2020?"}
-            ]
-            generated_response = model.chat(messages=messages)
+            .. tab-item:: Simple conversation
 
-            # Print all response
-            print(generated_response)
+                .. code-block:: python
 
-            # Print only content
-            print(generated_response['choices'][0]['message']['content'])
+                    messages = [
+                        {"role": "system", "content": "You are a helpful assistant."},
+                        {"role": "user", "content": "Who won the world series in 2020?"}
+                    ]
+                    generated_response = model.chat(messages=messages)
 
+                    # Print full response
+                    print(generated_response)
+
+                    # Print only content
+                    print(generated_response["choices"][0]["message"]["content"])
+
+            .. tab-item:: Control messages
+
+                .. note::
+
+                    Control messages enable inclusion of enhanced reasoning to chat response content.
+                    They are available only for IBM Granite reasoning models 3.2 and above.
+                    For more information, visit: https://www.ibm.com/granite/docs/models/granite/
+
+                .. code-block:: python
+
+                    messages = [
+                        {"role": "control", "content": "thinking"},
+                        {"role": "system", "content": "You are a helpful assistant."},
+                        {"role": "user", "content": "Who won the world series in 2020?"}
+                    ]
+                    generated_response = model.chat(messages=messages)
+
+                    # Print full response
+                    print(generated_response)
+
+                    # Print only content
+                    print(generated_response["choices"][0]["message"]["content"])
         """
         self._validate_type(messages, "messages", list, True)
         self._validate_type(params, "params", [dict, TextChatParameters], False, True)
@@ -532,6 +564,7 @@ class ModelInference(BaseModelInference):
         concurrency_limit: int = ...,
         async_mode: Literal[False] = ...,
         validate_prompt_variables: bool = ...,
+        guardrails_granite_guardian_params: dict | None = ...,
     ) -> dict | list[dict]: ...
 
     @overload
@@ -545,6 +578,7 @@ class ModelInference(BaseModelInference):
         concurrency_limit: int,
         async_mode: Literal[True],
         validate_prompt_variables: bool,
+        guardrails_granite_guardian_params: dict | None,
     ) -> Generator: ...
 
     @overload
@@ -558,6 +592,7 @@ class ModelInference(BaseModelInference):
         concurrency_limit: int = ...,
         async_mode: bool = ...,
         validate_prompt_variables: bool = ...,
+        guardrails_granite_guardian_params: dict | None = ...,
     ) -> dict | list[dict] | Generator: ...
 
     def generate(
@@ -570,6 +605,7 @@ class ModelInference(BaseModelInference):
         concurrency_limit: int = BaseModelInference.DEFAULT_CONCURRENCY_LIMIT,
         async_mode: bool = False,
         validate_prompt_variables: bool = True,
+        guardrails_granite_guardian_params: dict | None = None,
     ) -> dict | list[dict] | Generator:
         """Generates a completion text as generated_text after getting a text prompt as input and parameters for the
         selected model (model_id) or deployment (deployment_id). For prompt template deployment, `prompt` should be None.
@@ -600,16 +636,68 @@ class ModelInference(BaseModelInference):
                                           This parameter is only applicable in a Prompt Template Asset deployment scenario and should not be changed for different cases, defaults to True
         :type validate_prompt_variables: bool, optional
 
+        :param guardrails_granite_guardian_params: parameters for Granite Guardian moderations
+        :type guardrails_granite_guardian_params: dict, optional
+
         :return: scoring result the contains the generated content
         :rtype: dict
 
-        **Example:**
+        **Examples:**
 
-        .. code-block:: python
+        .. tab-set::
 
-            q = "What is 1 + 1?"
-            generated_response = model_inference.generate(prompt=q)
-            print(generated_response['results'][0]['generated_text'])
+            .. tab-item:: Generate
+
+                .. code-block:: python
+
+                    q = "What is 1 + 1?"
+
+                    generated_response = model_inference.generate(prompt=q)
+
+                    print(generated_response['results'][0]['generated_text'])
+
+            .. tab-item:: Generate with Params
+
+                .. code-block:: python
+
+                    from ibm_watsonx_ai.foundation_models.schema import (
+                        TextGenParameters,
+                        TextGenDecodingMethod
+                    )
+
+                    q = "What is 1 + 1?"
+
+                    generate_params = TextGenParameters(
+                        decoding_method=TextGenDecodingMethod.SAMPLE,
+                        temperature=0.8,
+                        top_p=0.3
+                    )
+
+                    generated_response = model_inference.generate(
+                        prompt=q,
+                        params=generate_params,
+                    )
+
+                    print(generated_response['results'][0]['generated_text'])
+
+            .. tab-item:: Generate with Granite Guardian
+
+                .. code-block:: python
+
+                    from ibm_watsonx_ai.metanames import GenTextModerationsMetaNames
+
+                    q = "<YOUR-QUESTION-TO-DETECT>"
+
+                    guardrails_granite_guardian_params = {
+                        GenTextModerationsMetaNames.INPUT: True,
+                        GenTextModerationsMetaNames.THRESHOLD: 0.01
+                    }
+
+                    generated_response = model_inference.generate(
+                        prompt=q,
+                        guardrails=True,
+                        guardrails_granite_guardian_params=guardrails_granite_guardian_params,
+                    )
 
         """
         self._validate_type(params, "params", [dict, TextGenParameters], False, True)
@@ -644,6 +732,7 @@ class ModelInference(BaseModelInference):
             concurrency_limit=concurrency_limit,
             async_mode=async_mode,
             validate_prompt_variables=validate_prompt_variables,
+            guardrails_granite_guardian_params=guardrails_granite_guardian_params,
         )
 
     async def _agenerate_single(  # type: ignore[override]
@@ -653,6 +742,7 @@ class ModelInference(BaseModelInference):
         guardrails: bool = False,
         guardrails_hap_params: dict | None = None,
         guardrails_pii_params: dict | None = None,
+        guardrails_granite_guardian_params: dict | None = None,
     ) -> dict:
         """
         Given a text prompt as input, and parameters the selected inference
@@ -666,6 +756,7 @@ class ModelInference(BaseModelInference):
             guardrails=guardrails,
             guardrails_hap_params=guardrails_hap_params,
             guardrails_pii_params=guardrails_pii_params,
+            guardrails_granite_guardian_params=guardrails_granite_guardian_params,
         )
 
     async def agenerate_stream(
@@ -676,6 +767,7 @@ class ModelInference(BaseModelInference):
         guardrails_hap_params: dict | None = None,
         guardrails_pii_params: dict | None = None,
         validate_prompt_variables: bool = True,
+        guardrails_granite_guardian_params: dict | None = None,
     ) -> AsyncGenerator:
         """Generates a stream as agenerate_stream after getting a text prompt as input and
         parameters for the selected model (model_id). For prompt template deployment, `prompt` should be None.
@@ -698,6 +790,9 @@ class ModelInference(BaseModelInference):
                                           This parameter is only applicable in a Prompt Template Asset deployment scenario and should not be changed for different cases, defaults to True
         :type validate_prompt_variables: bool
 
+        :param guardrails_granite_guardian_params: parameters for Granite Guardian moderations
+        :type guardrails_granite_guardian_params: dict, optional
+
         :return: scoring result that contains the generated content
         :rtype: AsyncGenerator
 
@@ -745,6 +840,7 @@ class ModelInference(BaseModelInference):
         guardrails_pii_params: dict | None = ...,
         concurrency_limit: int = ...,
         validate_prompt_variables: bool = ...,
+        guardrails_granite_guardian_params: dict | None = ...,
     ) -> str: ...
 
     @overload
@@ -758,6 +854,7 @@ class ModelInference(BaseModelInference):
         guardrails_pii_params: dict | None = ...,
         concurrency_limit: int = ...,
         validate_prompt_variables: bool = ...,
+        guardrails_granite_guardian_params: dict | None = ...,
     ) -> list[str]: ...
 
     @overload
@@ -771,6 +868,7 @@ class ModelInference(BaseModelInference):
         guardrails_pii_params: dict | None,
         concurrency_limit: int,
         validate_prompt_variables: bool,
+        guardrails_granite_guardian_params: dict | None,
     ) -> list[dict] | dict: ...
 
     @overload
@@ -784,6 +882,7 @@ class ModelInference(BaseModelInference):
         guardrails_pii_params: dict | None,
         concurrency_limit: int,
         validate_prompt_variables: bool,
+        guardrails_granite_guardian_params: dict | None,
     ) -> str | list | dict: ...
 
     def generate_text(
@@ -796,6 +895,7 @@ class ModelInference(BaseModelInference):
         guardrails_pii_params: dict | None = None,
         concurrency_limit: int = BaseModelInference.DEFAULT_CONCURRENCY_LIMIT,
         validate_prompt_variables: bool = True,
+        guardrails_granite_guardian_params: dict | None = None,
     ) -> str | list | dict:
         """Generates a completion text as generated_text after getting a text prompt as input and
         parameters for the selected model (model_id). For prompt template deployment, `prompt` should be None.
@@ -824,6 +924,9 @@ class ModelInference(BaseModelInference):
                                           This parameter is only applicable in a Prompt Template Asset deployment scenario and should not be changed for different cases, defaults to True
         :type validate_prompt_variables: bool
 
+        :param guardrails_granite_guardian_params: parameters for Granite Guardian moderations
+        :type guardrails_granite_guardian_params: dict, optional
+
         :return: generated content
         :rtype: str | list | dict
 
@@ -837,14 +940,43 @@ class ModelInference(BaseModelInference):
 
                 warnings.filterwarnings("always", category=HAPDetectionWarning)
 
+        **Examples:**
 
-        **Example:**
+        .. tab-set::
 
-        .. code-block:: python
+            .. tab-item:: Generate Text
 
-            q = "What is 1 + 1?"
-            generated_text = model_inference.generate_text(prompt=q)
-            print(generated_text)
+                .. code-block:: python
+
+                    q = "What is 1 + 1?"
+
+                    generated_text = model_inference.generate_text(prompt=q)
+
+                    print(generated_text)
+
+            .. tab-item:: Generate Text with Params
+
+                .. code-block:: python
+
+                    from ibm_watsonx_ai.foundation_models.schema import (
+                        TextGenParameters,
+                        TextGenDecodingMethod
+                    )
+
+                    q = "What is 1 + 1?"
+
+                    generate_params = TextGenParameters(
+                        decoding_method=TextGenDecodingMethod.SAMPLE,
+                        temperature=0.8,
+                        top_p=0.3
+                    )
+
+                    generated_text = model_inference.generate_text(
+                        prompt=q,
+                        params=generate_params
+                    )
+
+                    print(generated_text)
 
         """
         metadata = ModelInference.generate(
@@ -856,6 +988,7 @@ class ModelInference(BaseModelInference):
             guardrails_pii_params=guardrails_pii_params,
             concurrency_limit=concurrency_limit,
             validate_prompt_variables=validate_prompt_variables,
+            guardrails_granite_guardian_params=guardrails_granite_guardian_params,
         )
         if raw_response:
             return metadata
@@ -877,6 +1010,7 @@ class ModelInference(BaseModelInference):
         guardrails_hap_params: dict | None = None,
         guardrails_pii_params: dict | None = None,
         validate_prompt_variables: bool = True,
+        guardrails_granite_guardian_params: dict | None = None,
     ) -> Generator:
         """Generates a streamed text as generate_text_stream after getting a text prompt as input and
         parameters for the selected model (model_id). For prompt template deployment, `prompt` should be None.
@@ -902,6 +1036,9 @@ class ModelInference(BaseModelInference):
                                           This parameter is only applicable in a Prompt Template Asset deployment scenario and should not be changed for different cases, defaults to True
         :type validate_prompt_variables: bool
 
+        :param guardrails_granite_guardian_params: parameters for Granite Guardian moderations
+        :type guardrails_granite_guardian_params: dict, optional
+
         :return: scoring result that contains the generated content
         :rtype: generator
 
@@ -915,15 +1052,44 @@ class ModelInference(BaseModelInference):
 
                 warnings.filterwarnings("always", category=HAPDetectionWarning)
 
-        **Example:**
+        **Examples:**
 
-        .. code-block:: python
+        .. tab-set::
 
-            q = "Write an epigram about the sun"
-            generated_response = model_inference.generate_text_stream(prompt=q)
+            .. tab-item:: Generate Text Stream
 
-            for chunk in generated_response:
-                print(chunk, end='', flush=True)
+                .. code-block:: python
+
+                    q = "Write an epigram about the sun"
+                    generated_response = model_inference.generate_text_stream(prompt=q)
+
+                    for chunk in generated_response:
+                        print(chunk, end='', flush=True)
+
+            .. tab-item:: Generate Text Stream with Params
+
+                .. code-block:: python
+
+                    from ibm_watsonx_ai.foundation_models.schema import (
+                        TextGenParameters,
+                        TextGenDecodingMethod
+                    )
+
+                    q = "Write an epigram about the sun"
+
+                    generate_params = TextGenParameters(
+                        decoding_method=TextGenDecodingMethod.SAMPLE,
+                        temperature=0.8,
+                        top_p=0.3
+                    )
+
+                    generated_response = model_inference.generate_text_stream(
+                        prompt=q,
+                        params=generate_params,
+                    )
+
+                    for chunk in generated_response:
+                        print(chunk, end='', flush=True)
 
         """
         self._validate_type(params, "params", [dict, TextGenParameters], False, True)
@@ -936,6 +1102,7 @@ class ModelInference(BaseModelInference):
             guardrails_hap_params=guardrails_hap_params,
             guardrails_pii_params=guardrails_pii_params,
             validate_prompt_variables=validate_prompt_variables,
+            guardrails_granite_guardian_params=guardrails_granite_guardian_params,
         )
 
     def tokenize(self, prompt: str, return_tokens: bool = False) -> dict:
@@ -1072,6 +1239,7 @@ class ModelInference(BaseModelInference):
         guardrails_hap_params: dict | None = None,
         guardrails_pii_params: dict | None = None,
         validate_prompt_variables: bool = True,
+        guardrails_granite_guardian_params: dict | None = None,
     ) -> dict:
         """Generate a response in an asynchronous manner.
 
@@ -1093,6 +1261,9 @@ class ModelInference(BaseModelInference):
                                           This parameter is only applicable in a Prompt Template Asset deployment scenario and should not be changed for different cases, defaults to True
         :type validate_prompt_variables: bool, optional
 
+        :param guardrails_granite_guardian_params: parameters for Granite Guardian moderations
+        :type guardrails_granite_guardian_params: dict, optional
+
         :return: raw response that contains the generated content
         :rtype: dict
         """
@@ -1104,6 +1275,7 @@ class ModelInference(BaseModelInference):
             guardrails_hap_params=guardrails_hap_params,
             guardrails_pii_params=guardrails_pii_params,
             validate_prompt_variables=validate_prompt_variables,
+            guardrails_granite_guardian_params=guardrails_granite_guardian_params,
         )
 
     async def aclose_persistent_connection(self) -> None:
