@@ -4,17 +4,17 @@
 #  -----------------------------------------------------------------------------------------
 
 from __future__ import annotations
-
-from typing import TYPE_CHECKING, cast, Any, overload, Literal, TypeAlias
+from dataclasses import dataclass
+from typing import TYPE_CHECKING, cast, Any, overload, Literal, TypeAlias, Mapping
 
 if TYPE_CHECKING:
     import langchain
     from langchain.prompts import PromptTemplate as LcPromptTemplate
     from langchain.prompts import ChatPromptTemplate
-from dataclasses import dataclass
 
-import pandas
 import inspect
+import copy
+import pandas
 
 from ibm_watsonx_ai.foundation_models.prompts.base_prompt_template import (
     BasePromptTemplate,
@@ -22,7 +22,6 @@ from ibm_watsonx_ai.foundation_models.prompts.base_prompt_template import (
 from ibm_watsonx_ai.foundation_models.prompts.base_prompt import (
     BasePrompt,
 )
-import ibm_watsonx_ai._wrappers.requests as requests
 from ibm_watsonx_ai import APIClient, Credentials
 from ibm_watsonx_ai.wml_client_error import (
     WMLClientError,
@@ -163,13 +162,11 @@ class FreeformPromptTemplate(BasePromptTemplate):
 
         :raises ValidationError: raised when input_variables do not fit the placeholders in the input body.
         """
-        input_variables = (
-            self.input_variables if self.input_variables is not None else []
-        )
+        input_variables = self.input_variables or []
 
         self._validate_prompt(
             input_variables,
-            (self.input_text if self.input_text is not None else ""),
+            self.input_text or "",
         )
 
 
@@ -224,7 +221,7 @@ class PromptTemplate(BasePromptTemplate):
     :type output_prefix: str, optional
 
     :param examples: examples that might help the model adjust the response; [[input1, output1], ...], defaults to None.
-    :type examples: list[list[str]]], optional
+    :type examples: list[list[str]], optional
 
     :param validate_template: if True, the prompt template is validated for the presence of input variables, defaults to True.
     :type validate_template: bool, optional
@@ -296,7 +293,7 @@ class PromptTemplate(BasePromptTemplate):
         self.instruction = instruction
         self.input_prefix = input_prefix
         self.output_prefix = output_prefix
-        self.examples = examples.copy() if examples is not None else examples
+        self.examples = copy.deepcopy(examples) if examples is not None else examples
 
         supported_pt_kwargs = ["input_mode", "external_information"]
         unsupported_pt_keys = [
@@ -322,9 +319,7 @@ class PromptTemplate(BasePromptTemplate):
 
         :raises ValidationError: raised when input_variables do not fit the placeholders in the input body.
         """
-        input_variables = (
-            self.input_variables if self.input_variables is not None else []
-        )
+        input_variables = self.input_variables or []
         template_text = " ".join(
             filter(None, [self.instruction, self.input_prefix, self.output_prefix])
         )
@@ -334,11 +329,7 @@ class PromptTemplate(BasePromptTemplate):
 
         self._validate_prompt(
             input_variables,
-            (
-                template_text + self.input_text
-                if self.input_text is not None
-                else template_text
-            ),
+            template_text + (self.input_text or ""),
         )
 
 
@@ -403,9 +394,21 @@ class DetachedPromptTemplate(BasePromptTemplate):
     :param detached_model_url: URL for the external model, defaults to None
     :type detached_model_url: str | None, optional
 
-    :param validate_template: if True, the prompt template is validated for the presence of input variables, defaults to True.
+    :param validate_template: if True, the prompt template is validated for the presence of input variables, defaults to True
     :type validate_template: bool, optional
 
+    :param instruction: instruction for the model, defaults to None
+    :type instruction: str, optional
+
+    :param input_prefix: prefix string placed before the input text, defaults to None
+    :type input_prefix: str, optional
+
+    :param output_prefix: prefix placed before the model response, defaults to None
+    :type output_prefix: str, optional
+
+    :param examples: examples that might help the model adjust the response; [[input1, output1], ...], defaults to None
+    :type examples: list[list[str]], optional
+
     :raises ValidationError: raised when the set of input_variables is not consistent with the input variables present in the template.
                              Raised only when `validate_template` is set to True.
 
@@ -473,6 +476,10 @@ class DetachedPromptTemplate(BasePromptTemplate):
         detached_model_name: str | None = None,
         detached_model_url: str | None = None,
         validate_template: bool = True,
+        instruction: str | None = None,
+        input_prefix: str | None = None,
+        output_prefix: str | None = None,
+        examples: list[list[str]] | None = None,
     ) -> None:
         super().__init__(
             input_mode=self._input_mode,
@@ -485,6 +492,11 @@ class DetachedPromptTemplate(BasePromptTemplate):
             input_text=input_text,
             input_variables=input_variables,
         )
+        self.instruction = instruction
+        self.input_prefix = input_prefix
+        self.output_prefix = output_prefix
+        self.examples = copy.deepcopy(examples) if examples is not None else examples
+
         self.detached_prompt_id = detached_prompt_id
         self.detached_model_id = detached_model_id
         self.detached_model_provider = detached_model_provider
@@ -502,15 +514,19 @@ class DetachedPromptTemplate(BasePromptTemplate):
     def _validation(self) -> None:
         """Validate the template structure.
 
-        :raises ValidationError: raised when input_variables does not fit the placeholders in the input body.
+        :raises ValidationError: raised when input_variables do not fit the placeholders in the input body.
         """
-        input_variables = (
-            self.input_variables if self.input_variables is not None else []
+        input_variables = self.input_variables or []
+        template_text = " ".join(
+            filter(None, [self.instruction, self.input_prefix, self.output_prefix])
         )
+        if self.examples:
+            for example in self.examples:
+                template_text += " ".join(example)
 
         self._validate_prompt(
             input_variables,
-            (self.input_text if self.input_text is not None else ""),
+            template_text + (self.input_text or ""),
         )
 
 
@@ -739,7 +755,7 @@ class PromptTemplateManager(WMLResource):
             )
 
         data: dict = dict()
-        if isinstance(prompt_template, PromptTemplate):
+        if isinstance(prompt_template, PromptTemplate | DetachedPromptTemplate):
 
             if prompt_template.instruction is not None:
                 data.update({"instruction": prompt_template.instruction})
@@ -755,69 +771,71 @@ class PromptTemplateManager(WMLResource):
                 )
                 data.update({"examples": prompt_template.examples})
 
-        elif isinstance(prompt_template, DetachedPromptTemplate):
-            external_information: dict = dict()
-            PromptTemplateManager._validate_type(
-                prompt_template.detached_prompt_id, "detached_prompt_id", str, True
-            )
-            PromptTemplateManager._validate_type(
-                prompt_template.detached_model_id, "detached_model_id", str, True
-            )
-            PromptTemplateManager._validate_type(
-                prompt_template.detached_model_provider,
-                "detached_model_provider",
-                str,
-                True,
-            )
-            external_information.update(
-                {
-                    "external_prompt_id": prompt_template.detached_prompt_id,
-                    "external_model_id": prompt_template.detached_model_id,
-                    "external_model_provider": prompt_template.detached_model_provider,
-                }
-            )
-
-            if prompt_template.detached_prompt_additional_information is not None:
+            if isinstance(prompt_template, DetachedPromptTemplate):
+                external_information: dict = dict()
                 PromptTemplateManager._validate_type(
-                    prompt_template.detached_prompt_url,
-                    "detached_prompt_url",
+                    prompt_template.detached_prompt_id, "detached_prompt_id", str, True
+                )
+                PromptTemplateManager._validate_type(
+                    prompt_template.detached_model_id, "detached_model_id", str, True
+                )
+                PromptTemplateManager._validate_type(
+                    prompt_template.detached_model_provider,
+                    "detached_model_provider",
                     str,
                     True,
                 )
                 external_information.update(
                     {
-                        "external_prompt": {
-                            "url": prompt_template.detached_prompt_url,
-                            "additional_information": prompt_template.detached_prompt_additional_information,
-                        }
-                    }
-                )
-            if (
-                prompt_template.detached_model_name is not None
-                or prompt_template.detached_model_url is not None
-            ):
-                PromptTemplateManager._validate_type(
-                    prompt_template.detached_model_url,
-                    "detached_model_url",
-                    str,
-                    True,
-                )
-                PromptTemplateManager._validate_type(
-                    prompt_template.detached_model_name,
-                    "detached_model_name",
-                    str,
-                    True,
-                )
-                external_information.update(
-                    {
-                        "external_model": {
-                            "url": prompt_template.detached_model_url,
-                            "name": prompt_template.detached_model_name,
-                        }
+                        "external_prompt_id": prompt_template.detached_prompt_id,
+                        "external_model_id": prompt_template.detached_model_id,
+                        "external_model_provider": prompt_template.detached_model_provider,
                     }
                 )
 
-            json_data["prompt"].update({"external_information": external_information})
+                if prompt_template.detached_prompt_additional_information is not None:
+                    PromptTemplateManager._validate_type(
+                        prompt_template.detached_prompt_url,
+                        "detached_prompt_url",
+                        str,
+                        True,
+                    )
+                    external_information.update(
+                        {
+                            "external_prompt": {
+                                "url": prompt_template.detached_prompt_url,
+                                "additional_information": prompt_template.detached_prompt_additional_information,
+                            }
+                        }
+                    )
+                if (
+                    prompt_template.detached_model_name is not None
+                    or prompt_template.detached_model_url is not None
+                ):
+                    PromptTemplateManager._validate_type(
+                        prompt_template.detached_model_url,
+                        "detached_model_url",
+                        str,
+                        True,
+                    )
+                    PromptTemplateManager._validate_type(
+                        prompt_template.detached_model_name,
+                        "detached_model_name",
+                        str,
+                        True,
+                    )
+                    external_information.update(
+                        {
+                            "external_model": {
+                                "url": prompt_template.detached_model_url,
+                                "name": prompt_template.detached_model_name,
+                            }
+                        }
+                    )
+
+                json_data["prompt"].update(
+                    {"external_information": external_information}
+                )
 
         elif isinstance(prompt_template, ChatPrompt):
 
@@ -888,6 +906,10 @@ class PromptTemplateManager(WMLResource):
                     ),
                     input_variables=response.get("prompt_variables"),
                     input_text=prompt_field.get("input", [[None, None]])[0][0],
+                    instruction=data_field.get("instruction"),
+                    input_prefix=data_field.get("input_prefix"),
+                    output_prefix=data_field.get("output_prefix"),
+                    examples=data_field.get("examples"),
                     detached_prompt_id=external_information_field.get(
                         "external_prompt_id"
                     ),
@@ -965,7 +987,7 @@ class PromptTemplateManager(WMLResource):
         :rtype: List
         """
         headers = self._client._get_headers()
-        url = self._client.service_instance._href_definitions.get_prompts_all_href()
+        url = self._client._href_definitions.get_prompts_all_href()
         json_data: dict[str, int | str] = {
             "query": "asset.asset_type:wx_prompt",
             "sort": "-asset.created_at<string>",
@@ -982,7 +1004,7 @@ class PromptTemplateManager(WMLResource):
         prompts_list = []
         bookmark = True
         while bookmark is not None:
-            response = requests.post(
+            response = self._client.httpx_client.post(
                 url=url, json=json_data, headers=headers, params=self._params
             )
             details_json = self._handle_response(200, "Get next details", response)
@@ -1009,16 +1031,21 @@ class PromptTemplateManager(WMLResource):
         :rtype: dict
         """
         headers = self._client._get_headers()
-        params = self._params | {"prompt_id": prompt_id, "force": force}
+        params: Mapping[str, str | int | float | bool | None] = self._params | {
+            "prompt_id": prompt_id,
+            "force": force,
+        }
         json_data = {"locked": locked}
 
         url = (
-            self._client.service_instance._href_definitions.get_prompts_href(
+            self._client._href_definitions.get_prompts_href(
                 ga_api=self._client._use_pta_ga_api
             )
             + f"/{prompt_id}/lock"
         )
-        response = requests.put(url=url, json=json_data, headers=headers, params=params)
+        response = self._client.httpx_client.put(
+            url=url, json=json_data, headers=headers, params=params
+        )
 
         return self._handle_response(200, "change_lock", response)
 
@@ -1067,7 +1094,9 @@ class PromptTemplateManager(WMLResource):
         ] = PromptTemplateFormats.PROMPTTEMPLATE,
         *,
         prompt_variables: dict[str, str] | None = None,
-    ) -> FreeformPromptTemplate | PromptTemplate | DetachedPromptTemplate: ...
+    ) -> (
+        FreeformPromptTemplate | PromptTemplate | DetachedPromptTemplate | ChatPrompt
+    ): ...
 
     def load_prompt(
         self,
@@ -1109,7 +1138,7 @@ class PromptTemplateManager(WMLResource):
         headers = self._client._get_headers()
         params = self._params | {"prompt_id": prompt_id}
         url = (
-            self._client.service_instance._href_definitions.get_prompts_href(
+            self._client._href_definitions.get_prompts_href(
                 ga_api=self._client._use_pta_ga_api
             )
             + f"/{prompt_id}"
@@ -1119,12 +1148,16 @@ class PromptTemplateManager(WMLResource):
             astype = astype.value
 
         if astype == "prompt":
-            response = requests.get(url=url, headers=headers, params=params)
+            response = self._client.httpx_client.get(
+                url=url, headers=headers, params=params
+            )
             return self._from_json_to_prompt(
                 self._handle_response(200, "_load_json_prompt", response)
             )
         elif astype in ("langchain", "string"):
-            response = requests.post(url=url + "/input", headers=headers, params=params)
+            response = self._client.httpx_client.post(
+                url=url + "/input", headers=headers, params=params
+            )
             response_input = self._handle_response(200, "load_prompt", response).get(
                 "input"
             )
@@ -1137,9 +1170,11 @@ class PromptTemplateManager(WMLResource):
                         else response_input.format(**prompt_variables)
                     )
                 except KeyError as key:
-                    raise PromptVariablesError(str(key))
+                    raise PromptVariablesError(str(key)) from key
             else:
-                response = requests.get(url=url, headers=headers, params=params)
+                response = self._client.httpx_client.get(
+                    url=url, headers=headers, params=params
+                )
                 response_json = self._handle_response(
                     200, "_load_json_prompt", response
                 )
@@ -1402,6 +1437,21 @@ class PromptTemplateManager(WMLResource):
                             input_text=get_metadata_value(
                                 prompt_template, "input_text", prompt_template.template
                             ),
+                            instruction=get_metadata_value(
+                                prompt_template, "instruction", None
+                            ),
+                            input_prefix=get_metadata_value(
+                                prompt_template, "input_prefix", None
+                            ),
+                            output_prefix=get_metadata_value(
+                                prompt_template, "output_prefix", None
+                            ),
+                            examples=get_metadata_value(
+                                prompt_template,
+                                "examples",
+                                None,
+                                must_be_nested_list=True,
+                            ),
                             input_variables=get_metadata_value(
                                 prompt_template,
                                 "input_variables",
@@ -1441,15 +1491,15 @@ class PromptTemplateManager(WMLResource):
 
         json_data.update(self._create_request_body(prompt_template))
 
-        url = self._client.service_instance._href_definitions.get_prompts_href(
+        url = self._client._href_definitions.get_prompts_href(
             ga_api=self._client._use_pta_ga_api
         )
-        response = requests.post(
+        response = self._client.httpx_client.post(
             url=url, json=json_data, headers=headers, params=self._params
         )
-        response = self._handle_response(201, "store_prompt", response)
+        res = self._handle_response(201, "store_prompt", response)
 
-        return self._from_json_to_prompt(response)
+        return self._from_json_to_prompt(res)
 
     def delete_prompt(self, prompt_id: str, *, force: bool = False) -> str:
         """Remove a prompt template from a project or space.
@@ -1476,12 +1526,14 @@ class PromptTemplateManager(WMLResource):
         params = self._params | {"prompt_id": prompt_id}
 
         url = (
-            self._client.service_instance._href_definitions.get_prompts_href(
+            self._client._href_definitions.get_prompts_href(
                 ga_api=self._client._use_pta_ga_api
             )
             + f"/{prompt_id}"
         )
-        response = requests.delete(url=url, headers=headers, params=params)
+        response = self._client.httpx_client.delete(
+            url=url, headers=headers, params=params
+        )
 
         return self._handle_response(204, "delete_prompt", response)  # type: ignore[return-value]
 
@@ -1544,13 +1596,13 @@ class PromptTemplateManager(WMLResource):
         new_body.update(self._create_request_body(current_prompt_template))
 
         url = (
-            self._client.service_instance._href_definitions.get_prompts_href(
+            self._client._href_definitions.get_prompts_href(
                 ga_api=self._client._use_pta_ga_api
             )
             + f"/{prompt_id}"
         )
 
-        response = requests.patch(
+        response = self._client.httpx_client.patch(
             url=url, json=new_body, headers=headers, params=params
         )
         return self._handle_response(200, "update_prompt", response)
@@ -1573,13 +1625,15 @@ class PromptTemplateManager(WMLResource):
         headers = self._client._get_headers()
         params = self._params | {"prompt_id": prompt_id}
         url = (
-            self._client.service_instance._href_definitions.get_prompts_href(
+            self._client._href_definitions.get_prompts_href(
                 ga_api=self._client._use_pta_ga_api
             )
             + f"/{prompt_id}/lock"
         )
 
-        response = requests.get(url=url, headers=headers, params=params)
+        response = self._client.httpx_client.get(
+            url=url, headers=headers, params=params
+        )
 
         return self._handle_response(200, "get_lock", response)
 
@@ -1665,13 +1719,13 @@ class PromptTemplateManager(WMLResource):
         params = self._params
 
         url = (
-            self._client.service_instance._href_definitions.get_prompts_href(
+            self._client._href_definitions.get_prompts_href(
                 ga_api=self._client._use_pta_ga_api
             )
             + f"/{prompt_id}/chat_items"
         )
 
-        response = requests.post(
+        response = self._client.httpx_client.post(
             url=url, json=chat_items, headers=headers, params=params
         )
         if response.status_code == 201:
