@@ -80,6 +80,7 @@ class FineTuningParams:
     response_template: str | None = None
     gpu: dict | None = None
     peft_parameters: dict | None = None
+    gradient_checkpointing: bool | None = None
 
     def to_dict(self) -> dict:
         return {key: value for key, value in asdict(self).items() if value is not None}
@@ -670,6 +671,9 @@ class HAPDetectionWarning(UserWarning): ...
 class PIIDetectionWarning(UserWarning): ...
 
 
+class GraniteGuardianDetectionWarning(UserWarning): ...
+
+
 class LifecycleWarning(UserWarning): ...
 
 
@@ -727,7 +731,7 @@ def _copy_function(func: Callable) -> Callable:
 def _is_fine_tuning_endpoint_available(api_client: APIClient) -> bool:
 
     try:
-        url = api_client.service_instance._href_definitions.get_fine_tunings_href()
+        url = api_client._href_definitions.get_fine_tunings_href()
 
         response_fine_tuning_api = api_client._session.get(
             url=f"{url}?limit=1",
