@@ -117,7 +117,7 @@ class Functions(WMLResource):
                 )
 
             response_post = requests.post(
-                self._client.service_instance._href_definitions.get_functions_href(),
+                self._client._href_definitions.get_functions_href(),
                 json=function_metadata,
                 params=self._client._params(skip_for_create=True),
                 headers=self._client._get_headers(),
@@ -131,7 +131,7 @@ class Functions(WMLResource):
             ##TODO_V4 Take care of this since the latest swagger endpoint is not working
 
             function_content_url = (
-                self._client.service_instance._href_definitions.get_function_href(
+                self._client._href_definitions.get_function_href(
                     details["metadata"]["id"]
                 )
                 + "/code"
@@ -201,12 +201,10 @@ class Functions(WMLResource):
         details = self.get_details(function_id)
 
         patch_payload = self.ConfigurationMetaNames._generate_patch_payload(
-            details["entity"], changes, with_validation=True
+            details, changes, with_validation=True
         )
 
-        url = self._client.service_instance._href_definitions.get_function_href(
-            function_id
-        )
+        url = self._client._href_definitions.get_function_href(function_id)
         headers = self._client._get_headers()
         response = requests.patch(
             url, json=patch_payload, params=self._client._params(), headers=headers
@@ -240,10 +238,7 @@ class Functions(WMLResource):
         )
         try:
             function_content_url = (
-                self._client.service_instance._href_definitions.get_function_href(
-                    function_id
-                )
-                + "/code"
+                self._client._href_definitions.get_function_href(function_id) + "/code"
             )
 
             put_header = self._client._get_headers(no_content_type=True)
@@ -312,17 +307,10 @@ class Functions(WMLResource):
         Functions._validate_type(function_id, "function_id", str, True)
         Functions._validate_type(filename, "filename", str, True)
 
-        artifact_url = (
-            self._client.service_instance._href_definitions.get_function_href(
-                function_id
-            )
-        )
+        artifact_url = self._client._href_definitions.get_function_href(function_id)
 
         artifact_content_url = (
-            self._client.service_instance._href_definitions.get_function_href(
-                function_id
-            )
-            + "/code"
+            self._client._href_definitions.get_function_href(function_id) + "/code"
         )
 
         try:
@@ -384,13 +372,16 @@ class Functions(WMLResource):
             )
 
     def delete(
-        self, function_id: str | None = None, **kwargs: Any
+        self, function_id: str | None = None, force: bool = False, **kwargs: Any
     ) -> Literal["SUCCESS"]:
         """Delete a stored function.
 
         :param function_id: stored function ID
         :type function_id: str
 
+        :param force: if True, the delete operation will proceed even when the function deployment exists, defaults to False
+        :type force: bool, optional
+
         :return: status "SUCCESS" if deletion is successful
         :rtype: Literal["SUCCESS"]
 
@@ -401,40 +392,30 @@ class Functions(WMLResource):
             client._functions.delete(function_id)
         """
         function_id = _get_id_from_deprecated_uid(kwargs, function_id, "function")
-
-        ##For CP4D, check if either spce or project ID is set
-        self._client._check_if_either_is_set()
         Functions._validate_type(function_id, "function_id", str, True)
-        if self._if_deployment_exist_for_asset(function_id):
+
+        # For CP4D, check if either space or project ID is set
+        self._client._check_if_either_is_set()
+
+        if not force and self._if_deployment_exist_for_asset(function_id):
             raise WMLClientError(
                 "Cannot delete function that has existing deployments. Please delete all associated deployments and try again"
             )
 
-        function_endpoint = (
-            self._client.service_instance._href_definitions.get_function_href(
-                function_id
-            )
+        function_endpoint = self._client._href_definitions.get_function_href(
+            function_id
         )
+
         self._logger.debug(
-            "Deletion artifact function endpoint: {}".format(function_endpoint)
+            "Deletion artifact function endpoint: %s" % function_endpoint
         )
-        if not self._client.ICP_PLATFORM_SPACES:
-            response_delete = requests.delete(
-                function_endpoint,
-                params=self._client._params(),
-                headers=self._client._get_headers(),
-            )
-        else:
-            if Functions._if_deployment_exist_for_asset(self, function_id):
-                raise WMLClientError(
-                    "Cannot delete function that has existing deployments. Please delete all associated deployments and try again"
-                )
-            response_delete = requests.delete(
-                function_endpoint,
-                params=self._client._params(),
-                headers=self._client._get_headers(),
-            )
-        return self._handle_response(204, "function deletion", response_delete, False)
+        response = requests.delete(
+            function_endpoint,
+            params=self._client._params(),
+            headers=self._client._get_headers(),
+        )
+
+        return self._handle_response(204, "function deletion", response, False)
 
     def get_details(
         self,
@@ -507,7 +488,7 @@ class Functions(WMLResource):
         self._client._check_if_either_is_set()
         Functions._validate_type(function_id, "function_id", str, False)
         Functions._validate_type(limit, "limit", int, False)
-        url = self._client.service_instance._href_definitions.get_functions_href()
+        url = self._client._href_definitions.get_functions_href()
 
         if function_id is None:
             if spec_state:
@@ -654,29 +635,14 @@ class Functions(WMLResource):
             get_all=self._should_get_all_values(limit)
         )["resources"]
 
-        sw_spec_info = {
-            s["id"]: s
-            for s in self._client.software_specifications.get_details(state_info=True)[
-                "resources"
-            ]
-        }
-
-        def get_spec_info(spec_id: str | None, prop: str) -> str:
-            if spec_id and spec_id in sw_spec_info:
-                return sw_spec_info[spec_id].get(prop, "")
-            else:
-                return ""
-
         function_values = [
             (
                 m["metadata"]["id"],
                 m["metadata"]["name"],
                 m["metadata"]["created_at"],
                 m["entity"]["type"] if "type" in m["entity"] else None,
-                get_spec_info(m["entity"].get("software_spec", {}).get("id"), "state"),
-                get_spec_info(
-                    m["entity"].get("software_spec", {}).get("id"), "replacement"
-                ),
+                self._client.software_specifications._get_state(m),
+                self._client.software_specifications._get_replacement(m),
             )
             for m in function_resources
         ]
@@ -720,7 +686,7 @@ class Functions(WMLResource):
 
         Functions._validate_type(function_id, "function_id", str, False)
 
-        url = self._client.service_instance._href_definitions.get_functions_href()
+        url = self._client._href_definitions.get_functions_href()
         return self._create_revision_artifact(url, function_id, "functions")
 
     def get_revision_details(
@@ -755,9 +721,7 @@ class Functions(WMLResource):
         Functions._validate_type(function_id, "function_id", str, True)
         Functions._validate_type(rev_id, "rev_id", str, True)
 
-        url = self._client.service_instance._href_definitions.get_function_href(
-            function_id
-        )
+        url = self._client._href_definitions.get_function_href(function_id)
         return self._get_with_or_without_limit(
             url,
             limit=None,
@@ -794,9 +758,7 @@ class Functions(WMLResource):
 
         Functions._validate_type(function_id, "function_id", str, True)
 
-        url = self._client.service_instance._href_definitions.get_function_href(
-            function_id
-        )
+        url = self._client._href_definitions.get_function_href(function_id)
 
         # CP4D logic is wrong. By passing "revisions" in second param above for _get_artifact_details()
         # it won't even consider limit value and also GUID gives only rev number, not actual guid
