@@ -106,7 +106,7 @@ class BaseDataConnection(ABC):
 
         if self.type == DataConnectionTypes.FS:
             csv_url = (
-                self._api_client.service_instance._href_definitions.get_wsd_model_attachment_href()
+                self._api_client._href_definitions.get_wsd_model_attachment_href()
                 + f"/auto_ml/{path.split('/auto_ml/')[-1]}"
             )
 
@@ -127,6 +127,12 @@ class BaseDataConnection(ABC):
 
         elif self.type == DataConnectionTypes.CA or self.type == DataConnectionTypes.CN:
             if self._check_if_connection_asset_is_s3():
+                from .connections import _AmazonS3Connection
+
+                if isinstance(self.connection, _AmazonS3Connection):
+                    raise NotImplementedError(
+                        f"The operation is not supported for AmazonS3 connection. Try with Flight Service enabled"
+                    )
                 cos_client = self._init_cos_client()
 
                 try:
@@ -186,17 +192,17 @@ class BaseDataConnection(ABC):
         elif self.type == DataConnectionTypes.FS:
             if tuning_type == "prompt_tuning":
                 json_url = (
-                    self._api_client.service_instance._href_definitions.get_wsd_model_attachment_href()
+                    self._api_client._href_definitions.get_wsd_model_attachment_href()
                     + f"wx_prompt_tune/{path.split('/wx_prompt_tune/')[-1]}"
                 )
             elif tuning_type == "fine_tuning":
                 json_url = (
-                    self._api_client.service_instance._href_definitions.get_wsd_model_attachment_href()
-                    + path
+                    self._api_client._href_definitions.get_wsd_model_attachment_href()
+                    + f"wx_fine_tune/{path.split('/wx_fine_tune/')[-1]}"
                 )
             else:
                 json_url = (
-                    self._api_client.service_instance._href_definitions.get_wsd_model_attachment_href()
+                    self._api_client._href_definitions.get_wsd_model_attachment_href()
                     + f"/auto_ml/{path.split('/auto_ml/')[-1]}"
                 )
 
@@ -224,9 +230,18 @@ class BaseDataConnection(ABC):
         elif self.type == DataConnectionTypes.CA or self.type == DataConnectionTypes.CN:
             if self._check_if_connection_asset_is_s3():
                 cos_client = self._init_cos_client()
+                data_conn = self.to_dict()
+                bucket = data_conn.get("location", {}).get(
+                    "bucket", data_conn.get("connection", {}).get("bucket")
+                )  # AWS containers has bucket in connection
+                if bucket is None:
+
+                    raise ValueError(
+                        "Missing bucket in connection or location of the DataConnection object."
+                    )
 
                 try:
-                    file = cos_client.Object(self.location.bucket, path).get()
+                    file = cos_client.Object(bucket, path).get()
                     content = file["Body"].read()
                     try:
                         json_content = json.loads(content.decode("utf-8"))
@@ -254,11 +269,9 @@ class BaseDataConnection(ABC):
 
         return json_content
 
-    def _get_attachemnt_details(self, data_asset_id: str):
+    def _get_attachment_details(self, data_asset_id: str):
         response = requests.get(
-            self._api_client.service_instance._href_definitions.get_data_asset_href(
-                data_asset_id
-            ),
+            self._api_client._href_definitions.get_data_asset_href(data_asset_id),
             params=self._api_client._params(),
             headers=self._api_client._get_headers(),
         )
@@ -277,7 +290,7 @@ class BaseDataConnection(ABC):
             attachment_id = attachments_data_asset_details.get("id")
             if not self._api_client.ICP_PLATFORM_SPACES:
                 response = requests.get(
-                    self._api_client.service_instance._href_definitions.get_attachment_href(
+                    self._api_client._href_definitions.get_attachment_href(
                         data_asset_id, attachment_id
                     ),
                     params=self._api_client._params(),
@@ -285,7 +298,7 @@ class BaseDataConnection(ABC):
                 )
             else:
                 response = requests.get(
-                    self._api_client.service_instance._href_definitions.get_attachment_href(
+                    self._api_client._href_definitions.get_attachment_href(
                         data_asset_id, attachment_id
                     ),
                     params=self._api_client._params(),
@@ -299,92 +312,20 @@ class BaseDataConnection(ABC):
 
             return response.json()
 
-    def _prepare_connection_and_attachment_details(self) -> tuple[dict, dict]:
+    def _prepare_connection_details(self) -> tuple[dict, dict]:
         connection_details = {}
-        attachment_details = {}
 
-        if self.type == DataConnectionTypes.DS:
-            if self.connection is not None and hasattr(self.connection, "href"):
-                items = self.connection.href.split("/")
-
-            else:
-                items = self.location.href.split("/")
-            _id = items[-1].split("?")[0]
-
-            if self._api_client is not None:
-                attachment_details = self._get_attachemnt_details(_id)
-                if "connection_id" not in attachment_details:
-                    if (
-                        "handle" in attachment_details
-                        and "bucket" in attachment_details["handle"]
-                    ):
-                        connection_details = self._create_conn_details_for_container()
-                        attachment_details["connection_path"] = (
-                            "/"
-                            + attachment_details["handle"]["bucket"]
-                            + "/"
-                            + attachment_details["handle"]["key"]
-                        )
-                    elif (
-                        "url" in attachment_details
-                        and "name" in attachment_details
-                        and "bucket" in attachment_details
-                    ):
-                        connection_details = self._create_conn_details_for_container()
-                        attachment_details["connection_path"] = (
-                            "/"
-                            + attachment_details["bucket"]["bucket_name"]
-                            + "/"
-                            + attachment_details["name"]
-                        )
-                    else:
-                        raise NotS3Connection(_internal=True)
-
-                else:
-                    connection_details = self._api_client.connections.get_details(
-                        attachment_details["connection_id"]
-                    )
-
-            else:
-                try:
-                    from project_lib import Project
-
-                    project = Project.access()
-
-                    # note: Check if asset is located directly in the project files
-                    #       If yes it is not a connected data.
-                    #       [Prevents unnecessary logging].
-                    if any(file["asset_id"] == _id for file in project.get_files()):
-                        raise NotS3Connection
-                    # --- end note
-
-                    try:
-                        connection_details = project.get_connected_data(_id)
-
-                    except RuntimeError:  # when data asset is normal not s3 or database
-                        raise NotS3Connection
-
-                    attachment_details = {
-                        "connection_path": connection_details.get("datapath")
-                    }
-
-                except ModuleNotFoundError:
-                    raise NotImplementedError(
-                        f"This functionality can be run only on Watson Studio."
-                    )
-
-        elif self.type == DataConnectionTypes.CA:
+        if self.type == DataConnectionTypes.CA:
             if self._api_client is not None:
                 connection_details = self._api_client.connections.get_details(
                     self.connection.id
                 )
-
             else:
                 try:
-                    from project_lib import Project
+                    from ibm_watson_studio_lib import access_project_or_space
 
-                    project = Project.access()
-                    connection_details = project.get_connection(self.connection.id)
+                    wslib = access_project_or_space()
+                    connection_details = wslib.by_id.get_connection(self.connection.id)
 
                 except ModuleNotFoundError:
                     raise NotImplementedError(
@@ -394,7 +335,10 @@ class BaseDataConnection(ABC):
         elif self.type == DataConnectionTypes.CN:
             connection_details = self._create_conn_details_for_container()
 
-        return connection_details, attachment_details
+        else:
+            raise NotS3Connection(_internal=True)
+
+        return connection_details
 
     def _check_if_connection_asset_is_s3(self) -> bool:
         try:
@@ -411,17 +355,19 @@ class BaseDataConnection(ABC):
             ):
                 return True
 
-            if self.type in [DataConnectionTypes.S3, DataConnectionTypes.FS]:
+            if self.type in {
+                DataConnectionTypes.S3,
+                DataConnectionTypes.FS,
+                DataConnectionTypes.DS,
+            }:
                 return False
 
             try:
-                connection_details, attachment_details = (
-                    self._prepare_connection_and_attachment_details()
-                )
+                connection_details = self._prepare_connection_details()
             except NotS3Connection:
                 return False
 
-            # Note: Check with project libs if connection points to S3
+            # Note: Check with project libs if connection points to S3 (COS or AWS)
             if self._api_client is not None:
                 datasource_type = connection_details["entity"]["datasource_type"]
                 self._datasource_type = datasource_type
@@ -436,12 +382,13 @@ class BaseDataConnection(ABC):
                     )
                 )
 
-                if datasource_type in [
+                if self._datasource_type in {
                     datasource_type_id_ibm_cos,
                     datasource_type_id_aws_cos,
                     "bluemixcloudobjectstorage",
                     "cloudobjectstorage",
-                ]:
+                    "amazons3",
+                }:
                     is_s3 = True
 
                 else:
@@ -459,7 +406,10 @@ class BaseDataConnection(ABC):
             # --- end note
 
             if is_s3:
-                self._init_s3_connection(connection_details, attachment_details)
+                if self._datasource_type == "amazons3":
+                    self._init_s3_aws_connection(connection_details)
+                else:
+                    self._init_s3_cos_connection(connection_details)
 
             return is_s3
 
@@ -473,9 +423,11 @@ class BaseDataConnection(ABC):
             else:
                 return False  # if we are in WS, ignore this check even if there was some error
 
-    def _init_s3_connection(
-        self, connection_details: dict, attachment_details: dict
-    ) -> None:
+    def _init_s3_cos_connection(self, connection_details: dict) -> None:
+        """
+        Helper function that initializes internal `S3Connection` object based on connection_details retrieved
+         from connection asset or container (IBM Cloud).
+        """
         connection_props = connection_details["entity"]["properties"]
 
         connection_values = {
@@ -517,56 +469,66 @@ class BaseDataConnection(ABC):
         if self.type == DataConnectionTypes.CN:
             self.location.bucket = connection_props["bucket_name"]
 
-        if self.type == DataConnectionTypes.DS:
-            connection_path = attachment_details["connection_path"]
-            try:
-                # Workaround for connected data assets promoted from catalog
-                if not connection_path.startswith("/"):
-                    connection_path = "/" + connection_path
-                self.location.bucket = connection_path.split("/")[1]
-
-                # Remove excel sheet name from connection_path if data asset is excel file:
-                if ".xlsx/" in connection_path or ".xls/" in connection_path:
-                    excel_sheet = connection_path.split("/")[-1]
-                    # note: set recognized excel sheet in params
-                    if not self.auto_pipeline_params.get("excel_sheet"):
-                        self.auto_pipeline_params["excel_sheet"] = excel_sheet
-                    # end note
-                    self.location.path = "/".join(connection_path.split("/")[2:-1])
-                else:
-                    self.location.path = "/".join(connection_path.split("/")[2:])
-
-            except IndexError:
-                self.location.bucket = connection_props["bucket"]
-                self.location.path = connection_path
-
         self.connection.is_s3 = True
 
+    def _init_s3_aws_connection(self, connection_details: dict) -> None:
+        """
+        Helper function that initializes internal `AmazonS3Connection` object based on connection_details retrieved
+         from container (IBM Cloud on AWS).
+        """
+        from .connections import _AmazonS3Connection
+
+        connection_props = connection_details["entity"].get("properties", {})
+
+        self.connection = _AmazonS3Connection(
+            access_key=connection_props["credentials"]["access_key_id"],
+            bucket=connection_props["bucket_name"],
+            region=connection_props["bucket_region"],
+            secret_key=connection_props["credentials"]["secret_access_key"],
+            session_token=connection_props["credentials"]["session_token"],
+            shared_credentials=connection_props.get("shared", True),
+        )
+
+        if (
+            self.type == DataConnectionTypes.CN
+            and self.connection._shared_credentials
+            and self._api_client is not None
+        ):
+            container_path_prefix = (
+                self._api_client.default_project_id or self._api_client.default_space_id
+            )
+            if container_path_prefix and not self.location.path.startswith(
+                container_path_prefix
+            ):
+                self.location.path = f"{container_path_prefix}/{self.location.path}"
+
     def _is_data_asset_normal(self) -> bool:
         """Returns `True` if data asset is normal data asset - not connected data asset."""
         try:
             if self.type == "data_asset":
-                if self.connection is not None and hasattr(self.connection, "href"):
-                    items = self.connection.href.split("/")
-                else:
-                    items = self.location.href.split("/")
-                _id = items[-1].split("?")[0]
+                items = (
+                    self.connection.href.split("/")
+                    if self.connection is not None and hasattr(self.connection, "href")
+                    else self.location.href.split("/")
+                )
+
+                data_asset_id = items[-1].split("?")[0]
 
                 if self._api_client is not None:
-                    attachment_details = self._get_attachemnt_details(_id)
+                    attachment_details = self._get_attachment_details(data_asset_id)
                     return bool("connection_id" not in attachment_details)
-
                 else:
                     try:
-                        from project_lib import Project
+                        from ibm_watson_studio_lib import access_project_or_space
 
-                        project = Project.access()
+                        wslib = access_project_or_space()
 
                         # note: Check if asset is located directly in the project files
                         #       If yes it is not a connected data.
                         #       [Prevents unnecessary logging].
                         return any(
-                            file["asset_id"] == _id for file in project.get_files()
+                            file["asset_id"] == data_asset_id
+                            for file in wslib.list_stored_data()
                         )
                         # --- end note
 
@@ -586,14 +548,16 @@ class BaseDataConnection(ABC):
 
     def _is_data_asset_nfs(self):
         if self.type == "data_asset":
-            if self.connection is not None and hasattr(self.connection, "href"):
-                items = self.connection.href.split("/")
-            else:
-                items = self.location.href.split("/")
-            _id = items[-1].split("?")[0]
+            items = (
+                self.connection.href.split("/")
+                if self.connection is not None and hasattr(self.connection, "href")
+                else self.location.href.split("/")
+            )
+
+            data_asset_id = items[-1].split("?")[0]
 
             if self._api_client is not None:
-                attachment_details = self._get_attachemnt_details(_id)
+                attachment_details = self._get_attachment_details(data_asset_id)
 
                 return "connection_id" in attachment_details and attachment_details.get(
                     "datasource_type"
@@ -639,7 +603,10 @@ class BaseDataConnection(ABC):
         return data
 
     def _download_training_data_from_data_asset_storage(
-        self, binary: bool = False, is_flight_fallback: bool = False
+        self,
+        binary: bool = False,
+        is_flight_fallback: bool = False,
+        read_to_file: str | None = None,
     ) -> DataFrame | bytes:
         """Download training data for this connection. Data Storage."""
 
@@ -650,9 +617,7 @@ class BaseDataConnection(ABC):
 
             # note: download data asset details
             asset_response = requests.get(
-                self._api_client.service_instance._href_definitions.get_data_asset_href(
-                    asset_id
-                ),
+                self._api_client._href_definitions.get_data_asset_href(asset_id),
                 params=self._api_client._params(),
                 headers=self._api_client._get_headers(),
             )
@@ -663,7 +628,7 @@ class BaseDataConnection(ABC):
 
             attachment_id = asset_details["attachments"][0]["id"]
             response = requests.get(
-                self._api_client.service_instance._href_definitions.get_attachment_href(
+                self._api_client._href_definitions.get_attachment_href(
                     asset_id, attachment_id
                 ),
                 params=self._api_client.data_assets._client._params(),
@@ -699,6 +664,11 @@ class BaseDataConnection(ABC):
                                 self.auto_pipeline_params.get("encoding", "utf-8"),
                             )
 
+                        if read_to_file:
+                            with open(read_to_file, "wb") as file:
+                                buffer.seek(0)
+                                file.write(buffer.getvalue())
+
                         if binary:
                             buffer.seek(0)
                             return buffer.read()
@@ -723,7 +693,7 @@ class BaseDataConnection(ABC):
 
                 # note: make the whole url pointing out the csv
                 artifact_content_url = (
-                    f"{self._api_client.service_instance._href_definitions.get_wsd_model_attachment_href()}"
+                    f"{self._api_client._href_definitions.get_wsd_model_attachment_href()}"
                     f"{attachment_url}"
                 )
 
@@ -776,17 +746,16 @@ class BaseDataConnection(ABC):
 
         else:
             try:
-                from project_lib import Project
-
-                project = Project.access()
+                from ibm_watson_studio_lib import access_project_or_space
 
+                wslib = access_project_or_space()
             except ModuleNotFoundError:
                 raise NotImplementedError(
                     f"This functionality can be run only on Watson Studio."
                 )
 
             asset_id = self.location.href.split("?")[0].split("/")[-1]
-            assets_list = project.get_assets()
+            assets_list = wslib.assets.list_assets("asset")
 
             data_asset_name = None
             for asset in assets_list:
@@ -796,7 +765,7 @@ class BaseDataConnection(ABC):
             if data_asset_name is None:
                 raise FileNotFoundError(f"Cannot find data asset with id: {asset_id}")
 
-            buffer = project.get_file(data_asset_name)
+            buffer = wslib.load_data(data_asset_name)
 
             if is_flight_fallback:
                 _error_on_duplicate_columns_csv(
@@ -825,8 +794,8 @@ class BaseDataConnection(ABC):
 
         try:
             url = (
-                self._api_client.service_instance._href_definitions.get_wsd_model_attachment_href()
-                + f"/{self.location.path.split('/assets/')[-1]}"
+                self._api_client._href_definitions.get_wsd_model_attachment_href()
+                + f"/{self.location.path.split('/assets/', maxsplit=1)[-1]}"
             )
             # note: stream the whole CSV file
             csv_response = requests.get(
@@ -873,8 +842,8 @@ class BaseDataConnection(ABC):
 
         try:
             url = (
-                self._api_client.service_instance._href_definitions.get_wsd_model_attachment_href()
-                + f"/{location_path.split('/assets/')[-1]}"
+                self._api_client._href_definitions.get_wsd_model_attachment_href()
+                + f"/{location_path.split('/assets/', maxsplit=1)[-1]}"
             )
             # note: stream the whole CSV file
             csv_response = requests.get(
@@ -953,8 +922,10 @@ class BaseDataConnection(ABC):
                 raise conn_error
 
         if buffer is None:
-            href = self._api_client.volumes._client.service_instance._href_definitions.volume_upload_href(
-                connection_details["entity"]["properties"]["volume"]
+            href = (
+                self._api_client.volumes._client._href_definitions.volume_upload_href(
+                    connection_details["entity"]["properties"]["volume"]
+                )
             )
             full_href = f"{href[:-1]}{connection_path}"
 
@@ -1044,54 +1015,66 @@ class BaseDataConnection(ABC):
 
     def _create_conn_details_for_container(self) -> dict:
         if self._api_client is not None:
+            self._api_client._check_if_either_is_set()  # Space or project is required to use Container.
             if self._api_client.default_space_id is not None:
-                details = self._api_client.spaces.get_details(
-                    self._api_client.default_space_id
+                details = (
+                    self._api_client.spaces._get_details(  # get details with TTL cache
+                        self._api_client.default_space_id,
+                        include="everything,credentials",
+                    )
                 )
-
             else:
-                project_url = self._api_client.service_instance._href_definitions.get_project_href(
-                    self._api_client.default_project_id
+                details = self._api_client.projects._get_details(  # get details with TTL cache
+                    self._api_client.default_project_id,
+                    include="everything,credentials",
                 )
 
-                response = requests.get(
-                    project_url,
-                    headers=self._api_client._get_headers(projects_token=True),
-                )
+            match details["entity"]["storage"]["type"]:
+                case "bmcos_object_storage":
+                    properties = details["entity"]["storage"]["properties"]
+                    creds = details["entity"]["storage"]["properties"][
+                        "credentials"
+                    ].get("admin")
 
-                details = response.json()
+                    if (
+                        creds
+                        and creds.get("access_key_id", False)
+                        and creds.get("secret_access_key", False)
+                    ):
+                        pass
+                    else:  # missing admin credentials
+                        creds = details["entity"]["storage"]["properties"][
+                            "credentials"
+                        ].get("editor")
 
-            properties = details["entity"]["storage"]["properties"]
-            creds = details["entity"]["storage"]["properties"]["credentials"].get(
-                "admin"
-            )
+                    properties.update(creds)
+                    properties["url"] = properties["endpoint_url"]
+                    properties["access_key"] = properties.get("access_key_id")
+                    properties["secret_key"] = properties.get("secret_access_key")
 
-            if (
-                creds
-                and creds.get("access_key_id", False)
-                and creds.get("secret_access_key", False)
-            ):
-                pass
-            else:  # missing admin credentials
-                creds = details["entity"]["storage"]["properties"]["credentials"].get(
-                    "editor"
-                )
+                    connection_details = {
+                        "entity": {
+                            "datasource_type": "bluemixcloudobjectstorage",
+                            "properties": properties,
+                        }
+                    }
+                case "amazon_s3":
+                    properties = details["entity"]["storage"]["properties"]
+                    connection_details = {
+                        "entity": {
+                            "datasource_type": "amazons3",
+                            "properties": properties,
+                        }
+                    }
 
-            properties.update(creds)
-            properties["url"] = properties["endpoint_url"]
-            properties["access_key"] = properties.get("access_key_id")
-            properties["secret_key"] = properties.get("secret_access_key")
-
-            connection_details = {
-                "entity": {
-                    "datasource_type": "bluemixcloudobjectstorage",
-                    "properties": properties,
-                }
-            }
+                case _:
+                    raise ValueError(
+                        f'Container type not supported in the project with storage {details["entity"]["storage"]["type"]}.'
+                    )
 
         else:
             try:
-                from project_lib import Project
+                from ibm_watson_studio_lib import access_project_or_space
 
                 token = self._get_token_from_environment()
 
@@ -1112,11 +1095,8 @@ class BaseDataConnection(ABC):
 
                 token = token.split("Bearer ")[-1]
 
-                _project = Project(
-                    project_id=os.environ.get("PROJECT_ID"), project_access_token=token
-                )
-
-                details = _project.get_storage_metadata()
+                wslib = access_project_or_space({"token": token})
+                details = wslib.here.get_storage()
 
                 properties = details["properties"]
                 properties.update(properties["credentials"]["editor"])
@@ -1219,10 +1199,10 @@ class BaseDataConnection(ABC):
             # because data limit is calculated on data downloaded as varchar, before the conversion to more optimal type.
             try:
                 iterable_dataset = TabularIterableDataset(
-                    **experiment_iterable_dataset_setup_parameters
+                    **experiment_iterable_dataset_setup_parameters,
+                    infer_as_varchar="true",
                 )
 
-                iterable_dataset.connection.infer_as_varchar = "true"
                 data_loader = ExperimentDataLoader(dataset=iterable_dataset)
 
                 if not return_data_as_iterator:
@@ -1337,13 +1317,15 @@ class BaseDataConnection(ABC):
                     folder_name = os.path.basename(os.path.dirname(location))
                     asset_path = folder_name + "/" + remote_name
                 else:
-                    asset_path = location.split("/assets/")[-1] + "/" + remote_name
+                    asset_path = (
+                        location.split("/assets/", maxsplit=1)[-1] + "/" + remote_name
+                    )
             else:
                 asset_path = "/" + remote_name
-        elif data.raw.name:
-            asset_path = "/" + data.raw.name
         elif is_filename_location:
             asset_path = "/" + location
+        elif data.raw.name:
+            asset_path = "/" + data.raw.name
         else:
             raise ValueError(
                 "The `remote_name` and `location` fields provided are invalid."
@@ -1353,7 +1335,7 @@ class BaseDataConnection(ABC):
             asset_path = asset_path.replace("//", "/")
 
         content_upload_url = (
-            self._api_client.service_instance._href_definitions.get_wsd_model_attachment_href()
+            self._api_client._href_definitions.get_wsd_model_attachment_href()
             + asset_path
         )
 
@@ -1431,10 +1413,11 @@ class BaseDataConnection(ABC):
         from ibm_watsonx_ai.utils.autoai.connection import get_max_sample_size_limit
 
         if self._api_client and self.type == "data_asset":
-            if self.connection is not None and hasattr(self.connection, "href"):
-                items = self.connection.href.split("/")
-            else:
-                items = self.location.href.split("/")
+            items = (
+                self.connection.href.split("/")
+                if self.connection is not None and hasattr(self.connection, "href")
+                else self.location.href.split("/")
+            )
             data_asset_id = items[-1].split("?")[0]
             asset_size = (
                 self._api_client.data_assets.get_details(data_asset_id)
