@@ -5,6 +5,8 @@
 
 
 import sys
+from typing import Iterator
+
 import pandas as pd
 
 
@@ -19,12 +21,19 @@ class LocalBatchReader:
         self._determine_row_size()
 
     def _determine_row_size(self) -> None:
-        data_row = pd.read_csv(self.file_path, chunksize=1)
+        data_row = (
+            pd.read_excel(self.file_path, nrows=1)
+            if self.file_path.endswith(".xlsx") or self.file_path.endswith(".xls")
+            else pd.read_csv(self.file_path, chunksize=1)
+        )
         self.row_size = sys.getsizeof(data_row)
 
     def _calculate_chunk_size(self) -> int:
         return self.batch_size // self.row_size
 
-    def __iter__(self) -> pd.DataFrame:
+    def __iter__(self) -> Iterator[pd.DataFrame]:
         x = self._calculate_chunk_size()
-        return pd.read_csv(self.file_path, chunksize=self._calculate_chunk_size())
+        if self.file_path.endswith(".xlsx") or self.file_path.endswith(".xls"):
+            yield pd.read_excel(self.file_path, nrows=x)
+        else:
+            yield from pd.read_csv(self.file_path, chunksize=x)
