@@ -14,10 +14,11 @@ import platform
 import sys
 from pathlib import Path
 from typing import TYPE_CHECKING, Any
+from warnings import warn
 
 import ibm_watsonx_ai._wrappers.requests as requests
 import requests as req
-from ibm_watsonx_ai.utils.utils import get_module_version
+from ibm_watsonx_ai.utils.utils import get_module_version, _handle_fl_removal
 from ibm_watsonx_ai.wml_client_error import ApiRequestFailure
 from ibm_watsonx_ai.wml_resource import WMLResource
 
@@ -259,6 +260,8 @@ class Party(WMLResource):
         self.log_level = kwargs.get("log_level", "ERROR")
 
     def start(self) -> None:
+        _handle_fl_removal(self._client)
+
         self.Party.start()  # type: ignore
 
     def run(
@@ -303,6 +306,8 @@ class Party(WMLResource):
             party.run( experiment_id = "2466fa06-1110-4169-a166-01959adec995", asynchronous = False )
 
         """
+        _handle_fl_removal(self._client)
+
         import time
         from datetime import datetime
 
@@ -525,6 +530,8 @@ class Party(WMLResource):
             party.monitor_logs()
 
         """
+        _handle_fl_removal(self._client)
+
         self.log_level = log_level
 
         # Configure logging locally as well
@@ -549,6 +556,8 @@ class Party(WMLResource):
             party.monitor_metrics()
 
         """
+        _handle_fl_removal(self._client)
+
         self.metrics_output = metrics_file
 
     def is_running(self) -> bool:
@@ -564,6 +573,8 @@ class Party(WMLResource):
             party.is_running()
 
         """
+        _handle_fl_removal(self._client)
+
         return not self.connection.stopped  # type: ignore[attr-defined]
 
     def get_round(self) -> int:
@@ -579,6 +590,8 @@ class Party(WMLResource):
             party.get_round()
 
         """
+        _handle_fl_removal(self._client)
+
         return self.Party.proto_handler.metrics_recorder.get_round_no()  # type: ignore
 
     def cancel(self) -> None:
@@ -591,4 +604,6 @@ class Party(WMLResource):
             party.cancel()
 
         """
+        _handle_fl_removal(self._client)
+
         self.Party.stop_connection()  # type: ignore
