@@ -14,7 +14,7 @@ from requests import Response
 from ibm_watsonx_ai._wrappers import requests
 
 from ibm_watsonx_ai.metanames import PkgExtnMetaNames
-from ibm_watsonx_ai.utils import PKG_EXTN_DETAILS_TYPE
+from ibm_watsonx_ai.utils import PKG_EXTN_DETAILS_TYPE, content_type_for
 from ibm_watsonx_ai.wml_client_error import (
     WMLClientError,
     ApiRequestFailure,
@@ -55,9 +55,7 @@ class PkgExtn(WMLResource):
         PkgExtn._validate_type(pkg_extn_id, "pkg_extn_id", str, True)
 
         response = requests.get(
-            self._client.service_instance._href_definitions.get_pkg_extn_href(
-                pkg_extn_id
-            ),
+            self._client._href_definitions.get_pkg_extn_href(pkg_extn_id),
             params=self._client._params(),
             headers=self._client._get_headers(),
         )
@@ -111,7 +109,7 @@ class PkgExtn(WMLResource):
         PkgExtn._validate_type(file_path, "file_path", str, True)
         # Step1  : Create an asset
         print("Creating package extensions")
-        href = self._client.service_instance._href_definitions.get_pkg_extns_href()
+        href = self._client._href_definitions.get_pkg_extns_href()
 
         creation_response = requests.post(
             href,
@@ -166,7 +164,7 @@ class PkgExtn(WMLResource):
                         )
                 except Exception as e:
                     deletion_response = requests.delete(
-                        self._client.service_instance._href_definitions.get_pkg_extn_href(
+                        self._client._href_definitions.get_pkg_extn_href(
                             pkg_extn_asset_id
                         ),
                         params=self._client._params(),
@@ -183,9 +181,7 @@ class PkgExtn(WMLResource):
             #   (self._client.service_instance._wml_credentials.url if self._client.ICP_PLATFORM_SPACES else "") + pkg_extn_presigned_url)
 
             href = (
-                self._client.service_instance._credentials.url
-                if self._client.ICP_PLATFORM_SPACES
-                else ""
+                self._client.credentials.url if self._client.ICP_PLATFORM_SPACES else ""
             ) + pkg_extn_presigned_url
 
             try:
@@ -194,7 +190,12 @@ class PkgExtn(WMLResource):
 
                 with open(file_path, "rb") as file_object:
                     if not self._client.ICP_PLATFORM_SPACES:
-                        put_response = requests.put(href, data=file_object.read())
+                        content_type = content_type_for(filepath=file_path)
+                        put_response = requests.put(
+                            href,
+                            data=file_object.read(),
+                            headers={"Content-Type": content_type},
+                        )
                     else:
                         put_response = requests.put(
                             href,
@@ -208,9 +209,7 @@ class PkgExtn(WMLResource):
                         )
             except Exception as e:
                 deletion_response = requests.delete(
-                    self._client.service_instance._href_definitions.get_pkg_extn_href(
-                        pkg_extn_asset_id
-                    ),
+                    self._client._href_definitions.get_pkg_extn_href(pkg_extn_asset_id),
                     params=self._client._params(),
                     headers=self._client._get_headers(),
                 )
@@ -220,9 +219,7 @@ class PkgExtn(WMLResource):
             if put_response.status_code == 201 or put_response.status_code == 200:
                 # Step3: Mark the upload complete
                 complete_response = requests.post(
-                    self._client.service_instance._href_definitions.get_pkg_extn_href(
-                        pkg_extn_asset_id
-                    )
+                    self._client._href_definitions.get_pkg_extn_href(pkg_extn_asset_id)
                     + "/upload_complete",
                     headers=self._client._get_headers(),
                     params=self._client._params(),
@@ -264,7 +261,7 @@ class PkgExtn(WMLResource):
 
         """
 
-        href = self._client.service_instance._href_definitions.get_pkg_extns_href()
+        href = self._client._href_definitions.get_pkg_extns_href()
 
         response = requests.get(
             href, params=self._client._params(), headers=self._client._get_headers()
@@ -335,7 +332,7 @@ class PkgExtn(WMLResource):
         parameters.update(name=pkg_extn_name)
 
         response = requests.get(
-            self._client.service_instance._href_definitions.get_pkg_extns_href(),
+            self._client._href_definitions.get_pkg_extns_href(),
             params=parameters,
             headers=self._client._get_headers(),
         )
@@ -397,9 +394,7 @@ class PkgExtn(WMLResource):
         PkgExtn._validate_type(pkg_extn_id, "pkg_extn_id", str, True)
 
         response = requests.delete(
-            self._client.service_instance._href_definitions.get_pkg_extn_href(
-                pkg_extn_id
-            ),
+            self._client._href_definitions.get_pkg_extn_href(pkg_extn_id),
             params=self._client._params(),
             headers=self._client._get_headers(),
         )
@@ -466,9 +461,7 @@ class PkgExtn(WMLResource):
         PkgExtn._validate_type(pkg_extn_id, "pkg_extn_id", str, True)
 
         pkg_extn_response = requests.get(
-            self._client.service_instance._href_definitions.get_pkg_extn_href(
-                pkg_extn_id
-            ),
+            self._client._href_definitions.get_pkg_extn_href(pkg_extn_id),
             params=self._client._params(),
             headers=self._client._get_headers(),
         )
