@@ -6,11 +6,13 @@
 from __future__ import annotations
 
 from typing import TYPE_CHECKING
+from warnings import warn
 
 from ibm_watsonx_ai._wrappers import requests
 
 from ibm_watsonx_ai.metanames import RemoteTrainingSystemMetaNames
 from ibm_watsonx_ai.party_wrapper import Party
+from ibm_watsonx_ai.utils.utils import _handle_fl_removal
 from ibm_watsonx_ai.wml_client_error import WMLClientError
 from ibm_watsonx_ai.wml_resource import WMLResource
 
@@ -54,6 +56,8 @@ class RemoteTrainingSystem(WMLResource):
             client.set.default_space('3fc54cf1-252f-424b-b52d-5cdd9814987f')
             details = client.remote_training_systems.store(meta_props=metadata)
         """
+        _handle_fl_removal(self._client)
+
         self._client._check_if_either_is_set()
 
         RemoteTrainingSystem._validate_type(meta_props, "meta_props", dict, True)
@@ -68,9 +72,7 @@ class RemoteTrainingSystem(WMLResource):
         elif self._client.default_project_id is not None:
             meta["project_id"] = self._client.default_project_id
 
-        href = (
-            self._client.service_instance._href_definitions.remote_training_systems_href()
-        )
+        href = self._client._href_definitions.remote_training_systems_href()
 
         creation_response = requests.post(
             href,
@@ -122,15 +124,15 @@ class RemoteTrainingSystem(WMLResource):
 
             client.remote_training_systems.delete(remote_training_systems_id='6213cf1-252f-424b-b52d-5cdd9814956c')
         """
+        _handle_fl_removal(self._client)
+
         self._client._check_if_either_is_set()
         RemoteTrainingSystem._validate_type(
             remote_training_systems_id, "remote_training_systems_id", str, True
         )
 
-        href = (
-            self._client.service_instance._href_definitions.remote_training_system_href(
-                remote_training_systems_id
-            )
+        href = self._client._href_definitions.remote_training_system_href(
+            remote_training_systems_id
         )
 
         delete_response = requests.delete(
@@ -185,6 +187,8 @@ class RemoteTrainingSystem(WMLResource):
                 details.extend(entry)
 
         """
+        _handle_fl_removal(self._client)
+
         self._client._check_if_either_is_set()
 
         RemoteTrainingSystem._validate_type(
@@ -192,9 +196,7 @@ class RemoteTrainingSystem(WMLResource):
         )
         RemoteTrainingSystem._validate_type(limit, "limit", int, False)
 
-        href = (
-            self._client.service_instance._href_definitions.remote_training_systems_href()
-        )
+        href = self._client._href_definitions.remote_training_systems_href()
 
         if remote_training_system_id is None:
             return self._get_artifact_details(
@@ -225,6 +227,8 @@ class RemoteTrainingSystem(WMLResource):
 
             client.remote_training_systems.list()
         """
+        _handle_fl_removal(self._client)
+
         self._client._check_if_either_is_set()
 
         resources = self.get_details(get_all=self._should_get_all_values(limit))[
@@ -255,6 +259,7 @@ class RemoteTrainingSystem(WMLResource):
             details = client.remote_training_systems.get_details(remote_training_system_id)
             id = client.remote_training_systems.get_id(details)
         """
+
         RemoteTrainingSystem._validate_type(
             remote_training_system_details,
             "remote_training_system_details",
@@ -289,6 +294,8 @@ class RemoteTrainingSystem(WMLResource):
             details = client.remote_training_systems.update(remote_training_system_id, changes=metadata)
 
         """
+        _handle_fl_removal(self._client)
+
         self._client._check_if_either_is_set()
 
         self._validate_type(
@@ -302,10 +309,8 @@ class RemoteTrainingSystem(WMLResource):
             details["entity"], changes, with_validation=True
         )
 
-        href = (
-            self._client.service_instance._href_definitions.remote_training_system_href(
-                remote_training_system_id
-            )
+        href = self._client._href_definitions.remote_training_system_href(
+            remote_training_system_id
         )
 
         response = requests.patch(
@@ -336,13 +341,13 @@ class RemoteTrainingSystem(WMLResource):
 
             client.remote_training_systems.create_revision(remote_training_system_id)
         """
+        _handle_fl_removal(self._client)
+
         RemoteTrainingSystem._validate_type(
             remote_training_system_id, "remote_training_system_id", str, False
         )
 
-        href = (
-            self._client.service_instance._href_definitions.remote_training_systems_href()
-        )
+        href = self._client._href_definitions.remote_training_systems_href()
         return self._create_revision_artifact(
             href, remote_training_system_id, "remote training system"
         )
@@ -365,6 +370,8 @@ class RemoteTrainingSystem(WMLResource):
 
             details = client.remote_training_systems.get_details(remote_training_system_id, rev_id)
         """
+        _handle_fl_removal(self._client)
+
         self._client._check_if_either_is_set()
 
         RemoteTrainingSystem._validate_type(
@@ -372,10 +379,8 @@ class RemoteTrainingSystem(WMLResource):
         )
         RemoteTrainingSystem._validate_type(rev_id, "rev_id", str, True)
 
-        href = (
-            self._client.service_instance._href_definitions.remote_training_system_href(
-                remote_training_system_id
-            )
+        href = self._client._href_definitions.remote_training_system_href(
+            remote_training_system_id
         )
         return self._get_with_or_without_limit(
             href,
@@ -406,13 +411,15 @@ class RemoteTrainingSystem(WMLResource):
 
             client.remote_training_systems.list_revisions(remote_training_system_id)
         """
+        _handle_fl_removal(self._client)
+
         self._client._check_if_either_is_set()
 
         RemoteTrainingSystem._validate_type(
             remote_training_system_id, "remote_training_system_id", str, True
         )
 
-        href = self._client.service_instance._href_definitions.get_function_href(
+        href = self._client._href_definitions.get_function_href(
             remote_training_system_id
         )
 
@@ -492,6 +499,7 @@ class RemoteTrainingSystem(WMLResource):
             party = client.remote_training_systems.create_party(remote_training_system_id, party_metadata)
 
         """
+        _handle_fl_removal(self._client)
 
         RemoteTrainingSystem._validate_type(
             remote_training_system_id, "remote_training_system_id", str, True
