@@ -7,6 +7,7 @@ from __future__ import annotations
 
 import time
 from typing import TYPE_CHECKING, Any, Literal
+from cachetools import cached, TTLCache
 
 from ibm_watsonx_ai._wrappers import requests
 from ibm_watsonx_ai.service_instance import ServiceInstance
@@ -88,9 +89,7 @@ class Spaces(WMLResource):
 
         :raises WMLClientError: raised when connection is unauthorized
         """
-        href = (
-            self._client.service_instance._href_definitions.get_platform_spaces_href()
-        )
+        href = self._client._href_definitions.get_platform_spaces_href()
         response_get = self._client._session.get(
             href, headers=self._client._get_headers(), params={"limit": 1}
         )
@@ -176,13 +175,13 @@ class Spaces(WMLResource):
             space_meta["compute"] = payload_compute
 
         creation_response = requests.post(
-            self._client.service_instance._href_definitions.get_platform_spaces_href(),
+            self._client._href_definitions.get_platform_spaces_href(),
             headers=self._client._get_headers(),
             json=space_meta,
         )
 
         spaces_details = self._handle_response(
-            202, "creating new spaces", creation_response
+            202, "creating new spaces", creation_response, _silent_response_logging=True
         )
 
         # Cloud Convergence: Set self._client.credentials.instance_id to instance_id
@@ -331,10 +330,8 @@ class Spaces(WMLResource):
         """
         Spaces._validate_type(space_id, "space_id", str, True)
 
-        space_endpoint = (
-            self._client.service_instance._href_definitions.get_platform_space_href(
-                space_id
-            )
+        space_endpoint = self._client._href_definitions.get_platform_space_href(
+            space_id
         )
 
         response_delete = requests.delete(
@@ -354,8 +351,9 @@ class Spaces(WMLResource):
         asynchronous: bool | None = False,
         get_all: bool | None = False,
         space_name: str | None = None,
+        **kwargs: Any,
     ) -> dict:
-        """Get metadata of stored space(s).
+        """Get metadata of stored space(s). The method uses TTL cache.
 
         :param space_id: ID of the space
         :type space_id: str, optional
@@ -386,32 +384,59 @@ class Spaces(WMLResource):
                 space_details.extend(entry)
 
         """
+
         Spaces._validate_type(space_id, "space_id", str, False)
 
-        href = self._client.service_instance._href_definitions.get_platform_space_href(
-            space_id
-        )
+        href = self._client._href_definitions.get_platform_space_href(space_id)
+        query_params = {}
+        if include := kwargs.get("include"):
+            query_params["include"] = include
 
         if space_id is not None:
-            response_get = requests.get(href, headers=self._client._get_headers())
-
-            return self._handle_response(200, "Get space", response_get)
-
-        else:
-            query_params = {"name": space_name} if space_name else None
-
-            return self._get_with_or_without_limit(
-                self._client.service_instance._href_definitions.get_platform_spaces_href(),
-                limit,
-                "spaces",
-                summary=False,
-                pre_defined=False,
-                skip_space_project_chk=True,
-                query_params=query_params,
-                _async=asynchronous,
-                _all=get_all,
+            response_get = requests.get(
+                href, headers=self._client._get_headers(), params=query_params
             )
 
+            return self._handle_response(
+                200, "Get space", response_get, _silent_response_logging=True
+            )
+
+        if space_name:
+            query_params.update({"name": space_name})
+
+        return self._get_with_or_without_limit(
+            self._client._href_definitions.get_platform_spaces_href(),
+            limit,
+            "spaces",
+            summary=False,
+            pre_defined=False,
+            skip_space_project_chk=True,
+            query_params=query_params,
+            _async=asynchronous,
+            _all=get_all,
+            _silent_response_logging=True,
+        )
+
+    @cached(cache=TTLCache(maxsize=32, ttl=4.5 * 60))
+    def _get_details(
+        self,
+        space_id: str | None = None,
+        limit: int | None = None,
+        asynchronous: bool | None = False,
+        get_all: bool | None = False,
+        space_name: str | None = None,
+        **kwargs: Any,
+    ) -> dict:
+        """Get metadata of stored space(s) with caching. It's dedicated for internal usage."""
+        return self.get_details(
+            space_id=space_id,
+            limit=limit,
+            asynchronous=asynchronous,
+            get_all=get_all,
+            space_name=space_name,
+            **kwargs,
+        )
+
     def list(
         self,
         limit: int | None = None,
@@ -444,9 +469,7 @@ class Spaces(WMLResource):
         """
 
         Spaces._validate_type(limit, "limit", int, False)
-        href = (
-            self._client.service_instance._href_definitions.get_platform_spaces_href()
-        )
+        href = self._client._href_definitions.get_platform_spaces_href()
 
         params: dict[str, Any] = {}
 
@@ -474,6 +497,7 @@ class Spaces(WMLResource):
                 query_params=params,
                 _async=True,
                 _all=True,
+                _silent_response_logging=True,
             )
             for m in r["resources"]
         ]
@@ -540,15 +564,15 @@ class Spaces(WMLResource):
             details["entity"], changes
         )
 
-        href = self._client.service_instance._href_definitions.get_platform_space_href(
-            space_id
-        )
+        href = self._client._href_definitions.get_platform_space_href(space_id)
 
         response = requests.patch(
             href, json=patch_payload, headers=self._client._get_headers()
         )
 
-        updated_details = self._handle_response(200, "spaces patch", response)
+        updated_details = self._handle_response(
+            200, "spaces patch", response, _silent_response_logging=True
+        )
 
         # Cloud Convergence
         if (
@@ -623,14 +647,11 @@ class Spaces(WMLResource):
         )
 
         creation_response = requests.post(
-            self._client.service_instance._href_definitions.get_platform_spaces_members_href(
-                space_id
-            ),
+            self._client._href_definitions.get_platform_spaces_members_href(space_id),
             headers=self._client._get_headers(),
             json=space_meta,
         )
 
-        # TODO: Change response code one they change it to 201
         members_details = self._handle_response(
             200, "creating new members", creation_response
         )
@@ -658,7 +679,7 @@ class Spaces(WMLResource):
 
         Spaces._validate_type(member_id, "member_id", str, True)
 
-        href = self._client.service_instance._href_definitions.get_platform_spaces_member_href(
+        href = self._client._href_definitions.get_platform_spaces_member_href(
             space_id, member_id
         )
 
@@ -686,8 +707,10 @@ class Spaces(WMLResource):
         Spaces._validate_type(space_id, "space_id", str, True)
         Spaces._validate_type(member_id, "member_id", str, True)
 
-        member_endpoint = self._client.service_instance._href_definitions.get_platform_spaces_member_href(
-            space_id, member_id
+        member_endpoint = (
+            self._client._href_definitions.get_platform_spaces_member_href(
+                space_id, member_id
+            )
         )
 
         response_delete = requests.delete(
@@ -757,7 +780,7 @@ class Spaces(WMLResource):
         if state_str:
             patch_payload.append(state_str)
 
-        href = self._client.service_instance._href_definitions.get_platform_spaces_member_href(
+        href = self._client._href_definitions.get_platform_spaces_member_href(
             space_id, member_id
         )
 
@@ -815,9 +838,7 @@ class Spaces(WMLResource):
         if state is not None:
             params.update({"state": state})
 
-        href = self._client.service_instance._href_definitions.get_platform_spaces_members_href(
-            space_id
-        )
+        href = self._client._href_definitions.get_platform_spaces_members_href(space_id)
 
         member_resources = self._get_resources(href, "space members", params)[
             "resources"
@@ -878,9 +899,7 @@ class Spaces(WMLResource):
         if rev_id:
             promote_payload["revisionId"] = rev_id
 
-        promote_href = (
-            self._client.service_instance._href_definitions.promote_asset_href(asset_id)
-        )
+        promote_href = self._client._href_definitions.promote_asset_href(asset_id)
         response = requests.post(
             promote_href, headers=self._client._get_headers(), json=promote_payload
         )
