@@ -9,6 +9,7 @@ from typing import Any, TYPE_CHECKING, TypeAlias, Iterator, Literal, Callable
 import json
 import logging
 import time
+from warnings import warn
 
 from ibm_boto3.exceptions import Boto3Error
 from lomond import WebSocket
@@ -22,7 +23,7 @@ from ibm_watsonx_ai.utils import (
     TRAINING_RUN_DETAILS_TYPE,
     StatusLogger,
 )
-from ibm_watsonx_ai.utils.utils import _get_id_from_deprecated_uid
+from ibm_watsonx_ai.utils.utils import _get_id_from_deprecated_uid, _handle_fl_removal
 from ibm_watsonx_ai.wml_client_error import WMLClientError, ApiRequestFailure
 from ibm_watsonx_ai.wml_resource import WMLResource
 
@@ -146,11 +147,9 @@ class Training(WMLResource):
         Training._validate_type(training_id, "training_id", str, False)
 
         if _is_fine_tuning:
-            url = (
-                self._client.service_instance._href_definitions.get_fine_tunings_href()
-            )
+            url = self._client._href_definitions.get_fine_tunings_href()
         else:
-            url = self._client.service_instance._href_definitions.get_trainings_href()
+            url = self._client._href_definitions.get_trainings_href()
 
         if training_id is None:
             query_params: dict | None = {
@@ -447,17 +446,15 @@ class Training(WMLResource):
                 )
 
         if self.ConfigurationMetaNames.FEDERATED_LEARNING in meta_props:
+            _handle_fl_removal(self._client)
+
             training_configuration_metadata["federated_learning"] = meta_props[
                 self.ConfigurationMetaNames.FEDERATED_LEARNING
             ]
         if _is_fine_tuning:
-            train_endpoint = (
-                self._client.service_instance._href_definitions.get_fine_tunings_href()
-            )
+            train_endpoint = self._client._href_definitions.get_fine_tunings_href()
         else:
-            train_endpoint = (
-                self._client.service_instance._href_definitions.get_trainings_href()
-            )
+            train_endpoint = self._client._href_definitions.get_trainings_href()
 
         params = self._client._params()
         if "space_id" in params.keys():
@@ -752,16 +749,12 @@ class Training(WMLResource):
 
         if _is_fine_tuning:
 
-            train_endpoint = (
-                self._client.service_instance._href_definitions.get_fine_tuning_href(
-                    training_id
-                )
+            train_endpoint = self._client._href_definitions.get_fine_tuning_href(
+                training_id
             )
         else:
-            train_endpoint = (
-                self._client.service_instance._href_definitions.get_training_href(
-                    training_id
-                )
+            train_endpoint = self._client._href_definitions.get_training_href(
+                training_id
             )
 
         response_delete = requests.delete(
