@@ -4,6 +4,7 @@
 #  -----------------------------------------------------------------------------------------
 
 from ibm_watsonx_ai.utils.auth.base_auth import TokenAuth, get_auth_method
-from ibm_watsonx_ai.utils.auth.iam_auth import *
-from ibm_watsonx_ai.utils.auth.icp_auth import *
-from ibm_watsonx_ai.utils.auth.jwt_token_function_auth import *
+from ibm_watsonx_ai.utils.auth.iam_auth import IAMTokenAuth, get_iam_user_details
+from ibm_watsonx_ai.utils.auth.icp_auth import ICPAuth
+from ibm_watsonx_ai.utils.auth.jwt_token_function_auth import JWTTokenFunctionAuth
+from ibm_watsonx_ai.utils.auth.trusted_profile_auth import TrustedProfileAuth
