@@ -3,11 +3,17 @@
 #  https://opensource.org/licenses/BSD-3-Clause
 #  -----------------------------------------------------------------------------------------
 from __future__ import annotations
-from datetime import datetime, timedelta
-from typing import TYPE_CHECKING, Callable
+from typing import TYPE_CHECKING, Callable, Any
+import json
 
+from ibm_watsonx_ai.utils.utils import _requests_retry_session
 from ibm_watsonx_ai.wml_client_error import WMLClientError
-from ibm_watsonx_ai.utils.auth.base_auth import RefreshableTokenAuth, TokenInfo
+from ibm_watsonx_ai.utils.auth.base_auth import (
+    RefreshableTokenAuth,
+    TokenInfo,
+    STATUS_FORCELIST,
+    _get_token_info,
+)
 
 if TYPE_CHECKING:
     from ibm_watsonx_ai import APIClient
@@ -33,7 +39,7 @@ class IAMTokenAuth(RefreshableTokenAuth):
         on_token_refresh: Callable[[], None] | None = None,
     ) -> None:
         RefreshableTokenAuth.__init__(
-            self, api_client, on_token_creation, on_token_refresh, timedelta(minutes=15)
+            self, api_client, on_token_creation, on_token_refresh
         )
 
         if not api_client._is_IAM():
@@ -41,17 +47,6 @@ class IAMTokenAuth(RefreshableTokenAuth):
                 "api_key for IAM token is not provided in credentials for the client."
             )
 
-        self._save_token_data(self._generate_token())
-        if self._on_token_creation:
-            self._on_token_creation()
-
-        # update of minimal expiration datetime based on token expiration datetime
-        delta = self._get_expiration_datetime() - datetime.now()
-        if delta < self._expiration_timedelta:
-            self._expiration_timedelta = (
-                delta - timedelta(minutes=1) if delta > timedelta(minutes=1) else delta
-            )
-
     def _generate_token(self) -> TokenInfo:
         """Generate token using IAM authentication.
 
@@ -63,11 +58,33 @@ class IAMTokenAuth(RefreshableTokenAuth):
             "Authorization": "Basic Yng6Yng=",
         }
 
-        mystr = "apikey=" + self._href_definitions.get_iam_token_api()
-        response = self._session.post(
+        mystr = "apikey=" + self._href_definitions.get_iam_token_api(
+            self._credentials.api_key
+        )
+        response = _requests_retry_session(status_forcelist=STATUS_FORCELIST).post(
             self._href_definitions.get_iam_token_url(), data=mystr, headers=headers
         )
+
         if response.status_code == 200:
             return TokenInfo(response.json().get("access_token"))
         else:
             raise WMLClientError("Error getting IAM Token.", response)
+
+
+def get_iam_user_details(token: str) -> dict[str, Any]:
+    """Get IAM user details from token.
+
+    :param token: IAM token
+    :type token: str
+
+    :returns: IAM user details
+    :rtype: dict[str, Any]
+
+    :raises: WMLClientError if there is an error getting IAM user details
+    """
+    try:
+        token_info = _get_token_info(token)
+    except json.JSONDecodeError as e:
+        raise WMLClientError("Error getting IAM user details.") from e
+
+    return token_info
