@@ -12,7 +12,7 @@ __all__ = [
     "UseWMLClient",
     "PipelineNotLoaded",
     "MissingCOSStudioConnection",
-    "MissingProjectLib",
+    "MissingIBMWatsonStudioLib",
     "LocalInstanceButRemoteParameter",
     "DataFormatNotSupported",
     "HoldoutSplitNotSupported",
@@ -70,6 +70,9 @@ __all__ = [
     "InvalidLocationInDataConnection",
     "DirectoryHasNoFilename",
     "CannotGetFilename",
+    "TokenRemovedDuringClientCopy",
+    "FolderDownloadNotSupported",
+    "NoDocumentsLoaded",
 ]
 
 
@@ -131,13 +134,13 @@ class MissingCOSStudioConnection(WMLClientError, ValueError):
         WMLClientError.__init__(self, f"Missing COS Studio connection.", reason)
 
 
-class MissingProjectLib(WMLClientError, ValueError):
+class MissingIBMWatsonStudioLib(WMLClientError, ValueError):
     def __init__(self, reason=None):
         WMLClientError.__init__(
             self,
-            f"project-lib package missing in the environment, please make sure you are on Watson Studio"
-            f" and want to automatically initialize your COS connection."
-            f" If you want to initialize COS connection manually, do not use from_studio() method.",
+            "ibm-watson-studio-lib package missing in the environment, please make sure you are on Watson Studio "
+            "and want to automatically initialize your COS connection. "
+            "If you want to initialize COS connection manually, do not use from_studio() method.",
             reason,
         )
 
@@ -604,3 +607,30 @@ class CannotGetFilename(WMLClientError, ValueError):
         WMLClientError.__init__(
             self, f"Unsupported connection type for extracting file name."
         )
+
+
+class TokenRemovedDuringClientCopy(WMLClientError):
+    def __init__(self):
+        WMLClientError.__init__(
+            self,
+            f"APIClient had authentication data removed during `APIClient.get_copy()` operation."
+            f"Use `APIClient.set_token(token)` to setup authentication in client copy.",
+        )
+
+
+class FolderDownloadNotSupported(WMLClientError):
+    def __init__(self):
+        WMLClientError.__init__(
+            self,
+            "Folder download is not supported. Only connections for separate files are allowed.",
+        )
+
+
+class NoDocumentsLoaded(WMLClientError):
+    def __init__(self, class_name):
+        WMLClientError.__init__(
+            self,
+            f"During documents loading no documents where loaded. "
+            f"Use `error_callback` parameter of `{class_name}` class to check the exceptions "
+            f"which were raised during documents loading.",
+        )
