@@ -4,23 +4,24 @@
 #  -----------------------------------------------------------------------------------------
 
 from __future__ import annotations
-import re
-import os
-import sys
-import shutil
-import tarfile
-import logging
-import importlib.util
-import json
-import platform
-from enum import Enum
-import inspect
 
-import numpy
 import importlib
-import base64
+import functools
+import importlib.util
+import mimetypes
+import inspect
+import json
+import logging
+import os
+import platform
+import re
+import shutil
+import sys
+import tarfile
+from dataclasses import dataclass, field
 from datetime import datetime
-
+from enum import Enum
+from subprocess import check_call
 from typing import (
     TYPE_CHECKING,
     Any,
@@ -31,13 +32,17 @@ from typing import (
     TypeAlias,
     cast,
 )
-from subprocess import check_call
-from packaging import version
 from warnings import warn
+from pathlib import Path
+
+import httpx
+import numpy
+from packaging import version
 
-from ibm_watsonx_ai import ver, pkg_name
 import ibm_watsonx_ai._wrappers.requests as requests
-
+from ibm_watsonx_ai import package_name, __version__ as package_version
+from ibm_watsonx_ai._wrappers.requests import HTTPX_DEFAULT_TIMEOUT, HTTPX_DEFAULT_LIMIT
+from ibm_watsonx_ai.href_definitions import HrefDefinitions
 from ibm_watsonx_ai.wml_client_error import (
     WMLClientError,
     CannotInstallLibrary,
@@ -114,6 +119,29 @@ def get_url(
         return requests.get(url, headers=headers, params=params)
 
 
+@dataclass
+class HttpClientConfig:
+    """
+    A class for storing parameters used to initialize an `httpx.Client`.
+    Using this class is recommended when adjusting timeouts or limits instead of providing a separate `httpx.Client`.
+    :param timeout: The timeout configuration for sending requests.
+    :type timeout: httpx.Timeout, optional
+
+    :param limits: The limits configuration to control the connection pool size.
+    :type limits: httpx.Limits, optional
+    """
+
+    timeout: float | httpx.Timeout = field(
+        default_factory=lambda: HTTPX_DEFAULT_TIMEOUT
+    )
+    limits: httpx.Limits = field(default_factory=lambda: HTTPX_DEFAULT_LIMIT)
+
+
+DEFAULT_HTTP_CLIENT_CONFIG = HttpClientConfig(
+    timeout=HTTPX_DEFAULT_TIMEOUT, limits=HTTPX_DEFAULT_LIMIT
+)
+
+
 def print_text_header_h1(title: str) -> None:
     print("\n\n" + ("#" * len(title)) + "\n")
     print(title)
@@ -680,6 +708,8 @@ def next_resource_generator(
     params: dict | None = None,
     _all: bool = False,
     _filter_func: Callable | None = None,
+    use_httpx: bool = False,
+    _silent_response_logging: bool = False,
 ) -> Generator[dict, None, None]:
     """
     Generator to produce next list of resources from REST API.
@@ -701,19 +731,27 @@ def next_resource_generator(
 
     :param _filter_func: filtering function
     :type _filter_func: function, optional
+
+    :param use_httpx: use httpx.Client
+    :type use_httpx: bool, optional
     """
     next_href: str | None = href
 
+    request_handler = client.httpx_client if use_httpx else requests
+
     while next_href is not None:
         if "http" not in next_href:
             next_href = f"{url}/{next_href}"
-        response = requests.get(
+        response = request_handler.get(
             url=next_href,
             headers=client._get_headers(),
             params=(params if params is not None else client._params()),
         )
         details_json = client.training._handle_response(
-            200, "Get next details", response
+            200,
+            "Get next details",
+            response,
+            _silent_response_logging=_silent_response_logging,
         )
 
         if _all:
@@ -725,6 +763,9 @@ def next_resource_generator(
         if "resources" in details_json:
             resources = details_json["resources"]
 
+            if not resources:
+                next_href = None
+
         elif "metadata" in details_json:
             resources = [details_json]
 
@@ -871,19 +912,18 @@ def prepare_interaction_props_for_cos(source_params: dict, file_name: str) -> di
             interaction_properties["sheet_name"] = str(source_params.get("excel_sheet"))
 
     elif ".csv" in file_name:
-        if source_params.get("quote_character"):
-            interaction_properties["quote_character"] = str(
-                source_params.get("quote_character")
-            )
         if encoding is not None:
             interaction_properties["encoding"] = encoding
 
         input_file_separator = source_params.get("input_file_separator", ",")
-        if input_file_separator != ",":
+        if input_file_separator == ",":
+            file_format = "csv"
+        else:
             file_format = "delimited"
             interaction_properties["field_delimiter"] = input_file_separator
-        else:
-            file_format = "csv"
+
+            if quote_character := source_params.get("quote_character"):
+                interaction_properties["quote_character"] = str(quote_character)
 
     elif ".parquet" in file_name or ".prq" in file_name:
         file_format = "parquet"
@@ -960,8 +1000,6 @@ def get_user_agent_header() -> str:
     """
     Function which return User-Agent header
     """
-    package_name = pkg_name
-    package_version = ver
     lang = "python"
     try:
         arch = os.uname().machine
@@ -982,23 +1020,13 @@ def get_user_agent_header() -> str:
 
 
 def _get_expiration_datetime_from_headers(headers: dict) -> datetime | None:
-
     try:
+        from ibm_watsonx_ai.utils.auth.base_auth import _get_token_info
+
         token = headers.get("Authorization", " ").split(" ")[-1]
 
-        token_parts = token.split(".")
-        token_padded = token_parts[1] + "==="
-        try:
-            token_info = json.loads(
-                base64.b64decode(token_padded).decode("utf-8", errors="ignore")
-            )
-        except ValueError:
-            # If there is a problem with decoding (e.g. special char in token), add altchars
-            token_info = json.loads(
-                base64.b64decode(token_padded, altchars="_-").decode(
-                    "utf-8", errors="ignore"
-                )
-            )
+        token_info = _get_token_info(token)
+
         token_expire = token_info.get("exp")
 
         return datetime.fromtimestamp(token_expire)
@@ -1061,24 +1089,77 @@ def _get_default_args(func: Callable) -> dict[str, Any]:
     }
 
 
-def _is_iam_token(token: str) -> str | bool:
-    """Check if passed token is IAM."""
-    try:
-        token_parts = token.split(".")
-        token_padded = token_parts[1] + "==="
-        try:
-            token_info = json.loads(
-                base64.b64decode(token_padded).decode("utf-8", errors="ignore")
-            )
-        except ValueError:
-            # If there is a problem with decoding (e.g. special char in token), add altchars
-            token_info = json.loads(
-                base64.b64decode(token_padded, altchars="_-").decode(
-                    "utf-8", errors="ignore"
-                )
-            )
-        instanceId = token_info.get("instanceId")
+def _create_href_definitions(client: APIClient) -> HrefDefinitions:
+    return HrefDefinitions(
+        url=client.credentials.url,
+        instance_id=client.credentials.instance_id,
+        version=client.credentials.version,
+        bedrock_url=client.credentials.bedrock_url,
+        cloud_platform_spaces=client.CLOUD_PLATFORM_SPACES,
+        cp4d_platform_spaces=client.ICP_PLATFORM_SPACES,
+        platform_url=client.PLATFORM_URL,
+        project_type=client.project_type,
+        _use_fm_ga_api=client._use_fm_ga_api,
+    )
 
-        return instanceId
-    except:
-        return False
+
+def _handle_fl_removal(client: APIClient):
+    if client.ICP_PLATFORM_SPACES and client.CPD_version >= 5.2:
+        raise WMLClientError(
+            "Federated Learning is removed in IBM Cloud Pak for Data 5.2 and above."
+        )
+    else:
+        FL_DEPRECATED_WARNING = "Federated Learning is deprecated and will be removed in IBM Cloud Pak for Data 5.2."
+
+        warn(FL_DEPRECATED_WARNING, category=DeprecationWarning)
+
+
+def _requests_retry_session(
+    retries: int = 3,
+    backoff_factor: float = 0.3,
+    status_forcelist: Iterable[int] = (500, 502, 503, 504, 520, 521, 524),
+    session: requests.Session | None = None,
+) -> requests.Session:
+    from requests.adapters import HTTPAdapter
+    from requests.packages.urllib3.util.retry import Retry
+
+    session = session or requests.Session()
+    retry = Retry(
+        total=retries,
+        read=retries,
+        connect=retries,
+        backoff_factor=backoff_factor,
+        status_forcelist=status_forcelist,
+        raise_on_status=False,
+    )
+    adapter = HTTPAdapter(max_retries=retry)
+    session.mount("http://", adapter)
+    session.mount("https://", adapter)
+    return session
+
+
+def raise_exception_about_unsupported_on_cloud(func: Callable) -> Callable:
+    from ibm_watsonx_ai.wml_resource import WMLResource
+
+    @functools.wraps(func)
+    def wrapper(resource: WMLResource, *args: Any, **kwargs: Any) -> Any:
+        if resource._client.CLOUD_PLATFORM_SPACES:
+            raise WMLClientError(
+                error_msg=f"{resource.__class__} is not supported on IBM watsonx.ai for IBM Cloud!"
+            )
+        return func(resource, *args, **kwargs)
+
+    return wrapper
+
+
+def content_type_for(filepath: str, default: str = "application/octet-stream") -> str:
+    """
+    Return the best‐guess Content-Type for a file path, falling back to `default` if unknown.
+    """
+    # 1) Make sure .yaml/.yml map to text/yaml
+    mimetypes.add_type("text/yaml", ".yaml")
+    mimetypes.add_type("text/yaml", ".yml")
+
+    ext = Path(filepath).suffix
+    mime = mimetypes.types_map.get(ext.lower())
+    return mime or default
