@@ -25,6 +25,7 @@ from ibm_watsonx_ai.utils import (
     get_type_of_details,
     next_resource_generator,
 )
+from ibm_watsonx_ai.utils.utils import _get_id_from_deprecated_uid
 from ibm_watsonx_ai.wml_client_error import (
     MissingValue,
     WMLClientError,
@@ -64,10 +65,11 @@ class WMLResource:
         if client.credentials is None:
             raise NoWMLCredentialsProvided
         WMLResource._validate_type(client.credentials, "credentials", Credentials, True)
-        if not client.ICP_PLATFORM_SPACES:
-            WMLResource._validate_type(client.token, "token", str, True)
         self._client = client
-        self._credentials = client.service_instance._credentials
+
+    @property
+    def _credentials(self):
+        return self._client.credentials
 
     @overload
     def _handle_response(
@@ -207,9 +209,7 @@ class WMLResource:
 
         if asset_id:
             response = requests.get(
-                self._client.service_instance._href_definitions.get_data_asset_href(
-                    asset_id
-                ),
+                self._client._href_definitions.get_asset_href(asset_id),
                 params=self._client._params(),
                 headers=self._client._get_headers(),
             )
@@ -219,11 +219,7 @@ class WMLResource:
             )
 
         else:
-            href = (
-                self._client.service_instance._href_definitions.get_asset_search_href(
-                    asset_type
-                )
-            )
+            href = self._client._href_definitions.get_asset_search_href(asset_type)
 
             def get_chunk(data):
                 response = requests.post(
@@ -254,6 +250,68 @@ class WMLResource:
 
             return {"resources": result if limit is None else result[:limit]}
 
+    def _list_asset_based_resource(
+        self, url: str, column_names: list[str], limit: int | None = None
+    ) -> pd.DataFrame:
+        """Lists stored assets in a table format."""
+
+        self._validate_type(url, "url", str, True)
+        self._validate_type(column_names, "column_names", list, True)
+        self._validate_type(limit, "limit", int, False)
+
+        data: dict[str, Any] = {"query": "*:*"}
+        if limit is not None:
+            data.update({"limit": limit})
+
+        response = requests.post(
+            url,
+            params=self._client._params(),
+            headers=self._client._get_headers(),
+            json=data,
+        )
+        self._handle_response(200, "list assets", response)
+        asset_details = self._handle_response(200, "list assets", response)["results"]
+        space_values = [
+            [m["metadata"][col.lower()] for col in column_names] for m in asset_details
+        ]
+
+        table = self._list(
+            space_values,
+            column_names,
+            limit,
+        )
+        return table
+
+    def _delete_asset_based_resource(
+        self,
+        asset_id: str,
+        get_required_element_from_response: Callable,
+        purge_on_delete: bool | None = None,
+        **kwargs: Any,
+    ) -> dict | str:
+        """Soft delete the stored asset. The asset will be moved to trashed assets
+        and will not be visible in asset list. To permanently delete assets set `purge_on_delete` parameter to True.
+        """
+
+        asset_id = _get_id_from_deprecated_uid(kwargs, asset_id, "asset")
+
+        self._validate_type(asset_id, "asset_id", str, True)
+
+        params = self._client._params()
+
+        if purge_on_delete:
+            params["purge_on_delete"] = True
+
+        response = requests.delete(
+            self._client._href_definitions.get_asset_href(asset_id),
+            params=params,
+            headers=self._client._get_headers(),
+        )
+        if response.status_code == 200:
+            return get_required_element_from_response(response.json())
+        else:
+            return self._handle_response(204, "delete assets", response)
+
     @staticmethod
     def _validate_type(
         el: Any,
@@ -419,6 +477,7 @@ class WMLResource:
         _async: Literal[False] = ...,
         _all: bool = False,
         _filter_func: Callable | None = None,
+        _silent_response_logging: bool = False,
     ) -> dict: ...
 
     @overload
@@ -435,6 +494,7 @@ class WMLResource:
         _async: Literal[True] = ...,
         _all: bool = False,
         _filter_func: Callable | None = None,
+        _silent_response_logging: bool = False,
     ) -> Generator: ...
 
     @overload
@@ -451,6 +511,7 @@ class WMLResource:
         _async: bool = False,
         _all: bool = False,
         _filter_func: Callable | None = None,
+        _silent_response_logging: bool = False,
     ) -> ArtifactDetailsType: ...
 
     def _get_with_or_without_limit(
@@ -466,6 +527,7 @@ class WMLResource:
         _async: bool = False,
         _all: bool = False,
         _filter_func: Callable | None = None,
+        _silent_response_logging: bool = False,
     ) -> ArtifactDetailsType:
         params = self._client._params(skip_space_project_chk)
 
@@ -512,7 +574,14 @@ class WMLResource:
         url_2 = "/".join(url.split("/")[:3])
 
         resource_generator = next_resource_generator(
-            self._client, url_2, href, params, _all, _filter_func, use_httpx
+            self._client,
+            url_2,
+            href,
+            params,
+            _all,
+            _filter_func,
+            use_httpx,
+            _silent_response_logging=_silent_response_logging,
         )
 
         if _async:
@@ -531,7 +600,15 @@ class WMLResource:
                     params=params,
                 )
 
-                result = cast(dict, self._handle_response(200, op_name, response_get))
+                result = cast(
+                    dict,
+                    self._handle_response(
+                        200,
+                        op_name,
+                        response_get,
+                        _silent_response_logging=_silent_response_logging,
+                    ),
+                )
                 if "resources" in result:
                     resources.extend(result["resources"])
 
@@ -548,7 +625,7 @@ class WMLResource:
     def _if_deployment_exist_for_asset(self, asset_id: str) -> bool:
 
         deployment_href = (
-            self._client.service_instance._href_definitions.get_deployments_href()
+            self._client._href_definitions.get_deployments_href()
             + "?asset_id="
             + asset_id
         )
@@ -616,10 +693,7 @@ class WMLResource:
     ) -> dict | str:
         op_name = "Creation revision for {}".format(resource_name)
 
-        url = (
-            self._client.service_instance._href_definitions.get_asset_href(id)
-            + "/revisions"
-        )
+        url = self._client._href_definitions.get_asset_href(id) + "/revisions"
         commit_message = "Revision creation for " + resource_name + " " + id
 
         payload_json = {"commit_message": commit_message}
@@ -653,7 +727,7 @@ class WMLResource:
         if current_attachment_id is not None:
             # Delete existing attachment to upload new attachment
             attachments_id_url = (
-                self._client.service_instance._href_definitions.get_asset_href(asset_id)
+                self._client._href_definitions.get_asset_href(asset_id)
                 + "/attachments/"
                 + current_attachment_id
             )
@@ -674,8 +748,7 @@ class WMLResource:
             }
 
             attachments_url = (
-                self._client.service_instance._href_definitions.get_asset_href(asset_id)
-                + "/attachments"
+                self._client._href_definitions.get_asset_href(asset_id) + "/attachments"
             )
 
             # STEP 3b.
@@ -718,7 +791,7 @@ class WMLResource:
                     # STEP 3d.
                     # Mark attachment complete
                     complete_response = requests.post(
-                        self._client.service_instance._href_definitions.get_attachment_complete_href(
+                        self._client._href_definitions.get_attachment_complete_href(
                             asset_id, attachment_id
                         ),
                         headers=self._client._get_headers(),
